@@ -111,12 +111,10 @@
     (unsigned 64) (context (* os-context-t)))
 
 
-;;;; INTERNAL-ERROR-ARGS
 (defun internal-error-args (context)
   (declare (type (alien (* os-context-t)) context))
   (let* ((pc (context-pc context))
-         (error-number (sap-ref-8 pc 4)))
+         (trap-number (sap-ref-8 pc 3)))
     (declare (type system-area-pointer pc))
-    (values error-number
-            (sb!kernel::decode-internal-error-args (sap+ pc 5) error-number))))
+    (sb!kernel::decode-internal-error-args (sap+ pc 4) trap-number)))
 ) ; end PROGN
