@@ -45,8 +45,7 @@
 (defun internal-error-args (context)
   (declare (type (alien (* os-context-t)) context))
   (let* ((pc (context-pc context))
-         (error-number (sap-ref-8 pc 5)))
+         (trap-number (sap-ref-8 pc 4)))
     (declare (type system-area-pointer pc))
-    (values error-number
-            (sb!kernel::decode-internal-error-args (sap+ pc 6) error-number))))
+    (sb!kernel::decode-internal-error-args (sap+ pc 5) trap-number)))
 ) ; end PROGN
