@@ -11,12 +11,12 @@
 
 (in-package "SB!IMPL")
 
-(/show0 "entering backq.lisp")
-
 ;; An unquoting COMMA struct.
 (defstruct (comma (:constructor unquote (expr &optional (kind 0)))
+                  #+sb-xc-host (:include structure!object)
                   ;; READing unpretty commas requires a default constructor.
-                  (:constructor %default-comma-constructor)
+                  ;; Not needed on the host.
+                  #-sb-xc-host (:constructor %default-comma-constructor)
                   (:copier nil))
  (expr nil :read-only t)
  (kind nil :read-only t :type (member 0 1 2)))
@@ -32,22 +32,12 @@
 (defun comma-constructor (x)
   (svref #(unquote unquote-nsplice unquote-splice) (comma-kind x)))
 (defun comma-splicing-p (comma) (not (zerop (comma-kind comma))))
-
-#+sb-xc-host
-(progn
-  ;; tell the host how to dump it
-  (defmethod make-load-form ((self comma) &optional environment)
-    (declare (ignore environment))
-    (list (comma-constructor self) (list 'quote (comma-expr self))))
-  ;; tell the cross-compiler that it can do :just-dump-it-normally
-  (setf (get 'comma :sb-xc-allow-dumping-instances) t))
+(!set-load-form-method comma (:host :xc :target))
 
 (declaim (type (and fixnum unsigned-byte) *backquote-depth*))
-(defvar *backquote-depth* 0 #!+sb-doc "how deep we are into backquotes")
+(defvar *backquote-depth* 0 "how deep we are into backquotes")
 (defvar *bq-error* "Comma not inside a backquote.")
 
-(/show0 "backq.lisp 50")
-
 ;;; the actual character macro
 (defun backquote-charmacro (stream char)
   (declare (ignore char))
@@ -60,8 +50,6 @@
          stream "~S is not a well-formed backquote expression" result)
         result)))
 
-(/show0 "backq.lisp 64")
-
 (defun comma-charmacro (stream char)
   (declare (ignore char))
   (declare (notinline read-char unread-char))
@@ -82,8 +70,7 @@
     (unquote (let ((*backquote-depth* (1- *backquote-depth*)))
                (read stream t nil t)) flag)))
 
-(/show0 "backq.lisp 83")
-
+;; KLUDGE: 'sfunction' is not a defined type yet.
 (declaim (ftype (function (t fixnum boolean) (values t t &optional))
                 qq-template-to-sexpr qq-template-1))
 
@@ -145,8 +132,6 @@
                          subexpr)
                    operator)))))
 
-(/show0 "backq.lisp 139")
-
 ;; Find the longest suffix comprised wholly of self-evaluating and/or quoted
 ;; SUBFORMS. DOTTED-P indicates that the last item represents what was in the
 ;; CDR of the last cons of the original list. Return the modified SUBFORMS
@@ -340,5 +325,3 @@
 #+sb-xc-host ; proper definition happens for the target
 (defun simple-reader-error (stream format-string &rest format-args)
   (error "READER-ERROR on stream ~S: ~?" stream format-string format-args))
-
-(/show0 "done with backq.lisp")
