@@ -109,6 +109,15 @@
 ;;; This is a little ugly.  Fixing bug 188 would bring the ability to
 ;;; wrap a MACROLET or something similar around this whole thing would
 ;;; make things significantly less ugly.  --njf, 2005-02-23
+;;;
+;;; Well, it turns out that we *could* wrap a MACROLET around this,
+;;; but it's quite ugly in a different way: when you write
+;;;  (MACROLET ((GENERATOR () `(DEFUN F (X) ,@(INSANITY ...))) (GENERATOR)))
+;;; and then view the inline expansion of F, you'll see it has actually
+;;; captured the entirety of the MACROLET that surrounded it.
+;;; With respect to building SBCL, this means, among other things, that
+;;; it'd hang onto all the "!" symbols that would otherwise disappear,
+;;; as well as kilobytes of completely useless s-expressions.
 (eval-when (:compile-toplevel :load-toplevel :execute)
 
 ;;; Align the SAP to a word boundary, and update the offset accordingly.
@@ -145,20 +154,7 @@
   (let* ((bytes-per-word (/ n-word-bits bitsize))
          (byte-offset `(integer 0 (,bytes-per-word)))
          (byte-count `(integer 1 (,bytes-per-word)))
-         (max-bytes (ash sb!xc:most-positive-fixnum
-                         ;; FIXME: this reflects code contained in the
-                         ;; original bit-bash.lisp, but seems very
-                         ;; nonsensical.  Why shouldn't we be able to
-                         ;; handle M-P-FIXNUM bits?  And if we can't,
-                         ;; are these other shift amounts bogus, too?
-                         (ecase bitsize
-                           (1 -2)
-                           (2 -1)
-                           (4  0)
-                           (8  0)
-                           (16 0)
-                           (32 0)
-                           (64 0))))
+         (max-bytes sb!xc:most-positive-fixnum)
          (offset `(integer 0 ,max-bytes))
          (max-word-offset (ceiling max-bytes bytes-per-word))
          (word-offset `(integer 0 ,max-word-offset))
@@ -571,6 +567,8 @@
                                  #'word-sap-ref)))))))
 ) ; EVAL-WHEN
 
+(eval-when (:compile-toplevel)
+  (sb!xc:proclaim '(muffle-conditions compiler-note)))
 ;;; We would normally do this with a MACROLET, but then we run into
 ;;; problems with the lexical environment being too hairy for the
 ;;; cross-compiler and it cannot inline the basic basher functions.
@@ -610,6 +608,24 @@
                          (index start end)
                          (optimize (speed 3) (safety 0)))
                 (unless (= start end)
+                  ;; We use the traditional approach of scanning:
+                  ;;  (1) some number of bits (<= n-word-bits) initially,
+                  ;;  (2) then a whole number of intervening words,
+                  ;;  (3) then some number of trailing bits.
+                  ;; We combine two of the three scans, depending on FROM-END.
+                  ;; If FROM-END is true, we do (3) as one step, then (1) and (2).
+                  ;; If FROM-END is false, we do (1) as one step, then (2) and (3).
+                  ;; So first we calculate FIRST-BITS and LAST-BITS as the number
+                  ;; of "boundary" bits on each end. These values will never
+                  ;; be exactly N-WORD-BITS, though logically they should allow that.
+                  ;; Instead the convention is slightly odd. It does exactly the
+                  ;; right thing for START-MASK: compute a bit mask having anywhere
+                  ;; between 1 and N-WORD-BITS (inclusive) consecutive 1s, starting
+                  ;; from the appropriate end. END-MASK is weird: instead of getting
+                  ;; all 1s in the limiting case, it gets all 0s, and a LAST-WORD
+                  ;; value that is 1 too high.  Perhaps somebody smarter than I could
+                  ;; do this "correctly" with FLOOR and CEILING, but this code
+                  ;; compiles very efficiently and I don't want to disturb it much.
                   (let* ((last-word (ash end (- +bit-position-base-shift+)))
                          (last-bits (logand end +bit-position-base-mask+))
                          (first-word (ash start (- +bit-position-base-shift+)))
@@ -623,6 +639,10 @@
                     (flet ((#!+little-endian start-bit
                             #!+big-endian end-bit (x)
                              (declare (word x))
+                             #!+(or x86-64 x86)
+                             (truly-the (mod #.n-word-bits)
+                                 (%primitive unsigned-word-find-first-bit x))
+                             #!-(or x86-64 x86)
                              (- #!+big-endian sb!vm:n-word-bits
                                 (integer-length (logand x (- x)))
                                 #!+little-endian 1))
@@ -640,11 +660,24 @@
                                           (ash word-offset +bit-position-base-shift+)))))
                            (get-word (offset)
                              (,@frob (%vector-raw-bits vector offset))))
-                      (declare (inline start-bit end-bit get-word))
+                      ;; Inline FOUND makes the code smaller, as well as faster,
+                      ;; because it becomes clear that there is no establishing
+                      ;; of an exit point and hence no call to UNWIND.
+                      (declare (inline start-bit end-bit found get-word))
                       (if from-end
                           ;; Back to front
+                          ;; When the upper bound aligns exactly to a word, END-MASK
+                          ;; is 0, and LAST-WORD is 1 higher than the number of words
+                          ;; to scan. Reading it could overrun the vector's bound,
+                          ;; so detect and avoid that case.
+                          ;; This is actually simpler and clearer than adjusting
+                          ;; the loop indices, because the remainder of the loop
+                          ;; is right in the backwards scan, and the termination
+                          ;; condition is always FIRST-WORD inclusive.
                           (let* ((word-offset last-word)
-                                 (word (logand end-mask (get-word word-offset))))
+                                 (word (if (eql end-mask 0)
+                                           0
+                                           (logand end-mask (get-word word-offset)))))
                             (declare (word word)
                                      (index word-offset))
                             (unless (zerop word)
@@ -674,6 +707,15 @@
                               (unless (zerop word)
                                 (found (start-bit word) word-offset)))
                             (incf word-offset)
+                            ;; Boundary condition: if END-MASK is zero, then LAST-WORD
+                            ;; is an exclusive bound on word index to read. Otherwise,
+                            ;; when nonzero, LAST-WORD is the *inclusive* bound.
+                            ;; The loop works correctly with this assignment to END-MASK,
+                            ;; and it doesn't upset the compiler in the least -
+                            ;; it still uses unboxed reads and compares throughout.
+                            (when (eql end-mask 0)
+                              (setq end-mask sb!ext:most-positive-word)
+                              (decf last-word)) ; make it an inclusive bound
                             (loop
                               (when (> word-offset last-word)
                                 (return-from ,name nil))
