@@ -26,6 +26,10 @@
       (/show0 "in TRANS-P case")
       (let ((classoid (classoid-cell-classoid (find-classoid-cell name :create t)))
             (type (specifier-type translation)))
+        ;; The classoid T gets its translation dumped in genesis.
+        ;; May as well double-check that it's right.
+        (when (typep (built-in-classoid-translation classoid) 'ctype)
+          (aver (eq (built-in-classoid-translation classoid) type)))
         (setf (built-in-classoid-translation classoid) type)
         (setf (info :type :builtin name) type)))))
 
