@@ -66,11 +66,8 @@ provide bindings for printer control variables.")
 ;;; It actually works as long as the condition is not a subtype of WARNING
 ;;; or ERROR. (Any other direct descendant of CONDITION should be fine)
 (!defvar *stack-top-hint* nil)
-
-(defvar *real-stack-top* nil)
-(defvar *stack-top* nil)
-
 (defvar *current-frame* nil)
+(declaim (always-bound *stack-top-hint* *current-frame*))
 
 ;;; Beginner-oriented help messages are important because you end up
 ;;; in the debugger whenever something bad happens, or if you try to
@@ -210,7 +207,7 @@ backtraces. Possible values are :MINIMAL, :NORMAL, and :FULL.
    In the this case arguments may include values internal to SBCL's method
    dispatch machinery.")
 
-(define-deprecated-variable :early "1.1.4.9" *show-entry-point-details*
+(define-deprecated-variable :late "1.1.4.9" *show-entry-point-details*
   :value nil)
 
 (define-deprecated-function :early "1.2.15" backtrace (print-backtrace)
@@ -1169,10 +1166,8 @@ and LDB (the low-level debugger).  See also ENABLE-DEBUGGER."
 
 (defun debug-loop-fun ()
   (let* ((*debug-command-level* (1+ *debug-command-level*))
-         (*real-stack-top* (sb!di:top-frame))
-         (*stack-top* (or *stack-top-hint* *real-stack-top*))
-         (*stack-top-hint* nil)
-         (*current-frame* *stack-top*))
+         (*current-frame* (or *stack-top-hint* (sb!di:top-frame)))
+         (*stack-top-hint* nil))
     (handler-bind ((sb!di:debug-condition
                     (lambda (condition)
                       (princ condition *debug-io*)
@@ -1899,12 +1894,12 @@ forms that explicitly control this kind of evaluation.")
 ;;;; debug loop command utilities
 
 (defun read-prompting-maybe (prompt)
-  (unless (sb!int:listen-skip-whitespace *debug-io*)
+  (unless (listen-skip-whitespace *debug-io*)
     (princ prompt *debug-io*)
     (force-output *debug-io*))
   (read *debug-io*))
 
 (defun read-if-available (default)
-  (if (sb!int:listen-skip-whitespace *debug-io*)
+  (if (listen-skip-whitespace *debug-io*)
       (read *debug-io*)
       default))
