@@ -68,12 +68,10 @@
   '(character-set ((0 . #.(1- base-char-code-limit)))))
 
 (sb!xc:deftype extended-char ()
-  #!+sb-doc
   "Type of CHARACTERs that aren't BASE-CHARs."
   '(and character (not base-char)))
 
 (sb!xc:deftype standard-char ()
-  #!+sb-doc
   "Type corresponding to the characters required by the standard."
   '(member
     #\Newline #\Space #\! #\" #\# #\$ #\% #\& #\' #\( #\) #\* #\+ #\,
@@ -135,6 +133,9 @@
 (sb!xc:deftype format-control ()
   '(or string function))
 
+(sb!xc:deftype condition-designator-head ()
+  '(or format-control symbol condition sb!pcl::condition-class))
+
 (sb!xc:deftype restart-designator ()
   '(or (and symbol (not null)) restart))
 
@@ -164,9 +165,11 @@
 
 ;;; legal args to pathname functions
 (sb!xc:deftype pathname-designator ()
-  '(or string pathname #+sb-xc-host stream #-sb-xc-host file-stream))
+  '(or string pathname #+sb-xc-host stream #-sb-xc-host synonym-stream #-sb-xc-host file-stream))
 (sb!xc:deftype logical-host-designator ()
   '(or host string))
+(sb!xc:deftype pathname-component-case ()
+  '(member :local :common))
 
 (sb!xc:deftype package-designator () '(or string-designator package))
 ;;; a designator for a list of symbols
