@@ -45,7 +45,8 @@
 ;;; in the *FEATURES* list of this particular build.
 (defglobal *features-potentially-affecting-fasl-format*
     (append '(:sb-thread :sb-package-locks :sb-unicode :cheneygc
-              :gencgc :msan :sb-safepoint :sb-safepoint-strictly)
+              :gencgc :msan :sb-safepoint :sb-safepoint-strictly
+              :sb-dynamic-core)
             #!+(or x86 x86-64) '(:int4-breakpoints :ud2-breakpoints)))
 
 ;;; Return a string representing symbols in *FEATURES-POTENTIALLY-AFFECTING-FASL-FORMAT*
