@@ -19,7 +19,12 @@ in future versions."
   (%alive-p      nil :type boolean)
   (%ephemeral-p  nil :type boolean)
   #-sb-xc-host
-  (os-thread     0 :type sb!vm:word)
+  ;; 0 is used on thread-less builds
+  (os-thread  (ldb (byte sb!vm:n-word-bits 0) -1) :type sb!vm:word)
+  #-sb-xc-host
+  ;; Points to the SB-VM::THREAD primitive object.
+  ;; Yes, there are three different thread structures.
+  (primitive-thread 0 :type sb!vm:word)
   (interruptions nil :type list)
   ;; On succesful execution of the thread's lambda a list of values.
   (result 0)
@@ -53,3 +58,14 @@ in future versions."
               'call-with-system-mutex))
        #'with-system-mutex-thunk
        ,mutex)))
+
+;; Similar to above. The host doesn't need this one at all.
+#-sb-xc-host
+(defmacro with-recursive-system-lock ((lock &key without-gcing) &body body)
+  `(dx-flet ((with-recursive-system-lock-thunk () ,@body))
+     (,(cond (without-gcing
+              'call-with-recursive-system-lock/without-gcing)
+             (t
+              'call-with-recursive-system-lock))
+      #'with-recursive-system-lock-thunk
+       ,lock)))
