@@ -18,7 +18,6 @@
 ;; but it's compiled after this file is.
 (!defglobal *user-hash-table-tests* nil)
 
-(sb!int::/show0 "fdefinition.lisp 22")
 
 ;;;; fdefinition (fdefn) objects
 
@@ -202,6 +201,10 @@
     (function callable)
     (symbol (%coerce-name-to-fun callable symbol-fdefn t))))
 
+;;; Bevahes just like %COERCE-CALLABLE-TO-FUN but has an ir2-convert optimizer.
+(%defun '%coerce-callable-for-call
+        #'%coerce-callable-to-fun)
+
 
 ;;;; definition encapsulation
 
