@@ -272,7 +272,7 @@ statistics are appended to it."
   ;; another GC. FIXME: it can potentially exceed maximum interrupt
   ;; nesting by triggering GCs.
   ;;
-  ;; Can that be avoided by having the finalizers and hooks run only
+  ;; Can that be avoided by having the hooks run only
   ;; from the outermost SUB-GC? If the nested GCs happen in interrupt
   ;; handlers that's not enough.
   ;;
@@ -285,13 +285,26 @@ statistics are appended to it."
   ;;    don't want to run user code in such a case.
   ;;
   ;; The long-term solution will be to keep a separate thread for
-  ;; finalizers and after-gc hooks.
+  ;; after-gc hooks.
+  ;; Finalizers are in a separate thread (usually),
+  ;; but it's not permissible to invoke CONDITION-NOTIFY from a
+  ;; dying thread, so we still need the guard for that, but not
+  ;; the guard for whether interupts are enabled.
   (when (sb!thread:thread-alive-p sb!thread:*current-thread*)
-    (when *allow-with-interrupts*
-      (sb!thread::without-thread-waiting-for ()
-        (with-interrupts
-          (run-pending-finalizers)
-          (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))))))
+    (let ((threadp #!+sb-thread (%instancep sb!impl::*finalizer-thread*)))
+      (when threadp
+        ;; It's OK to frob a condition variable regardless of
+        ;; *allow-with-interrupts*, and probably OK to start a thread.
+        ;; For consistency with the previous behavior, we delay finalization
+        ;; if there is no finalizer thread and interrupts are disabled.
+        ;; That's my excuse anyway, not having looked more in-depth.
+        (run-pending-finalizers))
+      (when *allow-with-interrupts*
+        (sb!thread::without-thread-waiting-for ()
+         (with-interrupts
+           (unless threadp
+             (run-pending-finalizers))
+           (call-hooks "after-GC" *after-gc-hooks* :on-error :warn)))))))
 
 ;;; This is the user-advertised garbage collection function.
 (defun gc (&key (full nil) (gen 0) &allow-other-keys)
