@@ -16,18 +16,16 @@
 #!-hpux (error "missing :HPUX feature")
 
 (defun software-type ()
-  #!+sb-doc
   "Return a string describing the supporting software."
   (values "HPUX"))
 
 (defun software-version ()
-  #!+sb-doc
   "Return a string describing version of the supporting software, or NIL
   if not available."
   (or *software-version*
       (setf *software-version*
             (string-trim '(#\newline)
-                         (with-output-to-string (stream)
+                         (sb!kernel:with-simple-output-to-string (stream)
                            (run-program "/bin/uname" `("-r")
                                         :output stream))))))
 
