@@ -79,15 +79,11 @@
 
 (defun internal-error-args (context)
   (declare (type (alien (* os-context-t)) context))
-  (/show0 "entering INTERNAL-ERROR-ARGS, CONTEXT=..")
-  (/hexstr context)
   (let* ((pc (context-pc context))
          (cause (context-bd-cause-int context))
          ;; KLUDGE: This exposure of the branch delay mechanism hurts.
-         (offset (if (logbitp 31 cause) 8 4))
-         (error-number (sap-ref-8 pc offset)))
+         (offset (if (logbitp 31 cause) 4 0))
+         (trap-number (ldb (byte 8 6) (sap-ref-32 pc offset))))
     (declare (type system-area-pointer pc))
-    (values error-number
-            (sb!kernel::decode-internal-error-args (sap+ pc (1+ offset))
-                                                   error-number))))
+    (sb!kernel::decode-internal-error-args (sap+ pc (+ offset 4)) trap-number)))
 ) ; end PROGN
