@@ -102,9 +102,8 @@
          (regnum (ldb (byte 5 0) op)))
     (declare (type system-area-pointer pc))
     (cond ((= op (logior (ash 3 10) (ash 6 5)))
-           (let ((error-number (sap-ref-8 pc 4)))
-             (values error-number
-                     (sb!kernel::decode-internal-error-args (sap+ pc 5) error-number))))
+           (let ((trap-number (sap-ref-8 pc 3)))
+             (sb!kernel::decode-internal-error-args (sap+ pc 4) trap-number)))
           ((and (= (ldb (byte 6 10) op) 3) ;; twi
                 (or (= regnum #.(sc-offset-offset arg-count-sc))
                     (= (ldb (byte 5 5) op) 24))) ;; :ne
