@@ -15,15 +15,17 @@
 ;;; Fix unknown types in globaldb
 (let ((l nil))
   (do-all-symbols (s)
-    (when (and (fboundp s) (not (macro-function s)))
-      (let ((ftype (info :function :type s)))
-        (when (contains-unknown-type-p ftype)
-          (setf (info :function :type s)
-                (specifier-type (type-specifier ftype)))
-          (push s l)))))
-  (let ((*print-pretty* nil)
-        (*print-length* nil))
-    (format t "~&; Fixed ftypes: ~S~%" (sort l #'string<))))
+    (multiple-value-bind (ftype present)
+        (info :function :type s)
+      (when (and present
+                 (contains-unknown-type-p ftype))
+        (setf (info :function :type s)
+              (specifier-type (type-specifier ftype)))
+        (push s l))))
+  (unless (sb-impl::!c-runtime-noinform-p)
+    (let ((*print-pretty* nil)
+          (*print-length* nil))
+      (format t "~&; Fixed ftypes: ~S~%" (sort l #'string<)))))
 
 (eval-when (:compile-toplevel :execute)
 
@@ -34,8 +36,10 @@
       (let ((res (type-specifier
                   (single-value-type
                    (values-specifier-type (third type)))))
-            (arglist (cons 'newval (sb-kernel:%fun-lambda-list
-                                    (symbol-function name)))))
+            (arglist (cons 'newval (or (sb-kernel:%fun-lambda-list
+                                        (symbol-function name))
+                                       ;; For low debug builds
+                                       (make-gensym-list (length args))))))
         `(locally
           (declare (muffle-conditions
                     ;; Expect SETF macro + function warnings.
