@@ -80,20 +80,11 @@
   (warn "stub CONTEXT-FLOATING-POINT-MODES")
   0)
 
-;;;; INTERNAL-ERROR-ARGS.
-
-;;; Given a (POSIX) signal context, extract the internal error
-;;; arguments from the instruction stream.  This is e.g.
-;;; 4       23      254     240     2       0       0       0
-;;; |       ~~~~~~~~~~~~~~~~~~~~~~~~~
-;;; length         data              (everything is an octet)
-;;;  (pc)
 (defun internal-error-args (context)
   (declare (type (alien (* os-context-t)) context))
   (/show0 "entering INTERNAL-ERROR-ARGS")
   (let* ((pc (context-pc context))
-         (error-number (sap-ref-8 pc 4)))
+         (trap-number (sap-ref-8 pc 3)))
     (declare (type system-area-pointer pc))
-    (values error-number
-            (sb!kernel::decode-internal-error-args (sap+ pc 5) error-number))))
+    (sb!kernel::decode-internal-error-args (sap+ pc 4) trap-number)))
 ) ; end PROGN
