@@ -13,7 +13,6 @@
   (sb!xc:defmacro %string (x) `(if (stringp ,x) ,x (string ,x))))
 
 (defun string (x)
-  #!+sb-doc
   "Coerces X into a string. If X is a string, X is returned. If X is a
    symbol, its name is returned. If X is a character then a one element
    string containing that character is returned. If X cannot be coerced
@@ -80,7 +79,6 @@
 ) ; EVAL-WHEN
 
 (defun char (string index)
-  #!+sb-doc
   "Given a string and a non-negative integer index less than the length of
   the string, returns the character object representing the character at
   that position in the string."
@@ -92,7 +90,6 @@
   (setf (char string index) new-el))
 
 (defun schar (string index)
-  #!+sb-doc
   "SCHAR returns the character object at an indexed position in a string
    just as CHAR does, except the string must be a simple-string."
   (declare (optimize (safety 1)))
@@ -234,7 +231,6 @@
   (string<>=*-body nil t))
 
 (defun string< (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically less than
   the second string, returns the longest common prefix (using char=)
   of the two strings. Otherwise, returns ()."
@@ -244,7 +240,6 @@
   (string<* string1 string2 0 nil 0 nil))
 
 (defun string> (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically greater than
   the second string, returns the longest common prefix (using char=)
   of the two strings. Otherwise, returns ()."
@@ -254,7 +249,6 @@
   (string>* string1 string2 0 nil 0 nil))
 
 (defun string<= (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically less than
   or equal to the second string, returns the longest common prefix
   (using char=) of the two strings. Otherwise, returns ()."
@@ -264,7 +258,6 @@
   (string<=* string1 string2 0 nil 0 nil))
 
 (defun string>= (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically greater
   than or equal to the second string, returns the longest common prefix
   (using char=) of the two strings. Otherwise, returns ()."
@@ -282,7 +275,6 @@
 ;;; efficient way to test for strings which begin with a particular
 ;;; pattern. Alas..) -- WHN 19991206
 (defun string= (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings (string1 and string2), and optional integers start1,
   start2, end1 and end2, compares characters in string1 to characters in
   string2 (using char=)."
@@ -292,7 +284,6 @@
   (string=* string1 string2 0 nil 0 nil))
 
 (defun string/= (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is not lexicographically equal
   to the second string, returns the longest common prefix (using char=)
   of the two strings. Otherwise, returns ()."
@@ -312,25 +303,25 @@
   (let ((end-test (if (= end 1)
                       `(= index1 (the fixnum end1))
                       `(= index2 (the fixnum end2)))))
-    `(locally (declare (inline two-arg-char-equal))
-       (do ((index1 start1 (1+ index1))
-            (index2 start2 (1+ index2)))
-           (,(if abortp
-                 end-test
-                 `(or ,end-test
-                      (not (char-equal (schar string1 index1)
-                                       (schar string2 index2)))))
-            ,end-value)
-         (declare (fixnum index1 index2))
-         ,@(if abortp
-               `((if (not (char-equal (schar string1 index1)
-                                      (schar string2 index2)))
-                     (return ,abort-value))))))))
+    `(do ((index1 start1 (1+ index1))
+         (index2 start2 (1+ index2)))
+        (,(if abortp
+              end-test
+              `(or ,end-test
+                   (not (two-arg-char-equal-inline
+                         (schar string1 index1)
+                         (schar string2 index2)))))
+         ,end-value)
+      (declare (fixnum index1 index2))
+      ,@(if abortp
+            `((if (not (two-arg-char-equal-inline
+                        (schar string1 index1)
+                        (schar string2 index2)))
+                  (return ,abort-value)))))))
 
 ) ; EVAL-WHEN
 
 (defun string-equal (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings (string1 and string2), and optional integers start1,
   start2, end1 and end2, compares characters in string1 to characters in
   string2 (using char-equal)."
@@ -352,7 +343,6 @@
         (string-not-equal-loop 1 t nil)))))
 
 (defun string-not-equal (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is not lexicographically equal
   to the second string, returns the longest common prefix (using char-equal)
   of the two strings. Otherwise, returns ()."
@@ -400,24 +390,23 @@
 (sb!xc:defmacro string-less-greater-equal (lessp equalp)
   (multiple-value-bind (length-test character-test)
       (string-less-greater-equal-tests lessp equalp)
-    `(locally (declare (inline two-arg-char-equal))
-       (with-two-strings string1 string2 start1 end1 offset1 start2 end2
-         (let ((slen1 (- (the fixnum end1) start1))
-               (slen2 (- (the fixnum end2) start2)))
-           (declare (fixnum slen1 slen2))
-           (do ((index1 start1 (1+ index1))
-                (index2 start2 (1+ index2))
-                (char1)
-                (char2))
-               ((or (= index1 (the fixnum end1)) (= index2 (the fixnum end2)))
-                (if (,length-test slen1 slen2) (- index1 offset1)))
-             (declare (fixnum index1 index2))
-             (setq char1 (schar string1 index1))
-             (setq char2 (schar string2 index2))
-             (if (not (char-equal char1 char2))
-                 (if ,character-test
-                     (return (- index1 offset1))
-                     (return ())))))))))
+    `(with-two-strings string1 string2 start1 end1 offset1 start2 end2
+       (let ((slen1 (- (the fixnum end1) start1))
+             (slen2 (- (the fixnum end2) start2)))
+         (declare (fixnum slen1 slen2))
+         (do ((index1 start1 (1+ index1))
+              (index2 start2 (1+ index2))
+              (char1)
+              (char2))
+             ((or (= index1 (the fixnum end1)) (= index2 (the fixnum end2)))
+              (if (,length-test slen1 slen2) (- index1 offset1)))
+           (declare (fixnum index1 index2))
+           (setq char1 (schar string1 index1))
+           (setq char2 (schar string2 index2))
+           (if (not (two-arg-char-equal-inline char1 char2))
+               (if ,character-test
+                   (return (- index1 offset1))
+                   (return ()))))))))
 
 ) ; EVAL-WHEN
 
@@ -438,7 +427,6 @@
   (string-less-greater-equal t t))
 
 (defun string-lessp (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically less than
   the second string, returns the longest common prefix (using char-equal)
   of the two strings. Otherwise, returns ()."
@@ -448,7 +436,6 @@
   (string-lessp* string1 string2 0 nil 0 nil))
 
 (defun string-greaterp (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically greater than
   the second string, returns the longest common prefix (using char-equal)
   of the two strings. Otherwise, returns ()."
@@ -458,7 +445,6 @@
   (string-greaterp* string1 string2 0 nil 0 nil))
 
 (defun string-not-lessp (string1 string2 &key (start1 0) end1 (start2 0) end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically greater
   than or equal to the second string, returns the longest common prefix
   (using char-equal) of the two strings. Otherwise, returns ()."
@@ -469,7 +455,6 @@
 
 (defun string-not-greaterp (string1 string2 &key (start1 0) end1 (start2 0)
                                     end2)
-  #!+sb-doc
   "Given two strings, if the first string is lexicographically less than
   or equal to the second string, returns the longest common prefix
   (using char-equal) of the two strings. Otherwise, returns ()."
@@ -482,7 +467,6 @@
 (defun make-string (count &key
                     (element-type 'character)
                     ((:initial-element fill-char)))
-  #!+sb-doc
   "Given a character count and an optional fill character, makes and returns a
 new string COUNT long filled with the fill character."
   (declare (index count))
@@ -562,28 +546,40 @@ new string COUNT long filled with the fill character."
                       (start)
                       (end)
                       :check-fill-pointer t)
-      (let* ((left-end (if left-p
-                           (do ((index start (1+ index)))
-                               ((or (= index (the fixnum end))
-                                    (not (find (schar string index)
-                                               char-bag
-                                               :test #'char=)))
-                                index)
-                             (declare (fixnum index)))
-                           start))
-             (right-end (if right-p
-                            (do ((index (1- (the fixnum end)) (1- index)))
-                                ((or (< index left-end)
-                                     (not (find (schar string index)
-                                                char-bag
-                                                :test #'char=)))
-                                 (1+ index))
-                              (declare (fixnum index)))
-                            end)))
-        (if (and (eql left-end start)
-                 (eql right-end end))
-            header
-            (subseq (the simple-string string) left-end right-end))))))
+      (flet ((trim-char-p (char)
+               (typecase char-bag
+                 (list
+                  (loop for bag-char in char-bag
+                        thereis (char= bag-char char)))
+                 (simple-string
+                  (loop for bag-char across char-bag
+                        thereis (char= bag-char char)))
+                 (simple-vector
+                  (loop for bag-char across char-bag
+                        thereis (char= bag-char char)))
+                 (vector
+                  (loop for bag-char across char-bag
+                        thereis (char= bag-char char)))
+                 (t
+                  (find char char-bag :test #'char=)))))
+        (let* ((left-end (if left-p
+                             (do ((index start (1+ index)))
+                                 ((or (= index (the fixnum end))
+                                      (not (trim-char-p (schar string index))))
+                                  index)
+                               (declare (fixnum index)))
+                             start))
+               (right-end (if right-p
+                              (do ((index (1- (the fixnum end)) (1- index)))
+                                  ((or (< index left-end)
+                                       (not (trim-char-p (schar string index))))
+                                   (1+ index))
+                                (declare (fixnum index)))
+                              end)))
+          (if (and (eql left-end start)
+                   (eql right-end end))
+              header
+              (subseq (the simple-string string) left-end right-end)))))))
 
 (defun string-left-trim (char-bag string)
   (generic-string-trim char-bag string t nil))
@@ -593,3 +589,14 @@ new string COUNT long filled with the fill character."
 
 (defun string-trim (char-bag string)
   (generic-string-trim char-bag string t t))
+
+(defun logically-readonlyize (vector &optional (always-shareable t))
+  ;; "Always" means that regardless of whether the user want
+  ;; coalescing of strings used as literals in code compiled to memory,
+  ;; the string is shareable.
+  ;;  #b01_ ; symbol name, literal compiled to fasl, some other stuff
+  ;;  #b10_ ; literal compiled to core
+  (set-header-data (the (simple-array * 1) vector)
+                   (if always-shareable
+                       sb!vm:+vector-shareable+
+                       sb!vm:+vector-shareable-nonstd+)))
