@@ -307,11 +307,6 @@ Examples:
         (malloc-error bytes (get-errno))
         sap)))
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defvar *saved-fp-and-pcs* nil)
-  ;; Can't use DECLAIM since always-bound is a non-standard declaration
-  (sb!xc:proclaim '(sb!ext:always-bound *saved-fp-and-pcs*)))
-
 #!+c-stack-is-control-stack
 (declaim (inline invoke-with-saved-fp-and-pc))
 ;;; On :c-stack-is-control-stack platforms, this DEFUN must appear prior to the
