@@ -26,14 +26,12 @@
 (declaim (type list *save-hooks* *init-hooks* *exit-hooks*))
 
 (defvar *save-hooks* nil
-  #!+sb-doc
   "A list of function designators which are called in an unspecified
 order before creating a saved core image.
 
 Unused by SBCL itself: reserved for user and applications.")
 
 (defvar *init-hooks* nil
-  #!+sb-doc
   "A list of function designators which are called in an unspecified
 order when a saved core image starts up, after the system itself has
 been initialized.
@@ -41,7 +39,6 @@ been initialized.
 Unused by SBCL itself: reserved for user and applications.")
 
 (defvar *exit-hooks* nil
-  #!+sb-doc
   "A list of function designators which are called in an unspecified
 order when SBCL process exits.
 
@@ -52,8 +49,9 @@ these hooks.")
 
 
 ;;; Binary search for simple vectors
-(defun binary-search (value seq &key (key #'identity))
+(defun binary-search* (value seq key)
   (declare (simple-vector seq))
+  (declare (function key))
   (labels ((recurse (start end)
              (when (< start end)
                (let* ((i (+ start (truncate (- end start) 2)))
@@ -64,9 +62,14 @@ these hooks.")
                        ((> value key-value)
                         (recurse (1+ i) end))
                        (t
-                        elt))))))
+                        i))))))
     (recurse 0 (length seq))))
 
+(defun binary-search (value seq &key (key #'identity))
+  (let ((index (binary-search* value seq key)))
+    (if index
+        (svref seq index))))
+
 (defun double-vector-binary-search (value vector)
   (declare (simple-vector vector)
            (optimize speed)
@@ -137,8 +140,3 @@ these hooks.")
                   (%shrink-vector string size)
                   string)))
          ,@body))))
-
-;;; The smallest power of two that is equal to or greater than X.
-(defun power-of-two-ceiling (x)
-  (declare (index x))
-  (ash 1 (integer-length (1- x))))
