@@ -20,9 +20,9 @@
   (declare (function function))
   ;; It's too bad that TYPECASE isn't able to generate equivalent code.
   (case (fun-subtype function)
-    (#.sb!vm:closure-header-widetag
+    (#.sb!vm:closure-widetag
      (%closure-fun function))
-    (#.sb!vm:funcallable-instance-header-widetag
+    (#.sb!vm:funcallable-instance-widetag
      ;; %FUNCALLABLE-INSTANCE-FUNCTION is not known to return a FUNCTION.
      ;; Is that right? Shouldn't we always initialize to something
      ;; that is a function, such as an error-signaling trampoline?
@@ -63,119 +63,145 @@
     (sb!interpreter:interpreted-function (sb!interpreter:%fun-type function))
     (t (%simple-fun-type (%fun-fun function)))))
 
-(!defglobal *closure-name-marker* (make-symbol ".CLOSURE-NAME."))
-(defun closure-name (closure)
-  (declare (closure closure))
-  (let ((len (get-closure-length closure)))
-    (if (and (>= len 4)
-             ;; The number of closure-values is 1- the len.
-             ;; The index of the last value is 1- that.
-             ;; The index of the name-marker is 1- that.
-             ;; (closure index 0 is the first closed-over value)
-             (eq (%closure-index-ref closure (- len 3))
-                 (load-time-value *closure-name-marker* t)))
-        (values (%closure-index-ref closure (- len 2)) t)
-        (values nil nil))))
+(define-load-time-global **closure-names**
+    (make-hash-table :test 'eq :weakness :key :synchronized t))
 
-;; Add 2 "slots" to the payload of a closure, one for the magic symbol
-;; signifying that there is a name, and one for the name itself.
-(defun nameify-closure (closure)
-  (declare (closure closure))
-  (let* ((physical-len (get-closure-length closure)) ; excluding header
-         ;; subtract 1 because physical-len includes the trampoline word.
-         (new-n-closure-vals (+ 2 (1- physical-len)))
-         ;; Closures and funcallable-instances are pretty much the same to GC.
-         ;; They're both varying-length boxed-payload objects.
-         ;; But funcallable-instance has <tramp, function, info>
-         ;; where closure has <tramp, info> so subtract 1 more word.
-         (copy (%make-funcallable-instance (1- new-n-closure-vals))))
-    (with-pinned-objects (closure copy)
-      ;; change the widetag from funcallable-instance to closure.
-      (setf (sap-ref-word (int-sap (get-lisp-obj-address copy))
-                          (- sb!vm:fun-pointer-lowtag))
-            (logior (ash (+ physical-len 2) 8) sb!vm:closure-header-widetag))
-      (macrolet ((word (obj index)
-                   `(sap-ref-lispobj (int-sap (get-lisp-obj-address ,obj))
-                                     (+ (- sb!vm:fun-pointer-lowtag)
-                                        (ash ,index sb!vm:word-shift)))))
-        (loop for i from 1 to physical-len
-              do (setf (word copy i) (word closure i)))
-        (setf (word copy (1+ physical-len)) *closure-name-marker*)))
-    copy))
+(macrolet ((namedp-bit () #x800000)
+           (%closure-index-set (closure index val)
+             ;; Use the identical convention as %CLOSURE-INDEX-REF for the index.
+             ;; There are no closure slot setters, and in fact SLOT-SET
+             ;; does not exist in a variant that takes a non-constant index.
+             `(setf (sap-ref-lispobj (int-sap (get-lisp-obj-address ,closure))
+                                     (+ (ash sb!vm:closure-info-offset sb!vm:word-shift)
+                                        (ash ,index sb!vm:word-shift)
+                                        (- sb!vm:fun-pointer-lowtag)))
+                    ,val))
+           (closure-header-word (closure)
+             `(sap-ref-word (int-sap (get-lisp-obj-address ,closure))
+                            (- sb!vm:fun-pointer-lowtag))))
 
-;; Rename a closure. Doing so changes its identity unless it was already named.
-;; To do this without allocating a new closure, we'd need an interface that
-;; requests a placeholder from the outset. One possibility is that
-;; (NAMED-LAMBDA NIL (x) ...) would allocate the name, initially stored as nil.
-;; In that case, the simple-fun's debug-info could also contain a bit that
-;; indicates that all closures over it are named, eliminating the storage
-;; and check for *closure-name-marker* in the closure values.
-(defun set-closure-name (closure new-name)
-  (declare (closure closure))
-  (unless (nth-value 1 (closure-name closure))
-    (setq closure (nameify-closure closure)))
-  ;; There are no closure slot setters, and in fact SLOT-SET
-  ;; does not exist in a variant that takes a non-constant index.
-  (with-pinned-objects (closure)
-    (setf (sap-ref-lispobj (int-sap (get-lisp-obj-address closure))
-                           (+ (- sb!vm:fun-pointer-lowtag)
-                              (ash (get-closure-length closure)
-                                   sb!vm:word-shift)))
-          new-name))
-  closure)
+  ;;; Assign CLOSURE a NEW-NAME and return the closure (NOT the new name!).
+  ;;; If PERMIT-COPY is true, this function may return a copy of CLOSURE
+  ;;; to avoid using a global hashtable. We prefer to store the name in the
+  ;;; closure itself, which is possible in two cases:
+  ;;; (1) if the closure has a padding slot due to alignment constraints,
+  ;;;     the padding slot holds the name.
+  ;;; (2) if it doesn't, but the caller does not care about identity,
+  ;;;     then a copy of the closure with extra slots reduces to case (1).
+  (defun set-closure-name (closure permit-copy new-name)
+    (declare (closure closure))
+    (let ((payload-len (get-closure-length (truly-the function closure)))
+          (namedp (logtest (with-pinned-objects (closure)
+                             (closure-header-word closure))
+                           (namedp-bit))))
+      (when (and (not namedp) permit-copy (oddp payload-len))
+        ;; PAYLOAD-LEN includes the trampoline, so the number of slots we would
+        ;; pass to %COPY-CLOSURE is 1 less than that, were it not for
+        ;; the fact that we actually want to create 1 additional slot.
+        ;; So in effect, asking for PAYLOAD-LEN does exactly the right thing.
+        (let ((copy #!-(or x86 x86-64)
+                    (sb!vm::%copy-closure payload-len (%closure-fun closure))
+                    #!+(or x86 x86-64)
+                    (with-pinned-objects ((%closure-fun closure))
+                      ;; %CLOSURE-CALLEE manifests as a fixnum which remains
+                      ;; valid across GC due to %CLOSURE-FUN being pinned
+                      ;; until after the new closure is made.
+                      (sb!vm::%copy-closure payload-len
+                                            (sb!vm::%closure-callee closure)))))
+          (with-pinned-objects (copy)
+            (loop with sap = (int-sap (get-lisp-obj-address copy))
+                  for i from 0 below (1- payload-len)
+                  for ofs from (- (ash 2 sb!vm:word-shift) sb!vm:fun-pointer-lowtag)
+                  by sb!vm:n-word-bytes
+                  do (setf (sap-ref-lispobj sap ofs) (%closure-index-ref closure i)))
+            (setf (closure-header-word copy) ; Update the header
+                  ;; Closure copy lost its high header bits, so OR them in again.
+                  (logior #!+(and immobile-space 64-bit)
+                          (get-lisp-obj-address sb!vm:function-layout)
+                          (namedp-bit)
+                          (closure-header-word copy)))
+            ;; We copy only if there was no padding, which means that adding 1 slot
+            ;; physically added 2 slots. You might think that the name goes in the
+            ;; first new slot, followed by padding. Nope! Take an example of a
+            ;; closure header specifying 3 words, which we increase to 4 words,
+            ;; which is 5 with the header, which is 6 aligned to a doubleword:
+            ;;   word 0: 2030008300800435
+            ;;   word 1: 0000001001A0B280             ] words the closure's fun
+            ;;   word 2: 00000000203AA39F = #<thing1> ] "should" use given
+            ;;   word 3: 00000000203AA51F = #<thing2> ] 4 payload slots.
+            ;;   word 4: 0000000000000000 = 0         ]
+            ;;   word 5: 00000010019E2B9F = NAME    <-- name goes here
+            ;; This makes CLOSURE-NAME consistent (which is to say, work at all).
+            (%closure-index-set copy payload-len new-name)
+            (return-from set-closure-name copy))))
+      (unless (with-pinned-objects (closure)
+                (unless namedp ; Set the header bit
+                  (setf (closure-header-word closure)
+                        (logior (namedp-bit) (closure-header-word closure))))
+                (when (evenp payload-len)
+                  (%closure-index-set closure (1- payload-len) new-name)
+                  t))
+        (setf (gethash closure **closure-names**) new-name)))
+    closure)
+
+  ;;; Access the name of CLOSURE. Secondary value is whether it had a name.
+  ;;; Following the convention established by SET-CLOSURE-NAME,
+  ;;; an even-length payload takes the last slot for the name,
+  ;;; and an odd-length payload uses the global hashtable.
+  (defun closure-name (closure)
+    (declare (closure closure))
+    (with-pinned-objects (closure)
+      (if (not (logtest (closure-header-word closure) (namedp-bit)))
+          (values nil nil)
+          ;; CLOSUREP doesn't imply FUNCTIONP, so GET-CLOSURE-LENGTH
+          ;; issues an additional check. Silly.
+          (let ((len (get-closure-length (truly-the function closure))))
+            ;; GET-CLOSURE-LENGTH counts the 'fun' slot in the length,
+            ;; but %CLOSURE-INDEX-REF starts indexing from the value slots.
+            (if (oddp len)
+                (gethash closure **closure-names**) ; returns name and T
+                (values (%closure-index-ref closure (1- len)) t)))))))
 
 ;;; a SETFable function to return the associated debug name for FUN
 ;;; (i.e., the third value returned from CL:FUNCTION-LAMBDA-EXPRESSION),
 ;;; or NIL if there's none
 (defun %fun-name (function)
   (case (fun-subtype function)
-    (#.sb!vm:funcallable-instance-header-widetag
-     (let ((layout (%funcallable-instance-layout function)))
-       ;; We know that funcallable-instance-p is true,
-       ;; and so testing via TYPEP would be wasteful.
-       (cond #!+sb-eval
-             ((eq layout #.(find-layout 'sb!eval:interpreted-function))
-              (return-from %fun-name
-                (sb!eval:interpreted-function-debug-name function)))
-             #!+sb-fasteval
-             ((eq layout #.(find-layout 'sb!interpreter:interpreted-function))
-               (return-from %fun-name
-                 (sb!interpreter:proto-fn-name
-                  (sb!interpreter:fun-proto-fn
-                   (truly-the sb!interpreter:interpreted-function function)))))
-             ((classoid-cell-typep #.(find-classoid-cell 'standard-generic-function)
-                                   function)
-              (return-from %fun-name
-                (sb!mop:generic-function-name function))))))
-    (#.sb!vm:closure-header-widetag
-     (multiple-value-bind (name namedp) (closure-name function)
+    (#.sb!vm:closure-widetag
+     (multiple-value-bind (name namedp) (closure-name (truly-the closure function))
        (when namedp
-         (return-from %fun-name name)))))
+         (return-from %fun-name name))))
+    (#.sb!vm:funcallable-instance-widetag
+     (typecase (truly-the funcallable-instance function)
+       (generic-function
+        (return-from %fun-name
+          (sb!mop:generic-function-name function)))
+       #!+sb-eval
+       (sb!eval:interpreted-function
+        (return-from %fun-name (sb!eval:interpreted-function-debug-name function)))
+       #!+sb-fasteval
+       (sb!interpreter:interpreted-function
+        (return-from %fun-name
+          (sb!interpreter:proto-fn-name (sb!interpreter:fun-proto-fn function)))))))
   (%simple-fun-name (%fun-fun function)))
 
 (defun (setf %fun-name) (new-value function)
-  (typecase function
-    #!+sb-eval
-    (sb!eval:interpreted-function
-     (setf (sb!eval:interpreted-function-debug-name function) new-value))
-    #!+sb-fasteval
-    (sb!interpreter:interpreted-function
-     (setf (sb!interpreter:proto-fn-name (sb!interpreter:fun-proto-fn function))
-           new-value))
-    (generic-function
-     ;; STANDARD-GENERIC-FUNCTION definitely has a NAME,
-     ;; but other subtypes of GENERIC-FUNCTION could as well.
-     (when (slot-exists-p function 'sb!pcl::name)
-       (setf (slot-value function 'sb!pcl::name) new-value)))
-    ;; This does not set the name of an un-named closure because doing so
-    ;; is not a side-effecting operation that it ought to be.
-    ;; In contrast, SB-PCL::SET-FUN-NAME specifically says that only if the
-    ;; argument fun is a funcallable instance must it retain its identity.
-    ;; That function *is* allowed to cons a new closure to name it.
-    ((or simple-fun closure)
-     (if (and (closurep function) (nth-value 1 (closure-name function)))
-         (set-closure-name function new-value)
-         (setf (%simple-fun-name (%fun-fun function)) new-value))))
+  (case (fun-subtype function)
+    (#.sb!vm:closure-widetag
+     (set-closure-name (truly-the closure function) nil new-value))
+    (#.sb!vm:simple-fun-widetag
+     (setf (%simple-fun-name function) new-value))
+    (t
+     (typecase (truly-the funcallable-instance function)
+       (generic-function
+        (setf (sb!mop:generic-function-name function) new-value))
+       #!+sb-eval
+       (sb!eval:interpreted-function
+        (setf (sb!eval:interpreted-function-debug-name function) new-value))
+       #!+sb-fasteval
+       (sb!interpreter:interpreted-function
+        (setf (sb!interpreter:proto-fn-name (sb!interpreter:fun-proto-fn function))
+              new-value)))))
   new-value)
 
 (defun %fun-doc (function)
@@ -251,10 +277,12 @@
   (let ((f (%code-entry-point code-obj 0)))
     (or (and f
              (let ((from (code-header-words code-obj))
-                   (to (ash (with-pinned-objects (f)
+                   ;; Ignore the layout pointer (if present) in the upper bits
+                   ;; of the function header.
+                   (to (ldb (byte 24 sb!vm:n-widetag-bits)
+                            (with-pinned-objects (f)
                               (sap-ref-word (int-sap (get-lisp-obj-address f))
-                                            (- sb!vm:fun-pointer-lowtag)))
-                            (- sb!vm:n-widetag-bits))))
+                                            (- sb!vm:fun-pointer-lowtag))))))
                (and (< from to) (- to from))))
         0)))
 
@@ -262,12 +290,10 @@
 
 (defvar *features*
   '#.(sort (copy-list sb-cold:*shebang-features*) #'string<)
-  #!+sb-doc
   "a list of symbols that describe features provided by the
    implementation")
 
 (defun machine-instance ()
-  #!+sb-doc
   "Return a string giving the name of the local machine."
   #!+win32 (sb!win32::get-computer-name)
   #!-win32 (truly-the simple-string (sb!unix:unix-gethostname)))
@@ -276,7 +302,6 @@
 (defvar *machine-version*)
 
 (defun machine-version ()
-  #!+sb-doc
   "Return a string describing the version of the computer hardware we
 are running on, or NIL if we can't find any useful information."
   (unless (boundp '*machine-version*)
@@ -291,28 +316,22 @@ are running on, or NIL if we can't find any useful information."
 ;;; a symbol in COMMON-LISP..
 (declaim (type (or null string) *short-site-name* *long-site-name*))
 (defvar *short-site-name* nil
-  #!+sb-doc
   "The value of SHORT-SITE-NAME.")
 (defvar *long-site-name* nil
-  #!+sb-doc
   "The value of LONG-SITE-NAME.")
 (defun short-site-name ()
-  #!+sb-doc
   "Return a string with the abbreviated site name, or NIL if not known."
   *short-site-name*)
 (defun long-site-name ()
-  #!+sb-doc
   "Return a string with the long form of the site name, or NIL if not known."
   *long-site-name*)
 
 ;;;; ED
 (declaim (type list *ed-functions*))
 (defvar *ed-functions* '()
-  #!+sb-doc
   "See function documentation for ED.")
 
 (defun ed (&optional x)
-  #!+sb-doc
   "Starts the editor (on a file or a function if named).  Functions
 from the list *ED-FUNCTIONS* are called in order with X as an argument
 until one of them returns non-NIL; these functions are responsible for
@@ -322,7 +341,7 @@ the file system."
            (error 'extension-failure
                   :format-control "Don't know how to ~S ~A"
                   :format-arguments (list 'ed x)
-                  :references (list '(:sbcl :variable *ed-functions*))))
+                  :references '((:sbcl :variable *ed-functions*))))
     (when (funcall fun x)
       (return t))))
 
@@ -344,7 +363,6 @@ the file system."
 (defvar *dribble-stream* nil)
 
 (defun dribble (&optional pathname &key (if-exists :append))
-  #!+sb-doc
   "With a file name as an argument, dribble opens the file and sends a
   record of further I/O to that file. Without an argument, it closes
   the dribble file, and quits logging."
@@ -379,12 +397,10 @@ the file system."
 ;;;; some *LOAD-FOO* variables
 
 (defvar *load-print* nil
-  #!+sb-doc
   "the default for the :PRINT argument to LOAD")
 
 (defvar *load-verbose* nil
   ;; Note that CMU CL's default for this was T, and ANSI says it's
   ;; implementation-dependent. We choose NIL on the theory that it's
   ;; a nicer default behavior for Unix programs.
-  #!+sb-doc
   "the default for the :VERBOSE argument to LOAD")
