@@ -210,9 +210,12 @@ potentially stale even before the function returns, as the thread may exit at
 any time."
   (thread-%alive-p thread))
 
+;; NB: ephemeral threads must terminate strictly before the test of NTHREADS>1
+;; in DEINIT, i.e. this is not a promise that the thread will terminate
+;; just-in-time for the final call out to save, but rather by an earlier time.
 (defun thread-ephemeral-p (thread)
   "Return T if THREAD is `ephemeral', which indicates that this thread is
-used by SBCL for internal purposes, and specifically that it knows how to
+used by SBCL for internal purposes, and specifically that our runtime knows how
 to terminate this thread cleanly prior to core file saving without signalling
 an error in that case."
   (thread-%ephemeral-p thread))
@@ -820,7 +823,7 @@ IF-NOT-OWNER is :FORCE)."
   #!+sb-thread
   (let ((me *current-thread*))
     (barrier (:read))
-    (assert (eq me (mutex-%owner mutex)))
+    (aver (eq me (mutex-%owner mutex)))
     (let ((status :interrupted))
       ;; Need to disable interrupts so that we don't miss grabbing
       ;; the mutex on our way out.
@@ -1229,7 +1232,7 @@ on this semaphore, then N of them is woken up."
 
 (defmacro with-new-session (args &body forms)
   (declare (ignore args))               ;for extensibility
-  (sb!int:with-unique-names (fb-name)
+  (with-unique-names (fb-name)
     `(labels ((,fb-name () ,@forms))
       (call-with-new-session (function ,fb-name)))))
 
@@ -1275,6 +1278,7 @@ on this semaphore, then N of them is woken up."
             (lambda (c h)
               (sb!debug::debugger-disabled-hook c h :quit nil)
               (abort-thread :allow-exit t)))
+      (sb!impl::finalizer-thread-stop)
       (dolist (thread (list-all-threads))
         (cond ((eq thread current))
               ((main-thread-p thread)
@@ -1390,7 +1394,7 @@ session."
   (first (interactive-threads session)))
 
 (defun make-listener-thread (tty-name)
-  (assert (probe-file tty-name))
+  (aver (probe-file tty-name))
   (let* ((in (sb!unix:unix-open (namestring tty-name) sb!unix:o_rdwr #o666))
          (out (sb!unix:unix-dup in))
          (err (sb!unix:unix-dup in)))
@@ -1413,7 +1417,7 @@ session."
                  (with-new-session ()
                    (unwind-protect
                         (sb!impl::toplevel-repl nil)
-                     (sb!int:flush-standard-output-streams))))))
+                     (flush-standard-output-streams))))))
       (make-thread #'thread-repl))))
 
 
@@ -1515,7 +1519,7 @@ See also: RETURN-FROM-THREAD, ABORT-THREAD."
            (real-function (coerce function 'function))
            (arguments     (ensure-list arguments))
            #!+(or win32 darwin)
-           (fp-modes (dpb 0 sb!vm::float-sticky-bits ;; clear accrued bits
+           (fp-modes (dpb 0 sb!vm:float-sticky-bits ;; clear accrued bits
                           (sb!vm:floating-point-modes))))
       (declare (dynamic-extent setup-sem))
       (dx-flet ((initial-thread-function ()
