@@ -75,6 +75,7 @@
 ;;; also used in EQ comparisons to determined if two types have the
 ;;; "same kind".
 (def!struct (type-class
+             (:copier nil)
              #-no-ansi-print-object
              (:print-object (lambda (x stream)
                               (print-unreadable-object (x stream :type t)
@@ -150,7 +151,7 @@
   ;; is disjoint from MEMBER-TYPE and so forth. But types which can
   ;; contain other types, like HAIRY-TYPE and INTERSECTION-TYPE, can
   ;; violate this rule.
-  (might-contain-other-types-p nil)
+  (might-contain-other-types-p nil :type boolean :read-only t)
   ;; a function which returns T if the CTYPE could possibly be
   ;; equivalent to a MEMBER type. If not a function, then it's
   ;; a constant T or NIL for all instances of this type class.
@@ -162,7 +163,7 @@
   ;; set enumerable=T for NEGATION and HAIRY and some other things.
   ;; Conceptually the choices are really {yes, no, unknown}, but
   ;; whereas "no" means "definitely not", T means "yes or maybe".
-  (enumerable-p nil :type (or function null t))
+  (enumerable-p nil :type (or function boolean) :read-only t)
   ;; a function which returns T if the CTYPE is inhabited by a single
   ;; object and, as a value, the object.  Otherwise, returns NIL, NIL.
   ;; The default case (NIL) is interpreted as a function that always
@@ -222,7 +223,7 @@
 
 (def!struct (ctype (:conc-name type-)
                    (:constructor nil)
-                   (:make-load-form-fun make-type-load-form)
+                   (:copier nil)
                    #-sb-xc-host (:pure t))
   ;; the class of this type
   ;;
@@ -362,7 +363,10 @@
                                 (,(!type-class-fun-slot name) parent))))))))
     #-sb-xc
     `(if (find ',name *type-classes* :key #'type-class-name)
-         (warn "Not redefining type-class ~S" ',name)
+         ;; Careful: type-classes are very complicated things to redefine.
+         ;; For the sake of parallelized make-host-1 we have to allow it
+         ;; not to be an error to get here, but we can't overwrite anything.
+         (style-warn "Not redefining type-class ~S" ',name)
          (vector-push-extend ,make-it *type-classes*))
     ;; The Nth entry in the array of classes contain a list of instances
     ;; of the type-class created by genesis that need patching.
@@ -387,7 +391,10 @@
             (princ (length backpatch-list))
             (terpri))
           (dolist (instance backpatch-list)
-            (setf (%instance-ref instance ,slot-index) type-class)))))))
+            ;; Fixup the class first, in case fixing the hash needs the class.
+            ;; (It doesn't currently, but just in case it does)
+            (setf (%instance-ref instance ,slot-index) type-class)
+            (!fix-ctype-hash instance)))))))
 
 ;;; Define the translation from a type-specifier to a type structure for
 ;;; some particular type. Syntax is identical to DEFTYPE.
@@ -408,17 +415,18 @@
          (lexpr (make-macro-lambda nil lambda-list stuff nil nil
                                    :accessor (if allow-atom 'identity 'cdr)
                                    :environment nil))
-         (ll-decl (third lexpr)))
+         (ll-decl (third lexpr))
+         (defun-name (symbolicate "PARSE-<" name ">")))
     (aver (and (eq (car ll-decl) 'declare) (caadr ll-decl) 'sb!c::lambda-list))
-    `(!cold-init-forms
-      (setf (info :type :expander ',name)
-            (list
-             (named-lambda ,(format nil "~A-TYPE-PARSE" name) (,context spec)
-              ,ll-decl
-              ,@(unless context-var-p `((declare (ignore ,context))))
-              ,(if allow-atom
-                   `(,lexpr (and (listp spec) (cdr spec)))
-                   `(if (listp spec) (,lexpr spec)))))))))
+    `(progn
+       (defun ,defun-name (,context spec)
+         ,ll-decl
+         ,@(unless context-var-p `((declare (ignore ,context))))
+         ,(if allow-atom
+              `(,lexpr (and (listp spec) (cdr spec)))
+              `(if (listp spec) (,lexpr spec))))
+       (!cold-init-forms
+        (setf (info :type :expander ',name) (list #',defun-name))))))
 
 ;;; Invoke a type method on TYPE1 and TYPE2. If the two types have the
 ;;; same class, invoke the simple method. Otherwise, invoke any
@@ -516,8 +524,6 @@
         do (setq res (logxor (ash res -1) (type-hash-value type)))
         finally (return res)))
 
-(!defun-from-collected-cold-init-forms !type-class-cold-init)
-
 ;;; A few type representations need to be defined slightly earlier than
 ;;; 'early-type' is compiled, so they're defined here.
 
@@ -535,6 +541,17 @@
                        (:copier nil))
   (name nil :type symbol :read-only t))
 
+;;; A MEMBER-TYPE represent a use of the MEMBER type specifier. We
+;;; bother with this at this level because MEMBER types are fairly
+;;; important and union and intersection are well defined.
+(defstruct (member-type (:include ctype
+                                  (class-info (type-class-or-lose 'member)))
+                        (:copier nil)
+                        (:constructor %make-member-type (xset fp-zeroes))
+                        #-sb-xc-host (:pure nil))
+  (xset nil :type xset :read-only t)
+  (fp-zeroes nil :type list :read-only t))
+
 ;;; An ARRAY-TYPE is used to represent any array type, including
 ;;; things such as SIMPLE-BASE-STRING.
 (defstruct (array-type (:include ctype
@@ -613,9 +630,8 @@
 
 ;;; a list of all the float "formats" (i.e. internal representations;
 ;;; nothing to do with #'FORMAT), in order of decreasing precision
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defparameter *float-formats*
-    '(long-float double-float single-float short-float)))
+(defglobal *float-formats*
+    '(long-float double-float single-float short-float))
 
 ;;; The type of a float format.
 (deftype float-format () `(member ,@*float-formats*))
@@ -628,7 +644,7 @@
                          (:copier nil))
   ;; Formerly defined in every CTYPE, but now just in the ones
   ;; for which enumerability is variable.
-  (enumerable nil :read-only t)
+  (enumerable nil :type boolean :read-only t)
   ;; the kind of numeric type we have, or NIL if not specified (just
   ;; NUMBER or COMPLEX)
   ;;
@@ -673,7 +689,7 @@
 
 (in-package "SB!ALIEN")
 (def!struct (alien-type
-             (:make-load-form-fun sb!kernel:just-dump-it-normally)
+             (:copier nil)
              (:constructor make-alien-type
                            (&key class bits alignment
                             &aux (alignment
@@ -681,6 +697,7 @@
   (class 'root :type symbol :read-only t)
   (bits nil :type (or null unsigned-byte))
   (alignment nil :type (or null unsigned-byte)))
+(!set-load-form-method alien-type (:xc :target))
 
 (in-package "SB!KERNEL")
 (defstruct (alien-type-type
@@ -717,7 +734,8 @@
 
 ;;; (SPECIFIER-TYPE 'FUNCTION) and its subtypes
 (defstruct (fun-type (:include args-type
-                               (class-info (type-class-or-lose 'function)))
+                      (class-info (type-class-or-lose 'function)))
+                     (:copier nil)
                      (:constructor
                       %make-fun-type (required optional rest
                                       keyp keywords allowp wild-args returns)))
@@ -727,6 +745,13 @@
   ;; when multiple values were specified for the return.
   (returns (missing-arg) :type ctype :read-only t))
 
+(defstruct (fun-designator-type
+            (:include fun-type)
+            (:copier nil)
+            (:conc-name fun-type-)
+            (:constructor make-fun-designator-type
+                (required optional rest
+                 keyp keywords allowp wild-args returns))))
 (declaim (ftype (sfunction (ctype ctype) (values t t)) csubtypep))
 ;;; Look for nice relationships for types that have nice relationships
 ;;; only when one is a hierarchical subtype of the other.
@@ -748,3 +773,17 @@
   (cond ((csubtypep type1 type2) type2)
         ((csubtypep type2 type1) type1)
         (t nil)))
+
+;; KLUDGE: putting this here satisfies CMUCL for an inexplicable reason.
+;; It should suffice to put it anywhere before %MAKE-CHARACTER-SET-TYPE
+;; is actually called.
+;;
+;; all character-set types are enumerable, but it's not possible
+;; for one to be TYPE= to a MEMBER type because (MEMBER #\x)
+;; is not internally represented as a MEMBER type.
+;; So in case it wasn't clear already ENUMERABLE-P does not mean
+;;  "possibly a MEMBER type in the Lisp-theoretic sense",
+;; but means "could be implemented in SBCL as a MEMBER type".
+(!define-type-class character-set :enumerable nil
+                    :might-contain-other-types nil)
+(!defun-from-collected-cold-init-forms !type-class-cold-init)
