@@ -14,7 +14,6 @@
 ;;; type TYPE. This behavior is needed e.g. to test for the validity
 ;;; of numeric subtype bounds read when cross-compiling.)
 (defun typep (object type &optional environment)
-  #!+sb-doc
   "Is OBJECT of type TYPE?"
   (declare (type lexenv-designator environment) (ignore environment))
   (declare (explicit-check))
@@ -54,7 +53,9 @@
                          (realpart object)
                          object)))
             (ecase (numeric-type-class type)
-              (integer (integerp num))
+              (integer (and (integerp num)
+                            (or (not (complexp object))
+                                (integerp (imagpart object)))))
               (rational (rationalp num))
               (float
                (ecase (numeric-type-format type)
@@ -262,8 +263,8 @@
         (when (layout-invalid obj-layout)
           (setq obj-layout (update-object-layout-or-invalid object layout)))
         (%ensure-classoid-valid classoid layout "typep"))
-    (let ((obj-inherits (layout-inherits obj-layout)))
-      (or (eq obj-layout layout)
+    (or (eq obj-layout layout)
+        (let ((obj-inherits (layout-inherits obj-layout)))
           (dotimes (i (length obj-inherits) nil)
             (when (eq (svref obj-inherits i) layout)
               (return t)))))))
