@@ -88,18 +88,6 @@
 (unless (ignore-errors (funcall (constantly t) 1 2 3))
   (error "please find a binary that understands CONSTANTLY to build from"))
 
-;;;; Self-hosted issues
-
-#+sbcl
-(progn
-  (setq *compile-print* nil)
-  (load "src/cold/muffler.lisp")
-  ;; Let's just say we never care to see these.
-  (declaim (sb-ext:muffle-conditions
-            (satisfies unable-to-optimize-note-p)
-            (satisfies optional+key-style-warning-p)
-            sb-ext:code-deletion-note)))
-
 ;;;; general non-ANSI-ness
 
 (in-package :sb-cold)
