@@ -30,11 +30,11 @@
 ;;; backend-subfeatures
 (setf sb-cold:*shebang-backend-subfeatures* sb-c:*backend-subfeatures*)
 
-(handler-bind (#+sb-package-locks (sb-ext:package-locked-error #'continue))
+(handler-bind ((sb-ext:package-locked-error #'continue))
   ;; The nickname SB!XC now refers to the CL package.
   (rename-package "COMMON-LISP" "COMMON-LISP"
                   (cons "SB!XC" (package-nicknames "CL")))
-  #+sb-package-locks (sb-ext:unlock-package "CL")
+  (sb-ext:unlock-package "CL")
 
   ;; Any other name SB!FOO refers to the package now called SB-FOO.
   (dolist (package (list-all-packages))
@@ -48,7 +48,7 @@
         (let* ((stem (subseq name (length cold-name-prefix)))
                (cold-name (concatenate 'simple-string cold-name-prefix stem)))
           (rename-package package name (cons cold-name nicknames)))
-        #+sb-package-locks (sb-ext:unlock-package package)))))
+        (sb-ext:unlock-package package)))))
 
 ;; Reinstate the pre-cold-init variable-defining macros.
 (let ((*package* (find-package "SB-INT")))
@@ -60,17 +60,11 @@
     (def 'defparameter)
     (def 'defvar)))
 
-(export '(sb-int::def!method sb-int::defmacro-mundanely
-          sb-int::!cold-init-forms
+(export '(sb-int::!cold-init-forms
           sb-int::!coerce-to-specialized
-          sb-int::/show sb-int::/noshow sb-int::/show0 sb-int::/noshow0
-          sb-int::!uncross-format-control)
+          sb-int::/show sb-int::/noshow sb-int::/show0 sb-int::/noshow0)
         'sb-int)
 
-(setf (macro-function 'sb-int:def!method) (macro-function 'defmethod))
-(defmacro sb-int:defmacro-mundanely (name lambda-list &body body)
-  `(let () (defmacro ,name ,lambda-list ,@body)))
-
 (defmacro sb-int:!cold-init-forms (&rest forms) `(progn ,@forms))
 
 ;; This macro is never defined for the target Lisp,
@@ -79,7 +73,6 @@
 (defmacro sb-int:!coerce-to-specialized (a type)
   (declare (ignore type))
   a)
-(defmacro sb-int:!uncross-format-control (s) s)
 
 ;; If :sb-show is present, then these symbols are fboundp.
 ;; Otherwise define them as no-ops.
