@@ -51,9 +51,4 @@
                            (concatenate 'string (stem-remap-target stem)
                                         ".trace")
                            srcname))))
-        (handler-bind ((warning
-                        (lambda (c)
-                          (when (search "reserved by ANSI CL"
-                                        (write-to-string c :escape nil :pretty nil))
-                            (muffle-warning)))))
-          (target-compile-stem stem flags))))))
+        (target-compile-stem stem flags)))))
