@@ -17,12 +17,8 @@
 
 ;;;; compiler constants
 
-(def!constant +backend-fasl-file-implementation+ :alpha)
-
-(setf *backend-register-save-penalty* 3)
-
-(setf *backend-byte-order* :little-endian)
+(defconstant +backend-fasl-file-implementation+ :alpha)
 
 ;;; XXX the C runtime gets page size using getpagesize() - can't we
 ;;; look at that instead of hardcoding it here too?
-(setf *backend-page-bytes* 8192)
+(defconstant +backend-page-bytes+ 8192)
