@@ -13,14 +13,14 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(sb!vm::zero sb!vm::fp-single-zero sb!vm::fp-double-zero
             sb!vm::registers sb!vm::float-registers
             sb!vm::zero-tn sb!vm::fp-single-zero-tn sb!vm::fp-double-zero-tn
             sb!vm::zero-offset sb!vm::null-offset sb!vm::code-offset)))
 
-(setf *disassem-inst-alignment-bytes* 4)
+(defconstant +disassem-inst-alignment-bytes+ 4)
 
 
 ;;;; utility functions
@@ -483,27 +483,27 @@
 (defun bugchk-trap-control (chunk inst stream dstate)
   (declare (ignore inst))
   (flet ((nt (x) (if stream (note x dstate))))
-    (case (bugchk-trap-code chunk dstate)
-      (#.halt-trap
-       (nt "Halt trap"))
-      (#.pending-interrupt-trap
-       (nt "Pending interrupt trap"))
-      (#.error-trap
-       (nt "Error trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.cerror-trap
-       (nt "Cerror trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.breakpoint-trap
-       (nt "Breakpoint trap"))
-      (#.fun-end-breakpoint-trap
-       (nt "Function end breakpoint trap"))
-      (#.single-step-breakpoint-trap
-       (nt "Single step breakpoint trap"))
-      (#.single-step-around-trap
-       (nt "Single step around trap"))
-      (#.single-step-before-trap
-       (nt "Single step before trap")))))
+    (let ((trap (bugchk-trap-code chunk dstate)))
+      (case trap
+        (#.halt-trap
+         (nt "Halt trap"))
+        (#.pending-interrupt-trap
+         (nt "Pending interrupt trap"))
+        (#.cerror-trap
+         (nt "Cerror trap")
+         (handle-break-args #'snarf-error-junk trap stream dstate))
+        (#.breakpoint-trap
+         (nt "Breakpoint trap"))
+        (#.fun-end-breakpoint-trap
+         (nt "Function end breakpoint trap"))
+        (#.single-step-breakpoint-trap
+         (nt "Single step breakpoint trap"))
+        (#.single-step-around-trap
+         (nt "Single step around trap"))
+        (#.single-step-before-trap
+         (nt "Single step before trap"))
+        (t
+         (handle-break-args #'snarf-error-junk trap stream dstate))))))
 
 (define-instruction gentrap (segment code)
   (:printer bugchk () :default
@@ -642,12 +642,12 @@
 (define-instruction simple-fun-header-word (segment)
   (:cost 0)
   (:emitter
-   (emit-header-data segment simple-fun-header-widetag)))
+   (emit-header-data segment simple-fun-widetag)))
 
 (define-instruction lra-header-word (segment)
   (:cost 0)
   (:emitter
-   (emit-header-data segment return-pc-header-widetag)))
+   (emit-header-data segment return-pc-widetag)))
 
 (defun emit-compute-inst (segment vop dst src label temp calc)
   (declare (ignore temp))
