@@ -191,14 +191,9 @@
   (assemble ()
     (when vop
       (note-this-location vop :internal-error))
-    (inst gentrap kind)
-    (inst byte code)
-    (with-adjustable-vector (vector)
-      (dolist (tn values)
-        (write-var-integer
-         (make-sc-offset (sc-number (tn-sc tn)) (tn-offset tn)) vector))
-      (dotimes (i (length vector))
-        (inst byte (aref vector i))))
+    (emit-internal-error kind code values
+                         :trap-emitter (lambda (tramp-number)
+                                         (inst gentrap tramp-number)))
     (emit-alignment word-shift)))
 
 (defun generate-error-code (vop error-code &rest values)
