@@ -44,8 +44,6 @@
             :load-if (not (location= x y))))
   (:results (y :scs (sap-reg)
                :load-if (not (location= x y))))
-  (:effects)
-  (:affected)
   (:generator 0
     (move x y)))
 (define-move-vop sap-move :move
