@@ -14,9 +14,7 @@
 ;;;; unary operations.
 
 (define-vop (fast-safe-arith-op)
-  (:policy :fast-safe)
-  (:effects)
-  (:affected))
+  (:policy :fast-safe))
 
 (define-vop (fixnum-unop fast-safe-arith-op)
   (:args (x :scs (any-reg)))
@@ -342,15 +340,15 @@
     (inst cmp temp 0)
     (inst b :ge LEFT)
     (inst rsb temp temp 0) ;; negate
-    (inst cmp temp sb!vm:n-word-bits)
-    (inst mov :gt temp sb!vm:n-word-bits)
+    (inst cmp temp n-word-bits)
+    (inst mov :gt temp n-word-bits)
     (inst mov result (ecase variant
                        (:signed (asr number temp))
                        (:unsigned (lsr number temp))))
     (inst b END)
     LEFT
-    (inst cmp temp sb!vm:n-word-bits)
-    (inst mov :gt temp sb!vm:n-word-bits)
+    (inst cmp temp n-word-bits)
+    (inst mov :gt temp n-word-bits)
     (inst mov result (lsl number temp))
     END))
 
@@ -386,8 +384,8 @@
                 (:policy :fast-safe)
                 (:generator ,cost
                   (move temp amount)
-                  (inst cmp temp sb!vm:n-word-bits)
-                  (inst mov :gt temp sb!vm:n-word-bits)
+                  (inst cmp temp n-word-bits)
+                  (inst mov :gt temp n-word-bits)
                   (inst mov result (lsl number temp))))))
   ;; FIXME: There's the opportunity for a sneaky optimization here, I
   ;; think: a FAST-ASH-LEFT-C/FIXNUM=>SIGNED vop.  -- CSR, 2003-09-03
@@ -399,7 +397,6 @@
              fast-ash-left/unsigned=>unsigned)
   (:translate ash-left-mod32))
 
-#!+ash-right-vops
 (define-vop (fast-%ash/right/unsigned)
   (:translate %ash/right)
   (:policy :fast-safe)
@@ -411,7 +408,6 @@
   (:generator 4
     (inst mov result (lsr number amount))))
 
-#!+ash-right-vops
 (define-vop (fast-%ash/right/signed)
   (:translate %ash/right)
   (:policy :fast-safe)
@@ -423,7 +419,6 @@
   (:generator 4
     (inst mov result (asr number amount))))
 
-#!+ash-right-vops
 (define-vop (fast-%ash/right/fixnum)
   (:translate %ash/right)
   (:policy :fast-safe)
@@ -549,8 +544,6 @@
 
 (define-vop (fast-conditional)
   (:conditional :eq)
-  (:effects)
-  (:affected)
   (:policy :fast-safe))
 
 (define-vop (fast-conditional/fixnum fast-conditional)
@@ -869,7 +862,6 @@
   (:generator 1
     (inst umull lo hi x y)))
 
-#!+multiply-high-vops
 (define-vop (mulhi)
   (:translate %multiply-high)
   (:policy :fast-safe)
@@ -882,7 +874,6 @@
   (:generator 20
     (inst umull lo hi x y)))
 
-#!+multiply-high-vops
 (define-vop (mulhi/fx)
   (:translate %multiply-high)
   (:policy :fast-safe)
@@ -952,26 +943,3 @@
   (:translate sb!bignum:%ashl)
   (:generator 1
     (inst mov result (lsl digit count))))
-
-;;;; Static functions.
-
-(define-static-fun two-arg-gcd (x y) :translate gcd)
-(define-static-fun two-arg-lcm (x y) :translate lcm)
-
-(define-static-fun two-arg-+ (x y) :translate +)
-(define-static-fun two-arg-- (x y) :translate -)
-(define-static-fun two-arg-* (x y) :translate *)
-(define-static-fun two-arg-/ (x y) :translate /)
-
-(define-static-fun two-arg-< (x y) :translate <)
-(define-static-fun two-arg-> (x y) :translate >)
-(define-static-fun two-arg-= (x y) :translate =)
-
-(define-static-fun two-arg-and (x y) :translate logand)
-(define-static-fun two-arg-ior (x y) :translate logior)
-(define-static-fun two-arg-xor (x y) :translate logxor)
-(define-static-fun two-arg-eqv (x y) :translate logeqv)
-
-(define-static-fun eql (x y) :translate eql)
-
-(define-static-fun %negate (x) :translate %negate)
