@@ -14,17 +14,17 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(*condition-name-vec* conditional-opcode emit-word
+  (import '(conditional-opcode emit-word
             composite-immediate-instruction encodable-immediate
-            lsl lsr asr ror cpsr @) 'sb!vm)
+            lsl lsr asr ror cpsr @) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(sb!vm::nil-value sb!vm::registers sb!vm::null-tn sb!vm::null-offset
             sb!vm::pc-tn sb!vm::pc-offset sb!vm::code-offset)))
 
-(setf *disassem-inst-alignment-bytes* 4)
+(defconstant +disassem-inst-alignment-bytes+ 4)
 
 
-(defparameter *conditions*
+(defconstant-eqx +conditions+
   '((:eq . 0)
     (:ne . 1)
     (:cs . 2) (:hs . 2)
@@ -39,13 +39,14 @@
     (:lt . 11)
     (:gt . 12)
     (:le . 13)
-    (:al . 14)))
-(defparameter *condition-name-vec*
-  (let ((vec (make-array 16 :initial-element nil)))
-    (dolist (cond *conditions*)
-      (when (null (aref vec (cdr cond)))
-        (setf (aref vec (cdr cond)) (car cond))))
-    vec))
+    (:al . 14))
+  #'equal)
+(defconstant-eqx sb!vm::+condition-name-vec+
+  #.(let ((vec (make-array 16 :initial-element nil)))
+      (dolist (cond +conditions+ vec)
+        (when (null (aref vec (cdr cond)))
+          (setf (aref vec (cdr cond)) (car cond)))))
+  #'equalp)
 
 ;;; Set assembler parameters. (In CMU CL, this was done with
 ;;; a call to a macro DEF-ASSEMBLER-PARAMS.)
@@ -53,7 +54,7 @@
   (setf sb!assem:*assem-scheduler-p* nil))
 
 (defun conditional-opcode (condition)
-  (cdr (assoc condition *conditions* :test #'eq)))
+  (cdr (assoc condition +conditions+ :test #'eq)))
 
 ;;;; disassembler field definitions
 
@@ -354,7 +355,7 @@
   (let ((internal-emitter (gensym)))
     `(flet ((,internal-emitter ,arglist
               ,@body))
-       (if (assoc (car ,argvar) *conditions*)
+       (if (assoc (car ,argvar) +conditions+)
            (apply #',internal-emitter ,argvar)
            (apply #',internal-emitter :al ,argvar)))))
 
@@ -388,11 +389,11 @@
 
 (define-instruction simple-fun-header-word (segment)
   (:emitter
-   (emit-header-data segment simple-fun-header-widetag)))
+   (emit-header-data segment simple-fun-widetag)))
 
 (define-instruction lra-header-word (segment)
   (:emitter
-   (emit-header-data segment return-pc-header-widetag)))
+   (emit-header-data segment return-pc-widetag)))
 
 ;;;; Addressing mode 1 support
 
@@ -1115,7 +1116,7 @@
                                                                         :down
                                                                         :up)
                                                                     mode))
-                                            (tn-offset base) (tn-offset data)
+                                            pc-offset (tn-offset data)
                                             (ldb (byte 4 4) absolute-delta)
                                             opcode2 absolute-delta)))))
            ((integerp offset)
@@ -1186,7 +1187,7 @@
         (inst ldr temp (@ lip (- other-pointer-lowtag)))
         ;; And finally we use the header value (a count in words),
         ;; plus the fact that the top two bits of the widetag are
-        ;; clear (SIMPLE-FUN-HEADER-WIDETAG is #x2A and
+        ;; clear (SIMPLE-FUN-WIDETAG is #x2A and
         ;; RETURN-PC-HEADER-WIDETAG is #x36) to compute the boxed
         ;; address of the code component.
         (inst sub code lip (lsr temp (- 8 word-shift))))))))
