@@ -59,6 +59,16 @@
   (:results (res :scs (any-reg descriptor-reg)))
   (:generator 1
     (load-symbol-value res *binding-stack-pointer*)))
+
+(define-vop (current-nsp)
+  (:results (res :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move res nsp-tn)))
+
+(define-vop (set-nsp)
+  (:args (nsp :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move nsp-tn nsp)))
 
 ;;;; Unwind block hackery:
 
