@@ -85,22 +85,29 @@
     (inst asr temp temp n-widetag-bits)
     (inst sub temp temp (1- array-dimensions-offset))
     (inst lsl res temp n-fixnum-tag-bits)))
+
 ;;;; Bounds checking routine.
 (define-vop (check-bound)
   (:translate %check-bound)
   (:policy :fast-safe)
   (:args (array :scs (descriptor-reg))
          (bound :scs (any-reg descriptor-reg))
-         (index :scs (any-reg descriptor-reg) :target result))
-  (:results (result :scs (any-reg descriptor-reg)))
+         (index :scs (any-reg descriptor-reg)))
+  (:variant-vars %test-fixnum)
+  (:variant t)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 5
     (let ((error (generate-error-code vop 'invalid-array-index-error array bound index)))
+      (when %test-fixnum
+        (%test-fixnum index error t))
       (inst cmp index bound)
-      (inst b :hs error)
-      (move result index))))
+      (inst b :hs error))))
 
+(define-vop (check-bound/fast check-bound)
+  (:policy :fast)
+  (:variant nil)
+  (:variant-cost 4))
 ;;;; Accessors/Setters
 
 ;;; Variants built on top of word-index-ref, etc.  I.e. those vectors whos
