@@ -20,10 +20,6 @@
 ;;; address space)
 (defconstant n-machine-word-bits 64)
 
-;;; number of bits per byte where a byte is the smallest addressable
-;;; object
-(defconstant n-byte-bits 8)
-
 ;;; Floating-point related constants, both format descriptions and FPU
 ;;; control register descriptions.  These don't exactly match up with
 ;;; what the machine manuals say because the Common Lisp standard
@@ -90,10 +86,8 @@
   (defconstant linkage-table-space-end   #x0b000000)
   #!+linux
   (progn
-    (defconstant dynamic-0-space-start #x4f000000)
-    (defconstant dynamic-0-space-end   #x66fff000)
-    (defconstant dynamic-1-space-start #x67000000)
-    (defconstant dynamic-1-space-end   #x7efff000)))
+    (defparameter dynamic-0-space-start #x4f000000)
+    (defparameter dynamic-0-space-end   #x66fff000)))
 
 #!+gencgc
 (!gencgc-space-setup #xF0000000 :dynamic-space-start #x1000000000)
@@ -105,13 +99,13 @@
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   single-step-around-trap
   single-step-before-trap
-  invalid-arg-count-trap)
+  invalid-arg-count-trap
+  error-trap)
 
 ;;;; Static symbols.
 
@@ -123,32 +117,25 @@
 ;;; space directly after the static symbols.  That way, the raw-addr
 ;;; can be loaded directly out of them by indirecting relative to NIL.
 ;;;
-(defparameter *static-symbols*
-  (append
-   #!-sb-thread
-   '(*binding-stack-pointer*
-     *pseudo-atomic-atomic*
-     *pseudo-atomic-interrupted*)
-   '(*allocation-pointer*
+(defconstant-eqx +static-symbols+
+ `#(#!-sb-thread
+    ,@'(*binding-stack-pointer*
+        *pseudo-atomic-atomic*
+        *pseudo-atomic-interrupted*)
+    *allocation-pointer*
      ;; interrupt handling
+     ,@+common-static-symbols+)
+  #'equalp)
 
-
-     ;; Needed for callbacks to work across saving cores. see
-     ;; ALIEN-CALLBACK-ASSEMBLER-WRAPPER in c-call.lisp for gory
-     ;; details.
-     sb!alien::*enter-alien-callback*
-     #!+gencgc *restart-lisp-function*)
-   *common-static-symbols*
-   *c-callable-static-symbols*))
-
-(defparameter *static-funs*
-  '(two-arg-gcd two-arg-lcm
+(defconstant-eqx +static-fdefns+
+  #(two-arg-gcd two-arg-lcm
     two-arg-+ two-arg-- two-arg-* two-arg-/
     two-arg-< two-arg-> two-arg-=
     two-arg-and two-arg-ior two-arg-xor two-arg-eqv
 
     eql
-    sb!kernel:%negate))
+    sb!kernel:%negate)
+  #'equalp)
 
 
 ;;;; Assembler parameters:
