@@ -290,14 +290,18 @@
                    (let* ((arg (nth nth-arg deps))
                           (key-nth (getf options :key))
                           (key (and key-nth (nth key-nth deps)))
-                          (key-return-type (and key
-                                                (multiple-value-bind (type name) (lvar-fun-type key)
-                                                  (cond ((eq name 'identity)
-                                                         nil)
-                                                        ((fun-type-p type)
-                                                         (single-value-type (fun-type-returns type)))
-                                                        (t
-                                                         *universal-type*))))))
+                          (key-return-type (cond ((not key)
+                                                  nil)
+                                                 ((lvar-p key)
+                                                  (multiple-value-bind (type name) (lvar-fun-type key)
+                                                    (cond ((eq name 'identity)
+                                                           nil)
+                                                          ((fun-type-p type)
+                                                           (single-value-type (fun-type-returns type)))
+                                                          (t
+                                                           *universal-type*))))
+                                                 (t
+                                                  *universal-type*))))
                      (cond (key-return-type)
                            ((getf options :sequence)
                             (sequence-element-type arg))
