@@ -363,7 +363,9 @@ and no value was provided for it." name))))))))))
   (if (lambda-p functional)
       (make-fun-type
        :required (mapcar #'leaf-type (lambda-vars functional))
-       :returns (tail-set-type (lambda-tail-set functional)))
+       :returns (if (eq (functional-kind functional) :deleted)
+                    *empty-type*
+                    (tail-set-type (lambda-tail-set functional))))
       (let ((rest nil))
         (collect ((req)
                   (opt)
@@ -845,7 +847,6 @@ and no value was provided for it." name))))))))))
            ;; an automatic check.
            (let ((policy (lexenv-policy (functional-lexenv functional))))
              (when (and return
-                        (policy policy (> type-check 0))
                         (or (eq really-assert t)
                             (not (member :result (cdr really-assert)))))
                (assert-lvar-type (return-result return) type-returns
