@@ -908,7 +908,12 @@ One of :WARN, :ERROR or :NONE.")
      (format stream "~S {~A}" (leaf-debug-name leaf) (global-var-kind leaf)))
     (functional
      (format stream "~S ~S ~S" (type-of leaf) (functional-debug-name leaf)
-             (mapcar #'leaf-debug-name (lambda-vars leaf))))))
+             (mapcar #'leaf-debug-name
+                     (typecase leaf
+                       (clambda
+                        (lambda-vars leaf))
+                       (optional-dispatch
+                        (optional-dispatch-arglist leaf))))))))
 
 ;;; Attempt to find a block given some thing that has to do with it.
 (declaim (ftype (sfunction (t) cblock) block-or-lose))
