@@ -106,6 +106,7 @@
 (defvar *current-path*)
 (defvar *current-component*)
 (defvar *delayed-ir1-transforms*)
+#!+sb-dyncount
 (defvar *dynamic-counts-tn*)
 (defvar *elsewhere*)
 (defvar *elsewhere-label*)
@@ -155,9 +156,6 @@ the stack without triggering overflow protection.")
   `(sb!thread:with-recursive-lock (**world-lock**)
      ,@body))
 
-(declaim (type fixnum *compiler-sset-counter*))
-(defvar *compiler-sset-counter* 0)
-
 ;;; unique ID for the next object created (to let us track object
 ;;; identity even across GC, useful for understanding weird compiler
 ;;; bugs where something is supposed to be unique but is instead
@@ -310,6 +308,10 @@ the stack without triggering overflow protection.")
 ;;; compiler debugging, rather than policy control.)
 (defvar *assembly-optimize* t)
 
+;;; Bound during eval-when :compile-time evaluation.
+(defvar *compile-time-eval* nil)
+(declaim (always-bound *compile-time-eval*))
+
 (in-package "SB!ALIEN")
 
 ;;; Information describing a heap-allocated alien.
