@@ -10,20 +10,30 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-;;; Given the presence of docstrings and source locations,
-;;; this logic arguably belongs to the runtime kernel, not the compiler,
-;;; but such nuance isn't hugely important.
-(in-package "SB!C")
+(in-package "SB!IMPL")
 
-(declaim (ftype (function (t t t) (values t t &optional)) info)
-         (ftype (function (t t t) (values t &optional)) clear-info)
-         (ftype (function (t t t t) (values t &optional)) (setf info)))
+;;; Similar to FUNCTION, but the result type is "exactly" specified:
+;;; if it is an object type, then the function returns exactly one
+;;; value, if it is a short form of VALUES, then this short form
+;;; specifies the exact number of values.
+(def!type sfunction (args &optional result)
+  (let ((result (cond ((eq result '*) '*)
+                      ((or (atom result)
+                           (not (eq (car result) 'values)))
+                       `(values ,result &optional))
+                      ((intersection (cdr result) sb!xc:lambda-list-keywords)
+                       result)
+                      (t `(values ,@(cdr result) &optional)))))
+    `(function ,args ,result)))
+
+(declaim (ftype (sfunction (t t t) (values t t)) info)
+         (ftype (sfunction (t t t) t) clear-info)
+         (ftype (sfunction (t t t t) t) (setf info)))
 
 ;;; (:FUNCTION :TYPE) information is extracted through a wrapper.
 ;;; The globaldb representation is not necessarily literally a CTYPE.
 #-sb-xc-host
-(declaim (ftype (function (t) (values ctype boolean &optional))
-                proclaimed-ftype))
+(declaim (ftype (sfunction (t) (values ctype boolean)) proclaimed-ftype))
 
 ;;; At run time, we represent the type of a piece of INFO in the globaldb
 ;;; by a small integer between 1 and 63.  [0 is reserved for internal use.]
@@ -37,7 +47,8 @@
 ;;; The reverse mapping is obtained by reading the META-INFO.
 (declaim (type (simple-vector #.(ash 1 info-number-bits)) *info-types*))
 (!defglobal *info-types*
-            (make-array (ash 1 info-number-bits) :initial-element nil))
+            ;; Must be dumped as a literal for cold-load.
+            #.(make-array (ash 1 info-number-bits) :initial-element nil))
 
 (defstruct (meta-info
             (:constructor
@@ -72,9 +83,27 @@
 ;; Refer to info-vector.lisp for the meaning of this constant.
 (defconstant +no-auxilliary-key+ 0)
 
+;; Return the globaldb info for SYMBOL. With respect to the state diagram
+;; presented at the definition of SYMBOL-PLIST, if the object in SYMBOL's
+;; info slot is LISTP, it is in state 1 or 3. Either way, take the CDR.
+;; Otherwise, it is in state 2 so return the value as-is.
+;; In terms of this function being named "-vector", implying always a vector,
+;; it is understood that NIL is a proxy for +NIL-PACKED-INFOS+, a vector.
+;;
+#-sb-xc-host
+(progn
+ ;; Don't inline this if a vop translates it. Inlining occurs first,
+ ;; causing the vop not to be used.
+ #!-(vop-translates sb!kernel:symbol-info-vector) (declaim (inline symbol-info-vector))
+ (defun symbol-info-vector (symbol)
+  (let ((info-holder (symbol-info symbol)))
+    (truly-the (or null simple-vector)
+               (if (listp info-holder) (cdr info-holder) info-holder)))))
+
 ;;; SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
 ;;; But in the host Lisp, there is no such thing as a symbol-info slot.
 ;;; Instead, symbol-info is kept in the host symbol's plist.
+;;; This must be a SETFable place.
 #+sb-xc-host
 (defmacro symbol-info-vector (symbol) `(get ,symbol :sb-xc-globaldb-info))
 
@@ -226,6 +255,7 @@
 ;;;; functions and VOPs. To save space and allow for quick set
 ;;;; operations, we represent the attributes as bits in a fixnum.
 
+(in-package "SB!C")
 (deftype attributes () 'fixnum)
 
 ;;; Given a list of attribute names and an alist that translates them
@@ -258,21 +288,18 @@
         (test-name (symbolicate name "-ATTRIBUTEP")))
     `(progn
        (defmacro ,constructor (&rest attribute-names)
-         #!+sb-doc
          "Automagically generated boolean attribute creation function.
   See !DEF-BOOLEAN-ATTRIBUTE."
          (encode-attribute-mask attribute-names ,vector))
        (defun ,(symbolicate "DECODE-" name "-ATTRIBUTES") (attributes)
          (decode-attribute-mask attributes ,vector))
        (defmacro ,test-name (attributes &rest attribute-names)
-         #!+sb-doc
          "Automagically generated boolean attribute test function.
   See !DEF-BOOLEAN-ATTRIBUTE."
          `(logtest (the attributes ,attributes)
                    (,',constructor ,@attribute-names)))
        (define-setf-expander ,test-name (place &rest attributes
                                                &environment env)
-         #!+sb-doc
          "Automagically generated boolean attribute setter. See
  !DEF-BOOLEAN-ATTRIBUTE."
          (multiple-value-bind (temps values stores setter getter)
