@@ -50,15 +50,25 @@
                                        (or doc xrefs))))
     (when (policy bind (>= debug 1))
       (let ((args (functional-arg-documentation internal-fun)))
-        (aver (not (eq args :unspecified)))
         ;; When the component is dumped, the arglists of the entry
         ;; points will be dumped.  If they contain values that need
         ;; make-load-form processing then we need to do it now (bug
         ;; 310132).
         (setf (entry-info-arguments info)
-              (constant-value
-               (find-constant (strip-lambda-list args :arglist)))))
-      (setf (entry-info-type info) (type-specifier (leaf-type internal-fun)))))
+              (constant-value (find-constant args))))
+      ;; Arguably we should not parse/unparse if the type was obtained from
+      ;; a proclamation. On the other hand, this preserves exact semantics
+      ;; if a later DEFTYPE changes something. Be that as it may, storing
+      ;; just <X> instead of (VALUES <X> &OPTIONAL) saves 6 words per entry.
+      (let ((spec (type-specifier (leaf-type internal-fun)))
+            (result))
+        (setf (entry-info-type info)
+              (if (and (listp spec)
+                       (typep (setq result (third spec))
+                              '(cons (eql values)
+                                     (cons t (cons (eql &optional) null)))))
+                  `(sfunction ,(cadr spec) ,(cadr result))
+                  spec)))))
   (values))
 
 ;;; Replace all references to COMPONENT's non-closure XEPs that appear
@@ -90,8 +100,12 @@
                         :%source-name (functional-%source-name ef)
                         :%debug-name (functional-%debug-name ef)
                         :lexenv (make-null-lexenv)))
-                  (closure (physenv-closure
-                            (lambda-physenv (main-entry ef)))))
+                  (main-entry (main-entry ef))
+                  (closure (and
+                            ;; It may have been deleted due to none of
+                            ;; the optional entries reaching it.
+                            (neq (functional-kind main-entry) :deleted)
+                            (physenv-closure (lambda-physenv main-entry)))))
              (dolist (ref (leaf-refs lambda))
                (let ((ref-component (node-component ref)))
                  (cond ((eq ref-component component))
