@@ -91,7 +91,8 @@
   ;; constant, then the arguments will be swapped.
   commutative)
 
-(defstruct (fun-info #-sb-xc-host (:pure t))
+(defstruct (fun-info #-sb-xc-host (:pure t)
+                     (:copier nil))
   ;; boolean attributes of this function.
   (attributes (missing-arg) :type attributes)
   ;; TRANSFORM structures describing transforms for this function
@@ -108,9 +109,6 @@
   ;; further optimiz'ns) is backwards from the return convention for
   ;; transforms. -- WHN 19990917
   (optimizer nil :type (or function null))
-  ;; a function computing the constant or literal arguments which are
-  ;; destructively modified by the call.
-  (destroyed-constant-args nil :type (or function null))
   ;; If true, a special-case LTN annotation method that is used in
   ;; place of the standard type/policy template selection. It may use
   ;; arbitrary code to choose a template, decide to do a full call, or
@@ -159,15 +157,9 @@
   ;; If non-null, the index of the argument which becomes the result
   ;; of the function.
   (result-arg nil :type (or index null))
-  ;; For functions with attributes FOLDABLE & CALL check that the
-  ;; arguments declared as CALLABLE or FUNCTION are foldable as well.
-  (foldable-call-check nil :type (or function null))
-  ;; A function that is called with lvars to check that the functions
-  ;; passed to CALLABLE arguments have the right argument counts.
-  (callable-check nil :type (or function null))
   ;; Customizing behavior of ASSERT-CALL-TYPE
   (call-type-deriver nil :type (or function null))
-  (functional-args nil :type (or function null)))
+  annotation)
 
 (defprinter (fun-info)
   (attributes :test (not (zerop attributes))
