@@ -161,7 +161,6 @@
     *alloc-signal*
     sb!sys:*interrupt-pending*
     #!+sb-thruption sb!sys:*thruption-pending*
-    #!+sb-thruption sb!kernel:*restart-clusters*
     *in-without-gcing*
     *gc-inhibit*
     *gc-pending*
@@ -214,7 +213,7 @@
     #!+sb-dynamic-core +required-foreign-symbols+
 
     ;; List of Lisp specials bindings made by create_thread_struct()
-    ;; other than the per-thread-c-interface-symbols.
+    ;; excluding the names in !PER-THREAD-C-INTERFACE-SYMBOLS.
     sb!thread::*thread-initial-bindings*
 
     ;;; The following symbols aren't strictly required to be static
@@ -244,7 +243,5 @@
   (defconstant +highest-normal-generation+ 5)
   (defconstant +pseudo-static-generation+ 6))
 
-(defun !unintern-symbols ()
-  '("SB-VM"
-    +c-callable-fdefns+
-    +common-static-symbols+))
+(push '("SB-VM" +c-callable-fdefns+ +common-static-symbols+)
+      sb!impl::*!removable-symbols*)
