@@ -21,7 +21,7 @@
               initial-element-default
               n-bits
               primitive-type-name
-              &key (n-pad-elements 0) complex-typecode (importance 0) fixnum-p
+              &key (n-pad-elements 0) complex-typecode fixnum-p
               &aux (typecode
                     (symbol-value (symbolicate primitive-type-name "-WIDETAG")))))
             (:copier nil))
@@ -51,52 +51,40 @@
   ;; low level hackery (e.g., one element for arrays of BASE-CHAR,
   ;; which is used for a fixed #\NULL so that when we call out to C
   ;; we don't need to cons a new copy)
-  (n-pad-elements (missing-arg) :type index :read-only t)
-  ;; the relative importance of this array type.  Previously used for
-  ;; determining the order of the TYPECASE in
-  ;; HAIRY-DATA-VECTOR-{REF,SET}; currently (as of 2013-09-18) unused.
-  (importance (missing-arg) :type fixnum :read-only t))
+  (n-pad-elements (missing-arg) :type index :read-only t))
 
 ;; Simulate DEFINE-LOAD-TIME-GLOBAL - always bound in the image
 ;; but not eval'd in the compiler.
-(defglobal *specialized-array-element-type-properties* nil)
-(setq *specialized-array-element-type-properties*
+(define-load-time-global *specialized-array-element-type-properties*
   (map 'simple-vector
        (lambda (args)
          (apply #'!make-saetp args))
        `(;; Erm.  Yeah.  There aren't a lot of things that make sense
          ;; for an initial element for (ARRAY NIL). -- CSR, 2002-03-07
          (nil #:mu 0 simple-array-nil
-              :complex-typecode #.complex-vector-nil-widetag
-              :importance 0)
+              :complex-typecode #.complex-vector-nil-widetag)
          #!-sb-unicode
          (character ,(code-char 0) 8 simple-base-string
                     ;; (SIMPLE-BASE-STRINGs are stored with an extra
                     ;; trailing #\NULL for convenience in calling out
                     ;; to C.)
                     :n-pad-elements 1
-                    :complex-typecode #.complex-base-string-widetag
-                    :importance 17)
+                    :complex-typecode #.complex-base-string-widetag)
          #!+sb-unicode
          (base-char ,(code-char 0) 8 simple-base-string
                     ;; (SIMPLE-BASE-STRINGs are stored with an extra
                     ;; trailing #\NULL for convenience in calling out
                     ;; to C.)
                     :n-pad-elements 1
-                    :complex-typecode #.complex-base-string-widetag
-                    :importance 17)
+                    :complex-typecode #.complex-base-string-widetag)
          #!+sb-unicode
          (character ,(code-char 0) 32 simple-character-string
                     :n-pad-elements 1
-                    :complex-typecode #.complex-character-string-widetag
-                    :importance 17)
-         (single-float 0.0f0 32 simple-array-single-float
-          :importance 6)
-         (double-float 0.0d0 64 simple-array-double-float
-          :importance 5)
+                    :complex-typecode #.complex-character-string-widetag)
+         (single-float 0.0f0 32 simple-array-single-float)
+         (double-float 0.0d0 64 simple-array-double-float)
          (bit 0 1 simple-bit-vector
-              :complex-typecode #.complex-bit-vector-widetag
-              :importance 16)
+              :complex-typecode #.complex-bit-vector-widetag)
          ;; KLUDGE: The fact that these UNSIGNED-BYTE entries come
          ;; before their SIGNED-BYTE partners is significant in the
          ;; implementation of the compiler; some of the cross-compiler
@@ -106,70 +94,47 @@
          ;; (UNSIGNED-BYTE 7) is SUBTYPEP (SIGNED-BYTE 8), so if we're
          ;; not careful we could get the wrong specialized array when
          ;; we try to FIND-IF, below. -- CSR, 2002-07-08
-         ((unsigned-byte 2) 0 2 simple-array-unsigned-byte-2
-                            :importance 15)
-         ((unsigned-byte 4) 0 4 simple-array-unsigned-byte-4
-                            :importance 14)
-         ((unsigned-byte 7) 0 8 simple-array-unsigned-byte-7
-                            :importance 13)
-         ((unsigned-byte 8) 0 8 simple-array-unsigned-byte-8
-          :importance 13)
-         ((unsigned-byte 15) 0 16 simple-array-unsigned-byte-15
-          :importance 12)
-         ((unsigned-byte 16) 0 16 simple-array-unsigned-byte-16
-          :importance 12)
+         ((unsigned-byte 2) 0 2 simple-array-unsigned-byte-2)
+         ((unsigned-byte 4) 0 4 simple-array-unsigned-byte-4)
+         ((unsigned-byte 7) 0 8 simple-array-unsigned-byte-7)
+         ((unsigned-byte 8) 0 8 simple-array-unsigned-byte-8)
+         ((unsigned-byte 15) 0 16 simple-array-unsigned-byte-15)
+         ((unsigned-byte 16) 0 16 simple-array-unsigned-byte-16)
          #!-64-bit
          ((unsigned-byte #.n-positive-fixnum-bits)
           0 32 simple-array-unsigned-fixnum
-          :importance 8
           :fixnum-p t)
-         ((unsigned-byte 31) 0 32 simple-array-unsigned-byte-31
-          :importance 11)
-         ((unsigned-byte 32) 0 32 simple-array-unsigned-byte-32
-          :importance 11)
+         ((unsigned-byte 31) 0 32 simple-array-unsigned-byte-31)
+         ((unsigned-byte 32) 0 32 simple-array-unsigned-byte-32)
          #!+64-bit
          ((unsigned-byte #.n-positive-fixnum-bits)
           0 64 simple-array-unsigned-fixnum
-          :importance 8
           :fixnum-p t)
          #!+64-bit
-         ((unsigned-byte 63) 0 64 simple-array-unsigned-byte-63
-          :importance 9)
+         ((unsigned-byte 63) 0 64 simple-array-unsigned-byte-63)
          #!+64-bit
-         ((unsigned-byte 64) 0 64 simple-array-unsigned-byte-64
-          :importance 9)
-         ((signed-byte 8) 0 8 simple-array-signed-byte-8
-          :importance 10)
-         ((signed-byte 16) 0 16 simple-array-signed-byte-16
-          :importance 9)
+         ((unsigned-byte 64) 0 64 simple-array-unsigned-byte-64)
+         ((signed-byte 8) 0 8 simple-array-signed-byte-8)
+         ((signed-byte 16) 0 16 simple-array-signed-byte-16)
          ;; KLUDGE: See the comment in PRIMITIVE-TYPE-AUX,
          ;; compiler/generic/primtype.lisp, for why this is FIXNUM and
          ;; not (SIGNED-BYTE 30)
          #!-64-bit
-         (fixnum 0 32 simple-array-fixnum
-          :importance 8
-          :fixnum-p t)
-         ((signed-byte 32) 0 32 simple-array-signed-byte-32
-          :importance 7)
+         (fixnum 0 32 simple-array-fixnum :fixnum-p t)
+         ((signed-byte 32) 0 32 simple-array-signed-byte-32)
          ;; KLUDGE: see above KLUDGE for the 32-bit case
          #!+64-bit
-         (fixnum 0 64 simple-array-fixnum
-          :importance 8
-          :fixnum-p t)
+         (fixnum 0 64 simple-array-fixnum :fixnum-p t)
          #!+64-bit
-         ((signed-byte 64) 0 64 simple-array-signed-byte-64
-          :importance 7)
+         ((signed-byte 64) 0 64 simple-array-signed-byte-64)
          ((complex single-float) #C(0.0f0 0.0f0) 64
-          simple-array-complex-single-float
-          :importance 3)
+          simple-array-complex-single-float)
          ((complex double-float) #C(0.0d0 0.0d0) 128
-          simple-array-complex-double-float
-          :importance 2)
+          simple-array-complex-double-float)
          #!+long-float
          ((complex long-float) #C(0.0l0 0.0l0) #!+x86 192 #!+sparc 256
-          simple-array-complex-long-float
-          :importance 1)
-         (t 0 #.n-word-bits simple-vector :importance 18))))
+          simple-array-complex-long-float)
+         (t 0 #.n-word-bits simple-vector))))
 
 ;; The compiler can see that the number of types that must be present in a
 ;; union of arrays to convert (OR (array t1) ... (array tN)) to (ARRAY *)
@@ -203,7 +168,7 @@
 (!defglobal sb!kernel::*specialized-array-element-types*
             '#.sb!kernel::*specialized-array-element-types*)
 
-(defvar *vector-without-complex-typecode-infos*
+(define-load-time-global *vector-without-complex-typecode-infos*
   #+sb-xc-host
   (loop for saetp across *specialized-array-element-type-properties*
         for specifier = (saetp-specifier saetp)
@@ -218,15 +183,6 @@
   #-sb-xc-host
   '#.*vector-without-complex-typecode-infos*)
 
-(def!struct (room-info) ; FIXME: find a better home for this
-  ;; the name of this type
-  (name nil :type symbol)
-  ;; kind of type (how to reconstitute an object)
-  (kind (missing-arg)
-        :type (member :other :tiny-other :closure :instance :list
-                      :code :vector-nil :weak-pointer)))
-(!set-load-form-method room-info (:xc))
-
 ;;; Return the shift amount needed to turn length into bits
 (defun saetp-n-bits-shift (saetp)
   (max (1- (integer-length (saetp-n-bits saetp)))
