@@ -341,6 +341,7 @@
   (:args (x :target y :scs (single-int-carg-reg) :load-if nil)
          (fp :scs (any-reg) :load-if (not (sc-is y single-int-carg-reg))))
   (:results (y :scs (single-int-carg-reg) :load-if nil))
+  (:ignore fp)
   (:generator 1
     (unless (location= x y)
       (error "Huh? why did it do that?"))))
@@ -377,7 +378,7 @@
           (inst ldw offset nfp y)
           (inst ldw (+ offset n-word-bytes) nfp
                 (make-wired-tn (primitive-type-or-lose 'unsigned-byte-32)
-                               (sc-number-or-lose 'unsigned-reg)
+                               unsigned-reg-sc-number
                                (+ 1 (tn-offset y))))
           (inst stw old1 offset nfp)
           (inst stw old2 (+ offset n-word-bytes) nfp)))
@@ -387,7 +388,7 @@
         (inst ldw (- (* (1+ double-float-value-slot) n-word-bytes)
                      other-pointer-lowtag) x
                   (make-wired-tn (primitive-type-or-lose 'unsigned-byte-32)
-                                 (sc-number-or-lose 'unsigned-reg)
+                                 unsigned-reg-sc-number
                                  (+ 1 (tn-offset y))))))))
 (define-move-vop move-to-double-int-reg
   :move (double-reg descriptor-reg) (double-int-carg-reg))
@@ -396,6 +397,7 @@
   (:args (x :target y :scs (double-int-carg-reg) :load-if nil)
          (fp :scs (any-reg) :load-if (not (sc-is y double-int-carg-reg))))
   (:results (y :scs (double-int-carg-reg) :load-if nil))
+  (:ignore fp)
   (:generator 2
     (unless (location= x y)
       (error "Huh? why did it do that?"))))
