@@ -13,7 +13,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(sb!vm::zero sb!vm::registers sb!vm::float-registers
             sb!vm::single-reg sb!vm::double-reg
@@ -137,7 +137,7 @@
 
 ;;;; Initial disassembler setup.
 
-(setf *disassem-inst-alignment-bytes* 4)
+(defconstant +disassem-inst-alignment-bytes+ 4)
 
 (defvar *disassem-use-lisp-reg-names* t)
 
@@ -433,23 +433,27 @@
 (defun break-control (chunk inst stream dstate)
   (declare (ignore inst))
   (flet ((nt (x) (if stream (note x dstate))))
-    (case (break-im5 chunk dstate)
-      (#.error-trap
-       (nt "Error trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.cerror-trap
-       (nt "Cerror trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.breakpoint-trap
-       (nt "Breakpoint trap"))
-      (#.pending-interrupt-trap
-       (nt "Pending interrupt trap"))
-      (#.halt-trap
-       (nt "Halt trap"))
-      (#.fun-end-breakpoint-trap
-       (nt "Function end breakpoint trap"))
-      (#.single-step-around-trap
-       (nt "Single step around trap")))))
+    (let ((trap (break-im5 chunk dstate)))
+     (case trap
+       (#.cerror-trap
+        (nt "Cerror trap")
+        (handle-break-args #'snarf-error-junk trap stream dstate))
+       (#.breakpoint-trap
+        (nt "Breakpoint trap"))
+       (#.pending-interrupt-trap
+        (nt "Pending interrupt trap"))
+       (#.halt-trap
+        (nt "Halt trap"))
+       (#.fun-end-breakpoint-trap
+        (nt "Function end breakpoint trap"))
+       (#.single-step-around-trap
+        (nt "Single step around trap"))
+       (#.error-trap
+        (nt "Error trap")
+        (handle-break-args (lambda (sap offset trap &optional length-only)
+                             (snarf-error-junk sap offset trap length-only nil))
+                           trap
+                           stream dstate))))))
 
 (define-instruction-format (system-inst 32)
   (op1 :field (byte 6 26) :value 0)
@@ -1563,14 +1567,14 @@
   (:cost 0)
   (:delay 0)
   (:emitter
-   (emit-header-data segment simple-fun-header-widetag)))
+   (emit-header-data segment simple-fun-widetag)))
 
 (define-instruction lra-header-word (segment)
   :pinned
   (:cost 0)
   (:delay 0)
   (:emitter
-   (emit-header-data segment return-pc-header-widetag)))
+   (emit-header-data segment return-pc-widetag)))
 
 
 (defun emit-compute-inst (segment vop src label temp dst calc)
