@@ -48,6 +48,15 @@
   (:generator 1
     (move bsp-tn res)))
 
+(define-vop (current-nsp)
+  (:results (res :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move res nsp-tn)))
+
+(define-vop (set-nsp)
+  (:args (nsp :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move nsp-tn nsp)))
 
 ;;;; Unwind block hackery:
 
@@ -61,11 +70,11 @@
   (:temporary (:scs (descriptor-reg)) temp)
   (:temporary (:scs (non-descriptor-reg)) ndescr)
   (:generator 22
-    (inst addi (* (tn-offset tn) n-word-bytes) cfp-tn block)
+    (inst ldo (* (tn-offset tn) n-word-bytes) cfp-tn block)
     (load-symbol-value temp *current-unwind-protect-block*)
-    (storew temp block unwind-block-current-uwp-slot)
-    (storew cfp-tn block unwind-block-current-cont-slot)
-    (storew code-tn block unwind-block-current-code-slot)
+    (storew temp block unwind-block-uwp-slot)
+    (storew cfp-tn block unwind-block-cfp-slot)
+    (storew code-tn block unwind-block-code-slot)
     (inst compute-lra-from-code code-tn entry-label ndescr temp)
     (storew temp block catch-block-entry-pc-slot)))
 
@@ -81,11 +90,11 @@
   (:temporary (:scs (descriptor-reg) :target block :to (:result 0)) result)
   (:temporary (:scs (non-descriptor-reg)) ndescr)
   (:generator 44
-    (inst addi (* (tn-offset tn) n-word-bytes) cfp-tn result)
+    (inst ldo (* (tn-offset tn) n-word-bytes) cfp-tn result)
     (load-symbol-value temp *current-unwind-protect-block*)
-    (storew temp result catch-block-current-uwp-slot)
-    (storew cfp-tn result catch-block-current-cont-slot)
-    (storew code-tn result catch-block-current-code-slot)
+    (storew temp result catch-block-uwp-slot)
+    (storew cfp-tn result catch-block-cfp-slot)
+    (storew code-tn result catch-block-code-slot)
     (inst compute-lra-from-code code-tn entry-label ndescr temp)
     (storew temp result catch-block-entry-pc-slot)
 
@@ -102,7 +111,7 @@
   (:args (tn))
   (:temporary (:scs (descriptor-reg)) new-uwp)
   (:generator 7
-    (inst addi (* (tn-offset tn) n-word-bytes) cfp-tn new-uwp)
+    (inst ldo (* (tn-offset tn) n-word-bytes) cfp-tn new-uwp)
     (store-symbol-value new-uwp *current-unwind-protect-block*)))
 
 
@@ -121,7 +130,7 @@
   (:translate %unwind-protect-breakup)
   (:generator 17
     (load-symbol-value block *current-unwind-protect-block*)
-    (loadw block block unwind-block-current-uwp-slot)
+    (loadw block block unwind-block-uwp-slot)
     (store-symbol-value block *current-unwind-protect-block*)))
 
 
