@@ -10,10 +10,6 @@
 ;;; address space)
 (defconstant n-machine-word-bits 32)
 
-;;; number of bits per byte where a byte is the smallest addressable
-;;; object
-(defconstant n-byte-bits 8)
-
 (defconstant float-sign-shift 31)
 
 (defconstant single-float-bias 126)
@@ -68,10 +64,8 @@
 (defconstant static-space-start    #x4e000000)
 (defconstant static-space-end      #x4fff0000)
 
-(defconstant dynamic-0-space-start   #x50000000)
-(defconstant dynamic-0-space-end     #x5fff0000)
-(defconstant dynamic-1-space-start   #x60000000)
-(defconstant dynamic-1-space-end     #x6fff0000)
+(defparameter dynamic-0-space-start  #x50000000)
+(defparameter dynamic-0-space-end    #x5fff0000)
 
 ); eval-when
 
@@ -94,14 +88,14 @@
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   single-step-breakpoint-trap
   single-step-around-trap
   single-step-before-trap
-  single-step-after-trap)
+  single-step-after-trap
+  error-trap)
 
 ;;;; Static symbols.
 
@@ -112,14 +106,12 @@
 ;;; The fdefn objects for the static functions are loaded into static
 ;;; space directly after the static symbols.  That way, the raw-addr
 ;;; can be loaded directly out of them by indirecting relative to NIL.
-(defparameter *static-symbols*
-  (append
-   *common-static-symbols*
-   *c-callable-static-symbols*
-   '()))
+(defconstant-eqx +static-symbols+
+ `#(,@+common-static-symbols+)
+  #'equalp)
 
-(defparameter *static-funs*
-  '(length
+(defconstant-eqx +static-fdefns+
+  #(length
     two-arg-+
     two-arg--
     two-arg-*
@@ -136,5 +128,6 @@
     two-arg-ior
     two-arg-xor
     two-arg-gcd
-    two-arg-lcm))
+    two-arg-lcm)
+  #'equalp)
 
