@@ -347,154 +347,6 @@
                                  ,important
                                  policy))))))
 
-;;;; DEFKNOWN and DEFOPTIMIZER
-
-;;; This macro should be the way that all implementation independent
-;;; information about functions is made known to the compiler.
-;;;
-;;; FIXME: The comment above suggests that perhaps some of my added
-;;; FTYPE declarations are in poor taste. Should I change my
-;;; declarations, or change the comment, or what?
-;;;
-;;; FIXME: DEFKNOWN is needed only at build-the-system time. Figure
-;;; out some way to keep it from appearing in the target system.
-;;;
-;;; Declare the function NAME to be a known function. We construct a
-;;; type specifier for the function by wrapping (FUNCTION ...) around
-;;; the ARG-TYPES and RESULT-TYPE. ATTRIBUTES is an unevaluated list
-;;; of boolean attributes of the function. See their description in
-;;; (!DEF-BOOLEAN-ATTRIBUTE IR1). NAME may also be a list of names, in
-;;; which case the same information is given to all the names. The
-;;; keywords specify the initial values for various optimizers that
-;;; the function might have.
-(defmacro defknown (name arg-types result-type &optional (attributes '(any))
-                    &body keys)
-  #-sb-xc-host
-  (when (member 'unsafe attributes)
-    (style-warn "Ignoring legacy attribute UNSAFE. Replaced by its inverse: DX-SAFE.")
-    (setf attributes (remove 'unsafe attributes)))
-  (when (and (intersection attributes '(any call unwind))
-             (intersection attributes '(movable)))
-    (error "function cannot have both good and bad attributes: ~S" attributes))
-
-  (when (member 'any attributes)
-    (setq attributes (union '(unwind) attributes)))
-  (when (member 'flushable attributes)
-    (pushnew 'unsafely-flushable attributes))
-  (multiple-value-bind (callable-map arg-types)
-      (make-callable-map arg-types attributes)
-    `(%defknown ',(if (and (consp name)
-                           (not (legal-fun-name-p name)))
-                      name
-                      (list name))
-                '(sfunction ,arg-types ,result-type)
-                (ir1-attributes ,@attributes)
-                (source-location)
-                :callable-map ,callable-map
-                ,@keys)))
-
-(defun make-callable-map (arg-types attributes)
-  (if (member 'call attributes)
-      (multiple-value-bind (llks required optional rest keys)
-          (parse-lambda-list
-           arg-types
-           :context :function-type
-           :accept (lambda-list-keyword-mask
-                    '(&optional &rest &key &allow-other-keys))
-           :silent t)
-        (let (vars
-              call-vars)
-          (labels ((callable-p (x)
-                     (member x '(function function-designator)))
-                   (process-var (x &optional (name (gensym)))
-                     (if (callable-p (if (consp x)
-                                         (car x)
-                                         x))
-                         (push (list* name
-                                      (cond ((consp x)
-                                             (cdr x))))
-                               call-vars)
-                         (push name vars))
-                     name)
-                   (process-type (type)
-                     (if (and (consp type)
-                              (callable-p (car type)))
-                         (car type)
-                         type)))
-            (let* (key-arg
-                   rest-var
-                   (lambda-list
-                     `(,@(mapcar #'process-var required)
-                       ,@(and optional
-                              `(&optional ,@(mapcar #'process-var optional)))
-                       ,@(and rest
-                              `(&rest ,(setf rest-var (process-var (car rest)))))
-                       ,@(and (ll-kwds-keyp llks)
-                              `(&key
-                                ,@(loop for (key type) in keys
-                                        for var = (gensym)
-                                        do (process-var type var)
-                                        collect (if (eq key :key)
-                                                    (let ((var-supplied-p (gensym)))
-                                                      (setf key-arg
-                                                            (cons var var-supplied-p))
-                                                      `((,key ,var) nil ,var-supplied-p))
-                                                    `((,key ,var)))))))))
-
-              (values
-               `(lambda (function ,@lambda-list)
-                  (declare (ignorable ,@vars))
-                  ;; Turn all the (nth-arg n . options) into (arg-lvar . options)
-                  ,@(loop for (lvar args results . options) in call-vars
-                          for rest-p = (typep (car (last args)) '(cons (eql rest-args)))
-                          collect
-                          (labels ((handle-keys (options)
-                                   `(list ,@(loop for (key value) on options by #'cddr
-                                                  collect key
-                                                  collect (if (eq key :key)
-                                                              (destructuring-bind (var . var-p)
-                                                                  key-arg
-                                                                `(and ,var-p
-                                                                      ,var))
-                                                              value))))
-                                   (process-arg (arg)
-                                     (if (consp arg)
-                                         (ecase (car arg)
-                                           (nth-arg
-                                            `(list* ,(nth (cadr arg) lambda-list)
-                                                    ,(handle-keys (cddr arg))))
-                                           (or
-                                            `(list* (list 'or ,@(mapcar #'process-arg (cdr arg))))))
-                                         `(load-time-value (values-specifier-type ',arg)))))
-                            `(and ,lvar
-                                  (funcall function
-                                           ,lvar
-                                           (list*
-                                            ,@(loop for arg in (if rest-p
-                                                                   (butlast args)
-                                                                   args)
-                                                    collect (process-arg arg))
-                                            ,(and rest-p
-                                                  `(loop with rest-options = ',(cdar (last args))
-                                                         for x in ,rest-var
-                                                         collect (list* x rest-options))))
-                                           ,(if results
-                                                (process-arg results)
-                                                '*wild-type*)
-                                           ,@options)))))
-               `(,@(mapcar #'process-type required)
-                 ,@(and optional
-                        `(&optional ,@(mapcar #'process-type optional)))
-                 ,@(and (ll-kwds-restp llks)
-                        `(&rest ,@rest))
-                 ,@(and (ll-kwds-keyp llks)
-                        `(&key
-                          ,@(loop for (key type) in keys
-                                  collect `(,key ,(process-type type)))))
-                 ,@(and (ll-kwds-allowp llks)
-                        '(&allow-other-keys))))))))
-      (values nil arg-types)))
-
 ;;; Create a function which parses combination args according to WHAT
 ;;; and LAMBDA-LIST, where WHAT is either a function name or a list
 ;;; (FUN-NAME KIND) and does some KIND of optimization.
