@@ -95,9 +95,10 @@
 ;;; :CONSTANT-SCS (SC*)
 ;;;   A list of the names of all the constant SCs that can be loaded into this
 ;;;   SC by a move function.
-(defmacro define-storage-class (name number sb-name &key (element-size '1)
-                                     (alignment '1) locations reserve-locations
-                                     save-p alternate-scs constant-scs)
+(defmacro !define-storage-class (name number sb-name &key (element-size '1)
+                                      (alignment '1) locations reserve-locations
+                                      save-p alternate-scs constant-scs
+                                      operand-size)
   (declare (type symbol name))
   (declare (type sc-number number))
   (declare (type symbol sb-name))
@@ -136,9 +137,11 @@
              t nil)))
     `(progn
        (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-         (let ((res (make-sc :name ',name :number ',number
+         (let ((res (make-storage-class
+                             :name ',name :number ',number
                              :sb (sb-or-lose ',sb-name)
                              :element-size ,element-size
+                             :operand-size ,operand-size
                              :alignment ,alignment
                              :locations (make-sc-locations ',locations)
                              :reserve-locations (make-sc-locations ',reserve-locations)
