@@ -13,7 +13,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(;; SBs, SCs, and TNs
             sb!vm::immediate-constant
@@ -42,7 +42,7 @@
     (error "~S isn't a floating-point register." tn))
   (tn-offset tn))
 
-;;;(sb!disassem:set-disassem-params :instruction-alignment 32)
+(defconstant +disassem-inst-alignment-bytes+ 4)
 
 (defvar *disassem-use-lisp-reg-names* t)
 
@@ -1054,18 +1054,12 @@
 (defun break-control (chunk inst stream dstate)
   (declare (ignore inst))
   (flet ((nt (x) (if stream (note x dstate))))
-    (when (= (break-code chunk dstate) 0)
-      (case (break-subcode chunk dstate)
+    (let ((trap (break-subcode chunk dstate)))
+      (case trap
         (#.halt-trap
          (nt "Halt trap"))
         (#.pending-interrupt-trap
          (nt "Pending interrupt trap"))
-        (#.error-trap
-         (nt "Error trap")
-         (handle-break-args #'snarf-error-junk stream dstate))
-        (#.cerror-trap
-         (nt "Cerror trap")
-         (handle-break-args #'snarf-error-junk stream dstate))
         (#.breakpoint-trap
          (nt "Breakpoint trap"))
         (#.fun-end-breakpoint-trap
@@ -1075,7 +1069,12 @@
         (#.single-step-around-trap
          (nt "Single step around trap"))
         (#.single-step-before-trap
-         (nt "Single step before trap"))))))
+         (nt "Single step before trap"))
+        (#.cerror-trap
+         (nt "Cerror trap")
+         (handle-break-args #'snarf-error-junk trap stream dstate))
+        (t
+         (handle-break-args #'snarf-error-junk trap stream dstate))))))
 
 (define-instruction break (segment code &optional (subcode 0))
   (:declare (type (unsigned-byte 10) code subcode))
@@ -1149,14 +1148,14 @@
   (:cost 0)
   (:delay 0)
   (:emitter
-   (emit-header-data segment simple-fun-header-widetag)))
+   (emit-header-data segment simple-fun-widetag)))
 
 (define-instruction lra-header-word (segment)
   :pinned
   (:cost 0)
   (:delay 0)
   (:emitter
-   (emit-header-data segment return-pc-header-widetag)))
+   (emit-header-data segment return-pc-widetag)))
 
 
 (defun emit-compute-inst (segment vop dst src label temp calc)
