@@ -110,7 +110,11 @@ This option has no effect without COMPUTE-DEBUG-FUN.")
 
 (define-optimization-quality compute-debug-fun
     debug
-  ("no" "minimal" "yes" "yes"))
+  ("minimal" "yes" "yes" "yes"))
+
+(define-optimization-quality eval-store-source-form
+    debug
+  ("no" "yes" "yes" "yes"))
 
 (define-optimization-quality preserve-single-use-debug-variables
     (if (and (>= debug 2)
