@@ -13,7 +13,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(;; SBs and SCs
             sb!vm::zero sb!vm::immediate-constant
@@ -22,6 +22,8 @@
             sb!vm::zero-tn sb!vm::lip-tn
             sb!vm::zero-offset sb!vm::null-offset)))
 
+(defconstant +disassem-inst-alignment-bytes+ sb!vm:n-word-bytes)
+
 ;;; needs a little more work in the assembler, to realise that the
 ;;; delays requested here are not mandatory, so that the assembler
 ;;; shouldn't fill gaps with NOPs but with real instructions.  -- CSR,
@@ -125,7 +127,7 @@
 (define-compiler-macro valid-bo-encoding (&whole form enc)
   (declare (notinline valid-bo-encoding))
   (if (keywordp enc) (valid-bo-encoding enc) form))
-(eval-when (:compile-toplevel :load-toplevel :execute)
+(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 (defun valid-bo-encoding (enc)
   (or (if (integerp enc)
         (and (= enc (logand #x1f enc))
@@ -1999,13 +2001,13 @@
   :pinned
   (:delay 0)
   (:emitter
-   (emit-header-data segment simple-fun-header-widetag)))
+   (emit-header-data segment simple-fun-widetag)))
 
 (define-instruction lra-header-word (segment)
   :pinned
   (:delay 0)
   (:emitter
-   (emit-header-data segment return-pc-header-widetag)))
+   (emit-header-data segment return-pc-widetag)))
 
 
 ;;;; Instructions for converting between code objects, functions, and lras.
