@@ -33,18 +33,18 @@
 (defun unimp-control (chunk inst stream dstate)
   (declare (ignore inst))
   (flet ((nt (x) (if stream (note x dstate))))
-    (case (xinstr-data chunk dstate)
-      (#.error-trap
-       (nt "Error trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.cerror-trap
-       (nt "Cerror trap")
-       (handle-break-args #'snarf-error-junk stream dstate))
-      (#.breakpoint-trap
-       (nt "Breakpoint trap"))
-      (#.pending-interrupt-trap
-       (nt "Pending interrupt trap"))
-      (#.halt-trap
-       (nt "Halt trap"))
-      (#.fun-end-breakpoint-trap
-       (nt "Function end breakpoint trap")))))
+    (let ((trap (xinstr-data chunk dstate)))
+     (case trap
+       (#.cerror-trap
+        (nt "Cerror trap")
+        (handle-break-args #'snarf-error-junk trap stream dstate))
+       (#.breakpoint-trap
+        (nt "Breakpoint trap"))
+       (#.pending-interrupt-trap
+        (nt "Pending interrupt trap"))
+       (#.halt-trap
+        (nt "Halt trap"))
+       (#.fun-end-breakpoint-trap
+        (nt "Function end breakpoint trap"))
+       (t
+        (handle-break-args #'snarf-error-junk trap stream dstate))))))
