@@ -107,11 +107,8 @@
   (defconstant static-space-start        #x28000000)
   (defconstant static-space-end          #x2c000000)
 
-  (defconstant dynamic-0-space-start #x30000000)
-  (defconstant dynamic-0-space-end   #x38000000)
-
-  (defconstant dynamic-1-space-start #x40000000)
-  (defconstant dynamic-1-space-end   #x48000000))
+  (defparameter dynamic-0-space-start #x30000000)
+  (defparameter dynamic-0-space-end   #x38000000))
 
 #!+(and sunos cheneygc) ; might as well start by trying the same numbers
 (progn
@@ -124,11 +121,8 @@
   (defconstant static-space-start        #x28000000)
   (defconstant static-space-end          #x2c000000)
 
-  (defconstant dynamic-0-space-start     #x30000000)
-  (defconstant dynamic-0-space-end       #x38000000)
-
-  (defconstant dynamic-1-space-start     #x40000000)
-  (defconstant dynamic-1-space-end       #x48000000))
+  (defparameter dynamic-0-space-start    #x30000000)
+  (defparameter dynamic-0-space-end      #x38000000))
 
 #!+(and netbsd cheneygc) ; Need a gap at 0x4000000 for shared libraries
 (progn
@@ -141,30 +135,25 @@
   (defconstant static-space-start        #x18000000)
   (defconstant static-space-end          #x1c000000)
 
-  (defconstant dynamic-0-space-start     #x48000000)
-  (defconstant dynamic-0-space-end       #x5ffff000)
-
-  (defconstant dynamic-1-space-start     #x60000000)
-  (defconstant dynamic-1-space-end       #x77fff000))
+  (defparameter dynamic-0-space-start    #x48000000)
+  (defparameter dynamic-0-space-end      #x5ffff000))
 
 ;; Size of one linkage-table entry in bytes. See comment in
 ;; src/runtime/sparc-arch.c
 (defconstant linkage-table-entry-size 16)
 
 
-;;;; other random constants.
-
 (defenum (:start 8)
   halt-trap
   pending-interrupt-trap
-  error-trap
   cerror-trap
   breakpoint-trap
   fun-end-breakpoint-trap
   after-breakpoint-trap
   single-step-around-trap
   single-step-before-trap
-  #!+gencgc allocation-trap)
+  #!+gencgc allocation-trap
+  error-trap)
 
 ;;;; static symbols.
 
