@@ -21,11 +21,10 @@
 ;;; "newer machines, such as the SuperSPARC and MicroSPARC based
 ;;; ones". Welcome to the 21st century... -- CSR, 2002-05-06
 (defun sanctify-for-execution (component)
-  (without-gcing
-   (alien-funcall (extern-alien "os_flush_icache"
+  (alien-funcall (extern-alien "os_flush_icache"
                                 (function void
                                           system-area-pointer
                                           unsigned-long))
                   (code-instructions component)
-                  (%code-code-size component)))
+                  (%code-code-size component))
   nil)
