@@ -13,15 +13,6 @@
 
 ;;;; type frobbing VOPs
 
-(define-vop (lowtag-of)
-  (:translate lowtag-of)
-  (:policy :fast-safe)
-  (:args (object :scs (any-reg descriptor-reg)))
-  (:results (result :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:generator 1
-    (inst and result object lowtag-mask)))
-
 (define-vop (widetag-of)
   (:translate widetag-of)
   (:policy :fast-safe)
@@ -94,7 +85,13 @@
   (:result-types positive-fixnum)
   (:generator 6
     (loadw res x 0 fun-pointer-lowtag)
-    (inst srl res res n-widetag-bits)))
+    (let* ((n-size-bits (integer-length short-header-max-words))
+           (lshift (- n-word-bits (+ n-size-bits n-widetag-bits))))
+      ;; To avoid constructing a 15-bit mask which would require another
+      ;; instruction, (ldb (byte n-size-bits n-widetag-bits) ...)
+      ;; is done as "shift left, shift right" discarding bits out both ends.
+      (inst sll res res lshift)
+      (inst srl res res (+ lshift n-widetag-bits)))))
 
 (define-vop (set-header-data)
   (:translate set-header-data)
@@ -112,7 +109,12 @@
        (inst sll t2 data (- n-widetag-bits n-fixnum-tag-bits))
        (inst or t1 t2))
       (immediate
-       (inst or t1 (ash (tn-value data) n-widetag-bits)))
+       (let ((val (ash (tn-value data) n-widetag-bits)))
+         (cond ((typep val '(signed-byte 13))
+                (inst or t1 val))
+               (t
+                (inst li t2 val)
+                (inst or t1 t2)))))
       (zero))
     (storew t1 x 0 other-pointer-lowtag)
     (move res x)))
