@@ -11,6 +11,135 @@
 
 (in-package "SB!VM")
 
+;;;; allocation helpers
+
+;;; Most allocation is done by inline code with sometimes help
+;;; from the C alloc() function by way of the alloc-tramp
+;;; assembly routine.
+
+(defun allocation-dynamic-extent (alloc-tn size lowtag)
+  (aver (not (location= alloc-tn rsp-tn)))
+  (inst sub rsp-tn size)
+  ;; see comment in x86/macros.lisp implementation of this
+  ;; However that comment seems inapplicable here because:
+  ;; - PAD-DATA-BLOCK quite clearly enforces double-word alignment,
+  ;;   contradicting "... unfortunately not enforced by ..."
+  ;; - It's not the job of FIXED-ALLOC to realign anything.
+  ;; - The real issue is that it's not obvious that the stack is
+  ;;   16-byte-aligned at *all* times. Maybe it is, maybe it isn't.
+  (inst and rsp-tn #.(lognot lowtag-mask))
+  (inst lea alloc-tn (make-ea :byte :base rsp-tn :disp lowtag))
+  (values))
+
+(defun allocation-tramp (result-tn size lowtag)
+  (cond ((typep size '(and integer (not (signed-byte 32))))
+         ;; MOV accepts large immediate operands, PUSH does not
+         (inst mov result-tn size)
+         (inst push result-tn))
+        (t
+         (inst push size)))
+  ;; This really would be better if it recognized TEMP-REG-TN as the "good" case
+  ;; rather than specializing on R11, which just happens to be the temp reg.
+  ;; But the assembly routine is hand-written, not generated, and it has to match,
+  ;; so there's not much that can be done to generalize it.
+  (let* ((to-r11 (location= result-tn r11-tn))
+         (fixup (make-fixup (if to-r11
+                                'alloc-tramp-r11
+                                'alloc-tramp) :assembly-routine)))
+    (inst call (cond #!+immobile-code
+                     (sb!c::*code-is-immobile* fixup)
+                     (t (inst mov result-tn fixup)
+                        result-tn)))
+    (unless to-r11
+      (inst pop result-tn)))
+
+  (when lowtag
+    (inst or (reg-in-size result-tn :byte) lowtag))
+  (values))
+
+;;; Emit code to allocate an object with a size in bytes given by
+;;; SIZE into ALLOC-TN. The size may be an integer of a TN.
+;;; NODE may be used to make policy-based decisions.
+;;; This function should only be used inside a pseudo-atomic section,
+;;; which to the degree needed should also cover subsequent initialization.
+(defun allocation (alloc-tn size node &optional dynamic-extent lowtag)
+  (declare (ignore node))
+  (when dynamic-extent
+    (allocation-dynamic-extent alloc-tn size lowtag)
+    (return-from allocation (values)))
+  (aver (and (not (location= alloc-tn temp-reg-tn))
+             (or (integerp size) (not (location= size temp-reg-tn)))))
+
+  #!+(and (not sb-thread) sb-dynamic-core)
+  ;; We'd need a spare reg in which to load boxed_region from the linkage table.
+  ;; Probably could use r12 since it's reserved for the thread base,
+  ;; but that seems like the wrong answer, because r12 should be available
+  ;; to the register allocator when not using it as the thread base.
+  ;; Could push/pop any random register on the stack and own it temporarily,
+  ;; but seeing as nobody cared about this, just punt.
+  (allocation-tramp alloc-tn size lowtag)
+
+  #!-(and (not sb-thread) sb-dynamic-core)
+  ;; Otherwise do the normal inline allocation thing
+  (let ((NOT-INLINE (gen-label))
+        (DONE (gen-label))
+        ;; Yuck.
+        (in-elsewhere (eq *elsewhere* sb!assem::**current-segment**))
+        ;; thread->alloc_region.free_pointer
+        (free-pointer
+         #!+sb-thread
+         (make-ea :qword :base thread-base-tn
+                  :disp (* n-word-bytes thread-alloc-region-slot))
+         #!-sb-thread
+         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign)))
+        ;; thread->alloc_region.end_addr
+        (end-addr
+         #!+sb-thread
+         (make-ea :qword :base thread-base-tn
+                  :disp (* n-word-bytes (1+ thread-alloc-region-slot)))
+         #!-sb-thread
+         (make-ea :qword :disp (make-fixup "gc_alloc_region" :foreign 8))))
+    (cond ((or in-elsewhere
+               ;; large objects will never be made in a per-thread region
+               (and (integerp size)
+                    (>= size large-object-size)))
+           (allocation-tramp alloc-tn size lowtag))
+          (t
+           (inst mov temp-reg-tn free-pointer)
+           (cond ((integerp size)
+                  (inst lea alloc-tn (make-ea :qword :base temp-reg-tn :disp size)))
+                 ((location= alloc-tn size)
+                  (inst add alloc-tn temp-reg-tn))
+                 (t
+                  (inst lea alloc-tn (make-ea :qword :base temp-reg-tn :index size))))
+           (inst cmp alloc-tn end-addr)
+           (inst jmp :a NOT-INLINE)
+           (inst mov free-pointer alloc-tn)
+           (emit-label DONE)
+           (if lowtag
+               (inst lea alloc-tn (make-ea :byte :base temp-reg-tn :disp lowtag))
+               (inst mov alloc-tn temp-reg-tn))
+           (assemble (*elsewhere*)
+             (emit-label NOT-INLINE)
+             (cond ((and (tn-p size) (location= size alloc-tn)) ; recover SIZE
+                    (inst sub alloc-tn free-pointer)
+                    (allocation-tramp temp-reg-tn alloc-tn nil))
+                   (t ; SIZE is intact
+                    (allocation-tramp temp-reg-tn size nil)))
+             (inst jmp DONE))))
+    (values)))
+
+;;; Allocate an other-pointer object of fixed SIZE with a single word
+;;; header having the specified WIDETAG value. The result is placed in
+;;; RESULT-TN.
+(defun fixed-alloc (result-tn widetag size node &optional stack-allocate-p)
+  (maybe-pseudo-atomic stack-allocate-p
+      (allocation result-tn (pad-data-block size) node stack-allocate-p
+                  other-pointer-lowtag)
+      (storew* (logior (ash (1- size) n-widetag-bits) widetag)
+               result-tn 0 other-pointer-lowtag
+               (not stack-allocate-p))))
+
 ;;;; CONS, LIST and LIST*
 (define-vop (list-or-list*)
   (:args (things :more t))
@@ -128,12 +257,13 @@
     (:results (result :scs (descriptor-reg) :from :load))
     (:arg-types positive-fixnum positive-fixnum positive-fixnum)
     (:policy :fast-safe)
+    (:node-var node)
     (:generator 100
       ;; The LET generates instructions that needn't be pseudoatomic
       ;; so don't move it inside.
       (let ((size (calc-size-in-bytes words result)))
         (pseudo-atomic
-         (allocation result size nil nil other-pointer-lowtag)
+         (allocation result size node nil other-pointer-lowtag)
          (put-header result type length t)))))
 
   (define-vop (allocate-vector-on-stack)
@@ -310,11 +440,11 @@
   (:results (result :scs (descriptor-reg) :from :argument))
   (:node-var node)
   (:generator 37
-    (with-fixed-allocation (result fdefn-widetag fdefn-size node)
-      (storew name result fdefn-name-slot other-pointer-lowtag)
-      (storew nil-value result fdefn-fun-slot other-pointer-lowtag)
-      (storew (make-fixup 'undefined-tramp :assembly-routine)
-              result fdefn-raw-addr-slot other-pointer-lowtag))))
+    (fixed-alloc result fdefn-widetag fdefn-size node)
+    (storew name result fdefn-name-slot other-pointer-lowtag)
+    (storew nil-value result fdefn-fun-slot other-pointer-lowtag)
+    (storew (make-fixup 'undefined-tramp :assembly-routine)
+            result fdefn-raw-addr-slot other-pointer-lowtag)))
 
 (define-vop (make-closure)
   ; (:args (function :to :save :scs (descriptor-reg)))
@@ -348,9 +478,8 @@
   (:info stack-allocate-p)
   (:node-var node)
   (:generator 10
-    (with-fixed-allocation
-        (result value-cell-widetag value-cell-size node stack-allocate-p)
-      (storew value result value-cell-value-slot other-pointer-lowtag))))
+    (fixed-alloc result value-cell-widetag value-cell-size node stack-allocate-p)
+    (storew value result value-cell-value-slot other-pointer-lowtag)))
 
 ;;;; automatic allocators for primitive objects
 
@@ -396,16 +525,32 @@
                  ;; XXX: should layout fixups use a name, not a layout object?
                  (make-fixup type :layout))))))))
 
+;;; Allocate a non-vector variable-length object.
+;;; Exactly 4 allocators are rendered via this vop:
+;;;  BIGNUM               (%ALLOCATE-BIGNUM)
+;;;  FUNCALLABLE-INSTANCE (%MAKE-FUNCALLABLE-INSTANCE)
+;;;  CLOSURE              (%COPY-CLOSURE)
+;;;  INSTANCE             (%MAKE-INSTANCE)
+;;; WORDS accounts for the mandatory slots *including* the header.
+;;; EXTRA is the variable payload, also measured in words.
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
   (:arg-types positive-fixnum)
   (:info name words type lowtag)
   (:ignore name)
   (:results (result :scs (descriptor-reg) :from (:eval 1)))
-  (:temporary (:sc any-reg :from :eval :to (:eval 1)) bytes)
-  (:temporary (:sc any-reg :from :eval :to :result) header)
+  (:temporary (:sc unsigned-reg :from :eval :to (:eval 1)) bytes)
+  (:temporary (:sc unsigned-reg :from :eval :to :result) header)
   (:node-var node)
   (:generator 50
+   ;; With the exception of bignums, these objects have effectively
+   ;; 32-bit headers because the high half contains other data.
+   (multiple-value-bind (bytes header extra)
+      (if (= type bignum-widetag)
+          (values bytes header extra)
+          (values (reg-in-size bytes  :dword)
+                  (reg-in-size header :dword)
+                  (reg-in-size extra  :dword)))
     (inst lea bytes
           (make-ea :qword :disp (* (1+ words) n-word-bytes) :index extra
                    :scale (ash 1 (- word-shift n-fixnum-tag-bits))))
@@ -414,17 +559,18 @@
     (inst lea header                    ; (w-1 << 8) | type
           (make-ea :qword :base header
                    :disp (+ (ash -2 n-widetag-bits) type)))
-    (inst and bytes (lognot lowtag-mask))
+    (inst and bytes (lognot lowtag-mask)))
     (pseudo-atomic
-     (allocation result bytes node)
-     (inst lea result (make-ea :byte :base result :disp lowtag))
+     (allocation result bytes node nil lowtag)
      (storew header result 0 lowtag))))
 
 #!+immobile-space
 (define-vop (alloc-fixedobj)
   (:info lowtag size word0 word1)
-  (:temporary (:sc unsigned-reg :to :eval :offset rdi-offset) c-arg1)
-  (:temporary (:sc unsigned-reg :to :eval :offset rsi-offset) c-arg2)
+  (:temporary (:sc unsigned-reg :to :eval
+               :offset #.(first *c-call-register-arg-offsets*)) c-arg1)
+  (:temporary (:sc unsigned-reg :to :eval
+               :offset #.(second *c-call-register-arg-offsets*)) c-arg2)
   (:temporary (:sc unsigned-reg :from :eval :to (:result 0) :offset rax-offset) c-result)
   (:results (result :scs (descriptor-reg)))
   (:generator 50
@@ -449,11 +595,13 @@
                 (:args ,@(if (>= argc 1) `((arg1 :scs ,(first arg-scs) :target c-arg1)))
                        ,@(if (>= argc 2) `((arg2 :scs ,(second arg-scs) :target c-arg2))))
                 ,@(if (>= argc 1)
-                      '((:temporary (:sc unsigned-reg :from (:argument 0)
-                                     :to :eval :offset rdi-offset) c-arg1)))
+                      `((:temporary (:sc unsigned-reg :from (:argument 0)
+                                     :to :eval :offset ,(first *c-call-register-arg-offsets*))
+                                    c-arg1)))
                 ,@(if (>= argc 2)
-                      '((:temporary (:sc unsigned-reg :from (:argument 1)
-                                     :to :eval :offset rsi-offset) c-arg2)))
+                      `((:temporary (:sc unsigned-reg :from (:argument 1)
+                                     :to :eval :offset ,(second *c-call-register-arg-offsets*))
+                                    c-arg2)))
                 (:temporary (:sc unsigned-reg :from :eval :to (:result 0)
                              :offset rax-offset) c-result)
                 (:results (result :scs (descriptor-reg)))
