@@ -271,42 +271,75 @@
   (:save-p t)
   (:generator 0
     (move rbx function)
-    ;; Current PC - don't rely on function to keep it in a form that
-    ;; GC understands
-    #!+sb-safepoint
-    (let ((label (gen-label)))
-      (inst lea (reg-in-size rax :immobile-code-pc) (make-fixup nil :code-object label))
-      (emit-label label)
-      (move-dword-if-immobile-code pc-save rax))
-    (when sb!c::*msan-compatible-stack-unpoison*
-      (inst mov rax (static-symbol-value-ea 'msan-param-tls))
-      ;; Unpoison parameters
-      (do ((n 0 (+ n n-word-bytes))
-           (arg args (tn-ref-across arg)))
-          ((null arg))
-        ;; KLUDGE: assume all parameters are 8 bytes or less
-        (inst fs)
-        (inst mov (make-ea :qword :base rax :disp n) 0)))
-    #!-win32
-    ;; ABI: AL contains amount of arguments passed in XMM registers
-    ;; for vararg calls.
+    (emit-c-call vop rax rbx args
+                 sb!alien::*alien-fun-type-varargs-default*
+                 #!+sb-safepoint pc-save)))
+
+;;; Calls to C can generally be made without loading a register
+;;; with the function. We receive the function name as an info argument.
+#!+sb-dynamic-core ;; broken when calling ldso-stubs
+(define-vop (call-out-named)
+  (:args (args :more t))
+  (:results (results :more t))
+  (:info c-symbol varargsp)
+  (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
+  #!+sb-safepoint
+  (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  (:ignore results)
+  (:vop-var vop)
+  (:save-p t)
+  (:generator 0
+    (emit-c-call vop rax c-symbol args varargsp #!+sb-safepoint pc-save)))
+
+(defun emit-c-call (vop rax fun args varargsp #!+sb-safepoint pc-save)
+  (declare (ignorable varargsp))
+  ;; Current PC - don't rely on function to keep it in a form that
+  ;; GC understands
+  #!+sb-safepoint
+  (let ((label (gen-label)))
+    (inst lea (reg-in-size rax :immobile-code-pc) (make-fixup nil :code-object label))
+    (emit-label label)
+    (move-dword-if-immobile-code pc-save rax))
+  (when sb!c::*msan-compatible-stack-unpoison*
+    (inst mov rax (static-symbol-value-ea 'msan-param-tls))
+    ;; Unpoison parameters
+    (do ((n 0 (+ n n-word-bytes))
+         (arg args (tn-ref-across arg)))
+        ((null arg))
+      ;; KLUDGE: assume all parameters are 8 bytes or less
+      (inst fs)
+      (inst mov (make-ea :qword :base rax :disp n) 0)))
+  #!-win32
+  ;; ABI: AL contains amount of arguments passed in XMM registers
+  ;; for vararg calls.
+  (when varargsp
     (move-immediate rax
-                    (loop for tn-ref = args then (tn-ref-across tn-ref)
-                          while tn-ref
-                          count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
-                                    'float-registers)))
-    #!+sb-safepoint
-    ;; Store SP in thread struct
-    (storew rsp-tn thread-base-tn thread-saved-csp-offset)
-    #!+win32 (inst sub rsp-tn #x20) ;MS_ABI: shadow zone
-    (inst call rbx)
-    #!+win32 (inst add rsp-tn #x20) ;MS_ABI: remove shadow space
-    #!+sb-safepoint
-    ;; Zero the saved CSP
-    (inst xor (make-ea-for-object-slot thread-base-tn thread-saved-csp-offset 0)
-          rsp-tn)
-    ;; To give the debugger a clue. XX not really internal-error?
-    (note-this-location vop :internal-error)))
+                  (loop for tn-ref = args then (tn-ref-across tn-ref)
+                        while tn-ref
+                        count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
+                                  'float-registers))))
+  #!+sb-safepoint
+  ;; Store SP in thread struct
+  (storew rsp-tn thread-base-tn thread-saved-csp-offset)
+  #!+win32 (inst sub rsp-tn #x20)       ;MS_ABI: shadow zone
+  ;; From immobile space we use the "CALL rel32" format to the linkage
+  ;; table jump, and from dynamic space we use "CALL [ea]" format
+  ;; where ea is the address of the linkage table entry's operand.
+  ;; So while the former is a jump to a jump, we can optimize out
+  ;; one jump in a statically linked executable.
+
+  (inst call (cond ((tn-p fun) fun)
+                   #!+immobile-code
+                   ((sb!c::code-immobile-p (sb!c::vop-node vop))
+                    (make-fixup fun :foreign))
+                   (t (make-ea :qword :disp (make-fixup fun :foreign 8)))))
+  ;; For the undefined alien error
+  (note-this-location vop :internal-error)
+  #!+win32 (inst add rsp-tn #x20)       ;MS_ABI: remove shadow space
+  #!+sb-safepoint
+  ;; Zero the saved CSP
+  (inst xor (make-ea-for-object-slot thread-base-tn thread-saved-csp-offset 0)
+        rsp-tn))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
