@@ -156,67 +156,3 @@
   (def-atomic %atomic-inc-cdr cell-xadd cons-cdr-slot)
   (def-atomic %atomic-dec-car cell-xsub cons-car-slot)
   (def-atomic %atomic-dec-cdr cell-xsub cons-cdr-slot))
-
-;;; SLOT-REF and SLOT-SET are used to define VOPs like CLOSURE-REF,
-;;; where the offset is constant at compile time, but varies for
-;;; different uses.
-(define-vop (slot-ref)
-  (:args (object :scs (descriptor-reg)))
-  (:results (value :scs (descriptor-reg any-reg)))
-  (:variant-vars base lowtag)
-  (:info offset)
-  (:generator 4
-    (loadw value object (+ base offset) lowtag)))
-(define-vop (slot-set)
-  (:args (object :scs (descriptor-reg))
-         (value :scs (descriptor-reg any-reg immediate)))
-  (:temporary (:sc unsigned-reg) temp)
-  (:variant-vars base lowtag)
-  (:info offset)
-  (:generator 4
-     (if (sc-is value immediate)
-         (move-immediate (make-ea :qword :base object
-                                         :disp (- (* (+ base offset) n-word-bytes)
-                                                  lowtag))
-                         (encode-value-if-immediate value)
-                         temp)
-         ;; Else, value not immediate.
-         (storew value object (+ base offset) lowtag))))
-
-(define-vop (slot-set-conditional)
-  (:args (object :scs (descriptor-reg) :to :eval)
-         (old-value :scs (descriptor-reg any-reg) :target eax)
-         (new-value :scs (descriptor-reg any-reg) :target temp))
-  (:temporary (:sc descriptor-reg :offset eax-offset
-                   :from (:argument 1) :to :result :target result)  eax)
-  (:temporary (:sc descriptor-reg :from (:argument 2) :to :result) temp)
-  (:variant-vars base lowtag)
-  (:results (result :scs (descriptor-reg)))
-  (:info offset)
-  (:generator 4
-    (move eax old-value)
-    (move temp new-value)
-    (inst cmpxchg (make-ea :dword :base object
-                           :disp (- (* (+ base offset) n-word-bytes) lowtag))
-          temp)
-    (move result eax)))
-
-;;; X86 special
-;;; FIXME: Figure out whether we should delete this.
-;;; It looks just like 'cell-xadd' and is buggy in the same ways:
-;;;   - modifies 'value' operand which *should* be in the same physical reg
-;;;     as 'result' operand, but would cause harm if not.
-;;;   - operand width needs to be :qword
-;;;   - :LOCK is missing
-(define-vop (slot-xadd)
-  (:args (object :scs (descriptor-reg) :to :result)
-         (value :scs (any-reg) :target result))
-  (:results (result :scs (any-reg) :from (:argument 1)))
-  (:result-types tagged-num)
-  (:variant-vars base lowtag)
-  (:info offset)
-  (:generator 4
-    (move result value)
-    (inst xadd (make-ea :dword :base object
-                        :disp (- (* (+ base offset) n-word-bytes) lowtag))
-          value)))
