@@ -68,8 +68,6 @@
                         (and (sc-is x any-reg descriptor-reg immediate)
                              (sc-is y control-stack))))))
   (:temporary (:sc unsigned-reg) temp)
-  (:effects)
-  (:affected)
   (:generator 0
     (if (and (sc-is x immediate)
              (sc-is y any-reg descriptor-reg control-stack))
@@ -80,16 +78,31 @@
   (any-reg descriptor-reg immediate)
   (any-reg descriptor-reg))
 
-(defun move-immediate (target val &optional tmp-tn)
+(defun move-immediate (target val &optional tmp-tn zeroed)
+  ;; Try to emit the smallest immediate operand if the destination word
+  ;; is already zeroed. Otherwise a :qword.
   (cond
     ;; If target is a register, we can just mov it there directly
     ((and (tn-p target)
           (sc-is target signed-reg unsigned-reg descriptor-reg any-reg))
-     (if (zerop val)
-         (zeroize target)
-         (inst mov target val)))
+     ;; val can be a fixup for an immobile-space symbol, i.e. not a number,
+     ;; hence not acceptable to ZEROP.
+     (cond ((and (numberp val) (zerop val)) (zeroize target))
+           (t (inst mov target val))))
     ;; Likewise if the value is small enough.
-    ((typep val '(signed-byte 32))
+    ((typep val '(or (signed-byte 32) #!+immobile-space fixup))
+     ;; This logic is similar to that of STOREW*.
+     ;; It would be nice to pull it all together in one place.
+     ;; The basic idea is that storing any byte-aligned 8-bit value
+     ;; should be a single byte write, etc.
+     (when (and zeroed (ea-p target))
+       (setq target (sized-ea target
+                              (typecase val
+                                ((unsigned-byte 8) :byte)
+                                ((unsigned-byte 16) :word)
+                                ;; signed-32 is no good, as it needs sign-extension.
+                                ((unsigned-byte 32) :dword)
+                                (t :qword)))))
      (inst mov target val))
     ;; Otherwise go through the temporary register
     (tmp-tn
@@ -115,7 +128,8 @@
     (sc-case y
       ((any-reg descriptor-reg)
        (if (sc-is x immediate)
-           (inst mov y (encode-value-if-immediate x))
+           (let ((val (encode-value-if-immediate x)))
+             (if (eql val 0) (zeroize y) (inst mov y val)))
            (move y x)))
       ((control-stack)
        (if (= (tn-offset fp) esp-offset)
@@ -243,7 +257,7 @@
                     ;; but let's be future-proof and allow for it.
                     (unless (member x '(rsp rbp) :test 'string=)
                       (symbolicate "ALLOC-" signedp "-BIGNUM-IN-" x)))
-                  sb!x86-64-asm::*qword-reg-names*)
+                  +qword-register-names+)
            (ash (tn-offset ,tn) -1))))
 
 ;;; Convert an untagged signed word to a lispobj -- fixnum or bignum
@@ -316,8 +330,6 @@
                (not (or (location= x y)
                         (and (sc-is x signed-reg unsigned-reg)
                              (sc-is y signed-stack unsigned-stack))))))
-  (:effects)
-  (:affected)
   (:note "word integer move")
   (:generator 0
     (move y x)))
