@@ -30,8 +30,8 @@
   (:note "SAP to pointer coercion")
   (:node-var node)
   (:generator 20
-    (with-fixed-allocation (res sap-widetag sap-size node)
-      (storew sap res sap-pointer-slot other-pointer-lowtag))))
+    (fixed-alloc res sap-widetag sap-size node)
+    (storew sap res sap-pointer-slot other-pointer-lowtag)))
 (define-move-vop move-from-sap :move
   (sap-reg) (descriptor-reg))
 
