@@ -93,11 +93,12 @@
                            (short-combine-methods
                             type-name options operator ioa new-method doc))
                          args))
-            :definition-source source-location))
+            'source source-location))
     (when old-method
       (remove-method #'find-method-combination old-method))
     (add-method #'find-method-combination new-method)
-    (setf (random-documentation type-name 'method-combination) doc)
+    (when doc
+      (setf (random-documentation type-name 'method-combination) doc))
     type-name))
 
 (defun short-combine-methods (type-name options operator ioa method doc)
@@ -115,7 +116,7 @@
                  :options options
                  :operator operator
                  :identity-with-one-argument ioa
-                 :definition-source method
+                 'source method
                  :documentation doc))
 
 (defmethod invalid-qualifiers ((gf generic-function)
@@ -162,7 +163,7 @@
       `(load-long-defcombin ',type-name ',documentation #',function
                             ',args-option (sb-c:source-location)))))
 
-(defvar *long-method-combination-functions* (make-hash-table :test 'eq))
+(define-load-time-global *long-method-combination-functions* (make-hash-table :test 'eq))
 
 (defun load-long-defcombin
     (type-name doc function args-lambda-list source-location)
@@ -188,7 +189,7 @@
                                            :args-lambda-list args-lambda-list
                                            :documentation doc))
                           args))
-             :definition-source source-location)))
+             'source source-location)))
     (setf (gethash type-name *long-method-combination-functions*) function)
     (when old-method (remove-method #'find-method-combination old-method))
     (add-method #'find-method-combination new-method)
@@ -235,7 +236,7 @@
     (reference-condition simple-error)
   ()
   (:default-initargs
-   :references (list '(:ansi-cl :macro define-method-combination))))
+   :references '((:ansi-cl :macro define-method-combination))))
 
 ;;; NOTE:
 ;;;
@@ -395,11 +396,11 @@
     ;; ARGS-LAMBDA-LIST into NREQ and NOPT, and set WHOLE to the
     ;; name of a &WHOLE parameter, if any.
     (when (member '&whole (rest args-lambda-list))
-      (error 'simple-program-error
-             :format-control "~@<The value of the :ARGUMENTS option of ~
-                DEFINE-METHOD-COMBINATION is~2I~_~S,~I~_but &WHOLE may ~
-                only appear first in the lambda list.~:>"
-             :format-arguments (list args-lambda-list)))
+      (%program-error "~@<The value of the :ARGUMENTS option of ~
+                       DEFINE-METHOD-COMBINATION is~2I~_~S,~I~_but ~
+                       &WHOLE may only appear first in the lambda ~
+                       list.~:>"
+                      args-lambda-list))
     (loop with state = 'required
           for arg in args-lambda-list do
             (if (memq arg lambda-list-keywords)
