@@ -198,7 +198,7 @@
 ;;; This needs to be used recursively, in case a non-trivial user
 ;;; defined ADD/REMOVE-DIRECT-METHOD method ends up calling another
 ;;; function using the same lock.
-(defvar *specializer-lock* (sb-thread:make-mutex :name "Specializer lock"))
+(define-load-time-global *specializer-lock* (sb-thread:make-mutex :name "Specializer lock"))
 
 (defmethod add-direct-method :around ((specializer specializer) method)
   ;; All the actions done under this lock are done in an order
@@ -280,7 +280,8 @@
 ;;; This table is shared between threads, so needs to be synchronized.
 ;;; (though insertions are only performed when holding the specializer-lock,
 ;;; so the preceding claim is probably overly paranoid.)
-(defvar *class-eq-specializer-methods* (make-hash-table :test 'eq :synchronized t))
+(define-load-time-global *class-eq-specializer-methods*
+    (make-hash-table :test 'eq :synchronized t))
 
 (defmethod specializer-method-table ((specializer class-eq-specializer))
   *class-eq-specializer-methods*)
@@ -333,7 +334,7 @@
      (apply #'ensure-class name :metaclass metaclass-name
             :direct-superclasses supers
             :direct-slots slots
-            :definition-source source-location
+            'source source-location
             'safe-p safe-p
             other))))
 
@@ -366,7 +367,7 @@
                   (cond
                     ((eq class-or-name name)
                      (error "~@<Class named ~
-                             ~/sb-impl::print-symbol-with-prefix/ ~
+                             ~/sb-ext:print-symbol-with-prefix/ ~
                              specified as its own ~(~A~)class.~@:>"
                             class-or-name which))
                     ((find-class class-or-name (eq which :meta)))
@@ -449,11 +450,8 @@
                   (mapcar (lambda (pl) (make-direct-slotd class pl))
                           direct-slots))
             (slot-value class 'direct-slots)))
-  (if direct-default-initargs-p
-      (setf (plist-value class 'direct-default-initargs)
-            direct-default-initargs)
-      (setq direct-default-initargs
-            (plist-value class 'direct-default-initargs)))
+  (when direct-default-initargs-p
+    (setf (plist-value class 'direct-default-initargs) direct-default-initargs))
   (setf (plist-value class 'class-slot-cells)
         (let ((old-class-slot-cells (plist-value class 'class-slot-cells))
               (safe (safe-p class))
@@ -500,10 +498,10 @@
 
 (defmethod invalid-superclass ((class class) (superclass class))
   (error 'invalid-superclass :class class :superclass superclass
-         :references (list* '(:amop :generic-function validate-superclass)
-                            (and (typep superclass 'built-in-class)
-                                 (list '(:ansi-cl :system-class built-in-class)
-                                       '(:ansi-cl :section (4 3 7)))))))
+         :references `((:amop :generic-function validate-superclass)
+                       ,@(when (typep superclass 'built-in-class)
+                           '((:ansi-cl :system-class built-in-class)
+                             (:ansi-cl :section (4 3 7)))))))
 
 (defmethod shared-initialize :after ((class forward-referenced-class)
                                      slot-names &key &allow-other-keys)
@@ -921,7 +919,7 @@
 (define-condition cpl-protocol-violation (reference-condition error)
   ((class :initarg :class :reader cpl-protocol-violation-class)
    (cpl :initarg :cpl :reader cpl-protocol-violation-cpl))
-  (:default-initargs :references (list '(:sbcl :node "Metaobject Protocol")))
+  (:default-initargs :references '((:sbcl :node "Metaobject Protocol")))
   (:report
    (lambda (c s)
      (format s "~@<Protocol violation: the ~S class ~S ~
@@ -1030,7 +1028,7 @@
           (style-warn
            "~@<slot names with the same SYMBOL-NAME but ~
                   different SYMBOL-PACKAGE (possible package problem) ~
-                  for class ~S:~4I~@:_~<~@{~/sb-impl::print-symbol-with-prefix/~^~:@_~}~:>~@:>"
+                  for class ~S:~4I~@:_~<~@{~/sb-ext:print-symbol-with-prefix/~^~:@_~}~:>~@:>"
            class dupes)))
     (let* ((slot-name (slot-definition-name (car slots)))
            (oslots (and (not (eq (symbol-package slot-name)
@@ -1185,7 +1183,7 @@
                                   (push c (class-slot-cells from-class))
                                   c))))
                  (aver (consp cell))
-                 (if (eq +slot-unbound+ (cdr cell))
+                 (if (unbound-marker-p (cdr cell))
                      ;; We may have inherited an initfunction FIXME: Is this
                      ;; really right? Is the initialization in
                      ;; SHARED-INITIALIZE (STD-CLASS) not enough?
@@ -1305,7 +1303,7 @@
                              :slot-name slot-name
                              :object-class class
                              :method-class-function #'reader-method-class
-                             :definition-source source-location)))
+                             'source source-location)))
 
 (defmethod writer-method-class ((class slot-class) direct-slot &rest initargs)
   (declare (ignore direct-slot initargs))
@@ -1322,7 +1320,7 @@
                              :slot-name slot-name
                              :object-class class
                              :method-class-function #'writer-method-class
-                             :definition-source source-location)))
+                             'source source-location)))
 
 (defmethod add-boundp-method ((class slot-class) generic-function slot-name slot-documentation source-location)
   (add-method generic-function
@@ -1334,7 +1332,7 @@
                              (make-boundp-method-function class slot-name)
                              (or slot-documentation "automatically generated boundp method")
                              :slot-name slot-name
-                             :definition-source source-location)))
+                             'source source-location)))
 
 (defmethod remove-reader-method ((class slot-class) generic-function)
   (let ((method (get-method generic-function () (list class) nil)))
@@ -1488,7 +1486,7 @@
     (restart-case
         (bad-type value slot-type
                   "~@<Error during ~A. Current value in slot ~
-                   ~/sb-impl::print-symbol-with-prefix/ of an instance ~
+                   ~/sb-ext:print-symbol-with-prefix/ of an instance ~
                    of ~S is ~S, which does not match the new slot type ~
                    ~S in class ~S.~:@>"
                   context slot-name old-class value slot-type new-class)
@@ -1497,7 +1495,7 @@
         :report (lambda (stream)
                   (format stream "~@<Specify a new value to by used ~
                                   for slot ~
-                                  ~/sb-impl::print-symbol-with-prefix/ ~
+                                  ~/sb-ext:print-symbol-with-prefix/ ~
                                   instead of ~S.~@:>"
                           slot-name value))
         (setf value new-value))))
@@ -1506,7 +1504,7 @@
 (defun %set-slot-value-checking-type (context slots slot value
                                       safe old-class new-class)
   (setf (clos-slots-ref slots (slot-definition-location slot))
-        (if (and safe (neq value +slot-unbound+))
+        (if (and safe (not (unbound-marker-p value)))
             (let ((name (slot-definition-name slot))
                   (type (slot-definition-type slot)))
               (%ensure-slot-value-type context name type value
@@ -1514,8 +1512,6 @@
             value)))
 
 (defvar *in-obsolete-instance-trap* nil)
-(defvar *the-wrapper-of-structure-object*
-  (class-wrapper (find-class 'structure-object)))
 
 (define-condition obsolete-structure (error)
   ((datum :reader obsolete-structure-datum :initarg :datum))
@@ -1569,7 +1565,7 @@
          (dolist (old old-instance-slots)
            (let* ((name (slot-definition-name old))
                   (value (clos-slots-ref oslots (slot-definition-location old))))
-             (unless (eq value +slot-unbound+)
+             (unless (unbound-marker-p value)
                (let ((new (assq name layout)))
                  (cond (new
                         (set-value value new))
@@ -1606,8 +1602,7 @@
         instance added discarded plist)
 
        nwrapper))
-    (*in-obsolete-instance-trap*
-     *the-wrapper-of-structure-object*)
+    (*in-obsolete-instance-trap* #.(find-layout 'structure-object))
     (t
      (let ((*in-obsolete-instance-trap* t))
        (error 'obsolete-structure :datum instance)))))
@@ -1631,9 +1626,8 @@
                      return slot))
              (initarg-for-slot-p (slot)
                (dolist (slot-initarg (slot-definition-initargs slot))
-                 ;; Abuse +slot-unbound+
-                 (unless (eq +slot-unbound+
-                             (getf initargs slot-initarg +slot-unbound+))
+                 (unless (unbound-marker-p
+                          (getf initargs slot-initarg +slot-unbound+))
                    (return t))))
              (set-value (value slotd)
                (%set-slot-value-checking-type
@@ -1721,7 +1715,7 @@
   (declare (ignore initargs))
   (error "You can't change the class of ~S to ~S~@
           because it isn't already an instance with metaclass ~S."
-         instance new-class 'standard-class))
+         instance new-class 'funcallable-standard-class))
 
 (defmethod change-class ((instance funcallable-standard-object)
                          (new-class standard-class)
@@ -1729,7 +1723,7 @@
   (declare (ignore initargs))
   (error "You can't change the class of ~S to ~S~@
           because it isn't already an instance with metaclass ~S."
-         instance new-class 'funcallable-standard-class))
+         instance new-class 'standard-class))
 
 (defmethod change-class ((instance t) (new-class-name symbol) &rest initargs)
   (apply #'change-class instance (find-class new-class-name) initargs))
@@ -1754,7 +1748,7 @@
                        :format-control ,(coerce (format nil "~@<~A~@:>" control)
                                                 'base-string)
                        :format-arguments (list (class-name class))
-                       :references (list '(:amop :initialization "Class"))))))
+                       :references '((:amop :initialization "Class"))))))
   (def initialize-instance ((class system-class) &rest initargs)
     "Cannot initialize an instance of ~S.")
   (def reinitialize-instance ((class system-class) &rest initargs)
