@@ -18,7 +18,8 @@
 #include "genesis/sap.h"
 
 #ifdef LISP_FEATURE_GENCGC
-extern lispobj *general_alloc(sword_t bytes, int page_type_flag);
+extern lispobj *lisp_alloc(struct alloc_region *region, sword_t nbytes,
+                           int page_type_flag, struct thread *thread);
 extern lispobj alloc_code_object(unsigned boxed, unsigned unboxed);
 #endif
 
