@@ -3,9 +3,6 @@
 
 typedef struct ucontext os_context_t;
 
-static inline os_context_t *arch_os_get_context(void **void_context)
-{
-    return (os_context_t *) *void_context;
-}
+#include "arch-os-generic.inc"
 
 #endif /* _ALPHA_OSF1_OS_H */
