@@ -33,6 +33,8 @@
 #include "thread.h"
 #include "arch.h"
 
+#include "private-cons.inc"
+
 /* So you need to debug? */
 #if 0
 #define PRINTNOISE
@@ -50,6 +52,8 @@ lispobj *new_space_free_pointer;
 /* This does nothing. It's only to satisfy a reference from gc-common. */
 char gc_coalesce_string_literals = 0;
 
+boolean gc_active_p = 0;
+
 static void scavenge_newspace(void);
 
 
@@ -103,6 +107,8 @@ collect_garbage(generation_index_t ignore)
     gettimeofday(&start_tv, (struct timezone *) 0);
 #endif
 
+    gc_active_p = 1;
+
     /* it's possible that signals are blocked already if this was called
      * from a signal handler (e.g. with the sigsegv gc_trigger stuff) */
     block_blockable_signals(&old);
@@ -180,7 +186,7 @@ collect_garbage(generation_index_t ignore)
 #ifdef PRINTNOISE
     printf("Scanning weak hash tables ...\n");
 #endif
-    scan_weak_hash_tables(weak_ht_alivep_funs);
+    cull_weak_hash_tables(weak_ht_alivep_funs);
 
     /* Scan the weak pointers. */
 #ifdef PRINTNOISE
@@ -221,6 +227,7 @@ collect_garbage(generation_index_t ignore)
     set_auto_gc_trigger(size_retained+bytes_consed_between_gcs);
     thread_sigmask(SIG_SETMASK, &old, 0);
 
+    gc_active_p = 0;
 
 #ifdef PRINTNOISE
     gettimeofday(&stop_tv, (struct timezone *) 0);
@@ -256,14 +263,16 @@ scavenge_newspace(void)
     lispobj *here, *next;
 
     here = new_space;
-    while (here < new_space_free_pointer) {
+
+    do {
         /*      printf("here=%lx, new_space_free_pointer=%lx\n",
                 here,new_space_free_pointer); */
         next = new_space_free_pointer;
         heap_scavenge(here, next);
-        scav_weak_hash_tables(weak_ht_alivep_funs, gc_scav_pair);
         here = next;
-    }
+    } while (new_space_free_pointer > here ||
+             (test_weak_triggers(0, 0) && new_space_free_pointer > here));
+    gc_dispose_private_pages();
     /* printf("done with newspace\n"); */
 }
 
@@ -355,6 +364,7 @@ search_dynamic_space(void *pointer)
 void
 gc_init(void)
 {
+    weakobj_init();
     scavtab[WEAK_POINTER_WIDETAG] = scav_weak_pointer;
 }
 
