@@ -277,6 +277,7 @@ struct heap_adjust {
 #define relocate_heap(ignore)
 #else
 #include "genesis/gc-tables.h"
+#include "genesis/cons.h"
 #include "genesis/hash-table.h"
 #include "genesis/layout.h"
 #include "genesis/vector.h"
@@ -300,6 +301,18 @@ static inline lispobj adjust_word(struct heap_adjust* adj, lispobj word) {
     return word + calc_adjustment(adj, word);
 }
 
+// Given a post-relocation object 'x', compute the address at which
+// it was originally expected to have been placed as per the core file.
+static inline lispobj inverse_adjust(struct heap_adjust* adj, lispobj x)
+{
+    int j;
+    for (j=0; j<3; ++j)
+        if (adj->range[j].start + adj->range[j].delta <= x &&
+            x < adj->range[j].end + adj->range[j].delta)
+            return x - adj->range[j].delta;
+    return x;
+}
+
 #define SHOW_SPACE_RELOCATION 0
 #if SHOW_SPACE_RELOCATION > 1
 # define FIXUP(expr, addr) fprintf(stderr, "%p: (a) %lx", addr, *(long*)(addr)), \
@@ -344,11 +357,21 @@ static void adjust_pointers(lispobj *where, sword_t n_words, struct heap_adjust*
 #include "var-io.h"
 #include "unaligned.h"
 static void __attribute__((unused))
-adjust_code_refs(struct heap_adjust* adj, lispobj fixups, struct code* code)
+adjust_code_refs(struct heap_adjust* adj, struct code* code, lispobj original_vaddr)
 {
-    struct varint_unpacker unpacker;
-    varint_unpacker_init(&unpacker, fixups);
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    // Dynamic space is a always relocated before immobile space,
+    // and dynamic space code does not use fixups (except on 32-bit x86).
+    // So if we're here, it must be to relocate an immobile object.
+    // Therefore these CAR and CDR operations are correct,
+    // because what they point to has already been relocated.
+    lispobj fixups    = code->fixups;
+    lispobj absfixups = listp(fixups) ? CONS(fixups)->car : fixups;
+    lispobj relfixups = listp(fixups) ? CONS(fixups)->cdr : 0;
     char* instructions = (char*)((lispobj*)code + code_header_words(code->header));
+    struct varint_unpacker unpacker;
+
+    varint_unpacker_init(&unpacker, absfixups);
     int prev_loc = 0, loc;
     while (varint_unpack(&unpacker, &loc) && loc != 0) {
         // For extra compactness, each loc is relative to the prior,
@@ -361,6 +384,20 @@ adjust_code_refs(struct heap_adjust* adj, lispobj fixups, struct code* code)
         if (adjusted != ptr)
             FIXUP32(UNALIGNED_STORE32(fixup_where, adjusted), fixup_where);
     }
+    sword_t displacement = (lispobj)code - original_vaddr;
+    if (!displacement) // if this code didn't move, do nothing
+        return;
+    varint_unpacker_init(&unpacker, relfixups);
+    prev_loc = 0;
+    while (varint_unpack(&unpacker, &loc) && loc != 0) {
+        loc += prev_loc;
+        prev_loc = loc;
+        void* fixup_where = instructions + loc;
+        lispobj rel32operand = UNALIGNED_LOAD32(fixup_where);
+        lispobj adjusted = rel32operand - displacement;
+        FIXUP_rel(UNALIGNED_STORE32(fixup_where, adjusted), fixup_where);
+    }
+#endif
 }
 
 static inline void fix_fun_header_layout(lispobj* fun, struct heap_adjust* adj)
@@ -456,21 +493,20 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 #if FUN_SELF_FIXNUM_TAGGED
                 if (f->self != (lispobj)f->code)
                     FIXUP(f->self = (lispobj)f->code, &f->self);
-#endif
                 adjust_pointers(SIMPLE_FUN_SCAV_START(f), SIMPLE_FUN_SCAV_NWORDS(f), adj);
-            });
-            // Compute the address where the code "was" as the first argument
-            // by negating the adjustment for 'where'.
-            // Can't call calc_adjustment to get the negative of the adjustment!
-            gencgc_apply_code_fixups((struct code*)((char*)where -
-                                                    adj->range[DYNAMIC_SPACE_ADJ_INDEX].delta),
-                                     code);
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-            // Now that the packed integer comprising the list of fixup locations
-            // has been fixed-up (if necessary), apply them to the code.
-            if (code->fixups != 0)
-                adjust_code_refs(adj, code->fixups, code);
+#else
+                adjust_pointers(&f->self, (lispobj*)f->code - &f->self, adj);
 #endif
+            });
+            {
+              // Now that the packed integer comprising the list of fixup locations
+              // has been fixed-up (if necessary), apply them to the code.
+              lispobj original_vaddr = inverse_adjust(adj, (lispobj)code);
+              // code->fixups, if a bignum pointer, was fixed up as part of
+              // the constant pool.
+              gencgc_apply_code_fixups((struct code*)original_vaddr, code);
+              adjust_code_refs(adj, code, original_vaddr);
+            }
             continue;
         case CLOSURE_WIDETAG:
             fix_fun_header_layout(where, adj);
@@ -579,7 +615,11 @@ static void relocate_heap(struct heap_adjust* adj)
     SYMBOL(FUNCTION_LAYOUT)->value = \
         adjust_word(adj, SYMBOL(FUNCTION_LAYOUT)->value >> 32) << 32;
 #endif
+#ifdef LISP_FEATURE_CHENEYGC
+    relocate_space(DYNAMIC_0_SPACE_START, (lispobj*)get_alloc_pointer(), adj);
+#else
     relocate_space(DYNAMIC_SPACE_START, (lispobj*)get_alloc_pointer(), adj);
+#endif
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     // Pointers within varyobj space to varyobj space do not need adjustment
@@ -591,9 +631,7 @@ static void relocate_heap(struct heap_adjust* adj)
     relocate_space(VARYOBJ_SPACE_START, varyobj_free_pointer, adj);
 #endif
 }
-#endif
 
-#ifdef LISP_FEATURE_RELOCATABLE_HEAP
 static void
 set_adjustment(struct heap_adjust* adj,
                uword_t actual_addr,
@@ -627,7 +665,10 @@ process_directory(int count, struct ndir_entry *entry,
 #ifdef LISP_FEATURE_GENCGC
         {dynamic_space_size, 0, DYNAMIC_SPACE_START, 0},
 #else
-        {0, 0, 0, 0},
+        // Whatever address the core's dynamic space has
+        // becomes subspace 0. The other is subspace 1.
+        // It makes no difference which has the lower address.
+        {dynamic_space_size, 0, DYNAMIC_0_SPACE_START, 0},
 #endif
         // This order is determined by constants in compiler/generic/genesis
         {0, 0, STATIC_SPACE_START, &static_space_free_pointer},
@@ -686,16 +727,6 @@ process_directory(int count, struct ndir_entry *entry,
         // than code space (4K). So don't assert the alignment.
         if (enforce_address) {
             int fail;
-#ifdef LISP_FEATURE_CHENEYGC
-            if (id == DYNAMIC_CORE_SPACE_ID) {
-                if ((fail = (addr != DYNAMIC_0_SPACE_START) &&
-                            (addr != DYNAMIC_1_SPACE_START)) != 0)
-                    fprintf(stderr, "in core: %p; in runtime: %p or %p\n",
-                            (void*)addr,
-                            (void*)DYNAMIC_0_SPACE_START,
-                            (void*)DYNAMIC_1_SPACE_START);
-            } else
-#endif
             if ((fail = (addr != spaces[id].base)) != 0)
                 fprintf(stderr, "in core: %p; in runtime: %p\n",
                         (void*)addr, (void*)spaces[id].base);
@@ -743,12 +774,35 @@ process_directory(int count, struct ndir_entry *entry,
                 break;
 #endif
             case DYNAMIC_CORE_SPACE_ID:
+#ifdef LISP_FEATURE_CHENEYGC
+                aligned_start = ALIGN_UP(addr, BACKEND_PAGE_BYTES);
+                /* Misalignment can happen only if page size exceeds OS page.
+                 * Drop one page to avoid overrunning the allocated space */
+                if (aligned_start > addr) // not page-aligned
+                    dynamic_space_size -= BACKEND_PAGE_BYTES;
+                DYNAMIC_0_SPACE_START = addr = aligned_start;
+                current_dynamic_space = (lispobj*)addr;
+                { // Request that much again now
+#ifdef LISP_FEATURE_ALPHA /* meaning: is this a 32-on-64 SBCL implementation */
+                  uword_t addr1 = (uword_t)os_validate(MOVABLE_LOW, 0, request);
+                  extern os_set_cheneygc_spaces(uword_t,uword_t);
+                  os_set_cheneygc_spaces(addr, addr1);
+#else
+                  uword_t addr1 = (uword_t)os_allocate(request);
+#endif
+                  aligned_start = ALIGN_UP(addr1, BACKEND_PAGE_BYTES);
+                  if (aligned_start > addr) // same test again
+                      dynamic_space_size -= BACKEND_PAGE_BYTES;
+                  DYNAMIC_1_SPACE_START = aligned_start;
+                }
+#else
                 aligned_start = ALIGN_UP(addr, GENCGC_CARD_BYTES);
                 /* Misalignment can happen only if card size exceeds OS page.
                  * Drop one card to avoid overrunning the allocated space */
                 if (aligned_start > addr) // not card-aligned
                     dynamic_space_size -= GENCGC_CARD_BYTES;
                 DYNAMIC_SPACE_START = addr = aligned_start;
+#endif
                 break;
             }
 #endif /* LISP_FEATURE_RELOCATABLE_HEAP */
@@ -793,17 +847,23 @@ process_directory(int count, struct ndir_entry *entry,
     }
 
 #ifdef LISP_FEATURE_RELOCATABLE_HEAP
+#  ifdef LISP_FEATURE_GENCGC
     set_adjustment(adj, DYNAMIC_SPACE_START, // actual
                    spaces[DYNAMIC_CORE_SPACE_ID].base, // expected
                    spaces[DYNAMIC_CORE_SPACE_ID].len);
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+#    ifdef LISP_FEATURE_IMMOBILE_SPACE
     set_adjustment(adj, FIXEDOBJ_SPACE_START, // actual
                    spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].base, // expected
                    spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].len);
     set_adjustment(adj, VARYOBJ_SPACE_START, // actual
                    spaces[IMMOBILE_VARYOBJ_CORE_SPACE_ID].base, // expected
                    spaces[IMMOBILE_VARYOBJ_CORE_SPACE_ID].len);
-#endif
+#    endif
+#  else
+    set_adjustment(adj, DYNAMIC_0_SPACE_START, // actual
+                   spaces[DYNAMIC_CORE_SPACE_ID].base, // expected
+                   spaces[DYNAMIC_CORE_SPACE_ID].len);
+#  endif // LISP_FEATURE_GENCGC
     if (adj->range[0].delta | adj->range[1].delta | adj->range[2].delta)
         relocate_heap(adj);
 #endif
@@ -828,7 +888,7 @@ process_directory(int count, struct ndir_entry *entry,
     struct range range1 =
         {FIXEDOBJ_SPACE_START, FIXEDOBJ_SPACE_START + FIXEDOBJ_SPACE_SIZE};
     struct range range2 =
-        {VARYOBJ_SPACE_START, VARYOBJ_SPACE_START + VARYOBJ_SPACE_SIZE};
+        {VARYOBJ_SPACE_START, VARYOBJ_SPACE_START + varyobj_space_size};
     if (range2.start < range1.start) { // swap
         struct range temp = range1;
         range1 = range2;
