@@ -34,10 +34,6 @@ typedef ucontext_t os_context_t;
 
 #define SIG_STOP_FOR_GC (SIGUSR2)
 
-#ifdef LISP_FEATURE_MACH_EXCEPTION_HANDLER
-extern mach_port_t current_mach_task;
-#endif
-
 void darwin_init(void);
 
 #ifdef LISP_FEATURE_SB_THREAD
