@@ -28,19 +28,6 @@ extern lispobj call_into_lisp(lispobj fun, lispobj *args, int nargs)
 #endif
     ;
 
-static inline lispobj
-safe_call_into_lisp(lispobj fun, lispobj *args, int nargs)
-{
-#ifndef LISP_FEATURE_SB_SAFEPOINT
-    /* SIG_STOP_FOR_GC needs to be enabled before we can call lisp:
-     * otherwise two threads racing here may deadlock: the other will
-     * wait on the GC lock, and the other cannot stop the first
-     * one... */
-    check_gc_signals_unblocked_or_lose(0);
-#endif
-    return call_into_lisp(fun, args, nargs);
-}
-
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
 /* These functions are an interface to the Lisp call-in facility.
  * Since this is C we can know nothing about the calling environment.
@@ -57,14 +44,14 @@ funcall0(lispobj function)
     lispobj *args = NULL;
 
     FSHOW((stderr, "/entering funcall0(0x%lx)\n", (long)function));
-    return safe_call_into_lisp(function, args, 0);
+    return call_into_lisp(function, args, 0);
 }
 lispobj
 funcall1(lispobj function, lispobj arg0)
 {
     lispobj args[1];
     args[0] = arg0;
-    return safe_call_into_lisp(function, args, 1);
+    return call_into_lisp(function, args, 1);
 }
 
 lispobj
@@ -73,7 +60,7 @@ funcall2(lispobj function, lispobj arg0, lispobj arg1)
     lispobj args[2];
     args[0] = arg0;
     args[1] = arg1;
-    return safe_call_into_lisp(function, args, 2);
+    return call_into_lisp(function, args, 2);
 }
 
 lispobj
@@ -83,7 +70,7 @@ funcall3(lispobj function, lispobj arg0, lispobj arg1, lispobj arg2)
     args[0] = arg0;
     args[1] = arg1;
     args[2] = arg2;
-    return safe_call_into_lisp(function, args, 3);
+    return call_into_lisp(function, args, 3);
 }
 
 #else
@@ -95,7 +82,7 @@ funcall0(lispobj function)
         = &access_control_stack_pointer(arch_os_get_current_thread());
     lispobj *args = *stack_pointer;
 
-    return safe_call_into_lisp(function, args, 0);
+    return call_into_lisp(function, args, 0);
 }
 
 lispobj
@@ -108,7 +95,7 @@ funcall1(lispobj function, lispobj arg0)
     *stack_pointer += 1;
     args[0] = arg0;
 
-    return safe_call_into_lisp(function, args, 1);
+    return call_into_lisp(function, args, 1);
 }
 
 lispobj
@@ -122,7 +109,7 @@ funcall2(lispobj function, lispobj arg0, lispobj arg1)
     args[0] = arg0;
     args[1] = arg1;
 
-    return safe_call_into_lisp(function, args, 2);
+    return call_into_lisp(function, args, 2);
 }
 
 lispobj
@@ -137,6 +124,6 @@ funcall3(lispobj function, lispobj arg0, lispobj arg1, lispobj arg2)
     args[1] = arg1;
     args[2] = arg2;
 
-    return safe_call_into_lisp(function, args, 3);
+    return call_into_lisp(function, args, 3);
 }
 #endif
