@@ -12,6 +12,22 @@
 #ifndef _GENCGC_PRIVATE_H_
 #define _GENCGC_PRIVATE_H_
 
+void zero_dirty_pages(page_index_t start, page_index_t end);
+
+/// There is some additional cleverness that could potentially be had -
+/// the "need_to_zero" bit (a/k/a "page dirty") is obviously 1 if the page
+/// contains objects. Only for an empty page must we distinguish between pages
+/// not needing be zero-filled before next use and those which must be.
+/// Thus, masking off the dirty bit could be avoided by not storing it for
+/// any in-use page. But since that's not what we do - we set the bit to 1
+/// as soon as a page is used - we do have to mask off the bit.
+#define page_bytes_used(index) (page_table[index].bytes_used_ & ~1)
+#define page_need_to_zero(index) (page_table[index].bytes_used_ & 1)
+#define set_page_bytes_used(index,val) \
+  page_table[index].bytes_used_ = (val) | page_need_to_zero(index)
+#define set_page_need_to_zero(index,val) \
+  page_table[index].bytes_used_ = page_bytes_used(index) | val
+
 #if !CONDENSED_PAGE_TABLE
 
 // 32-bit doesn't need magic to reduce the size of scan_start_offset.
@@ -31,7 +47,8 @@
 //#define SCAN_START_OFS_MAX 0x3fff
 #define SCAN_START_OFS_MAX UINT_MAX
 
-static void set_page_scan_start_offset(page_index_t index, os_vm_size_t offset)
+static void __attribute__((unused))
+set_page_scan_start_offset(page_index_t index, os_vm_size_t offset)
 {
     // If the offset is nonzero and page-aligned
     unsigned int lsb = offset !=0 && IS_ALIGNED(offset, GENCGC_CARD_BYTES);
@@ -61,7 +78,7 @@ static os_vm_size_t scan_start_offset_iterated(page_index_t index)
     return (os_vm_size_t)tot_offset_in_pages << GENCGC_CARD_SHIFT;
 }
 
-static os_vm_size_t page_scan_start_offset(page_index_t index)
+static os_vm_size_t  __attribute__((unused)) page_scan_start_offset(page_index_t index)
 {
     return page_table[index].scan_start_offset_ != SCAN_START_OFS_MAX
         ? (os_vm_size_t)(page_table[index].scan_start_offset_ & ~1)
