@@ -534,7 +534,7 @@ void
 arch_handle_single_step_trap(os_context_t *context, int trap)
 {
     unsigned int code = *((u32 *)(*os_context_pc_addr(context)));
-    int register_offset = code >> 5 & 0x1f;
+    int register_offset = code >> 8 & 0x1f;
     handle_single_step_trap(context, trap, register_offset);
     arch_skip_instruction(context);
 }
@@ -566,7 +566,7 @@ sigtrap_handler(int signal, siginfo_t *siginfo, os_context_t *context)
 
     if ((code >> 16) == ((3 << 10) | (6 << 5))) {
         /* twllei reg_ZERO,N will always trap if reg_ZERO = 0 */
-        int trap = code & 0x1f;
+        int trap = code & 0xff;
         handle_trap(context,trap);
         return;
     }
