@@ -582,9 +582,6 @@ main(int argc, char *argv[], char *envp[])
             } else if (0 == strcmp(arg, "--no-merge-core-pages")) {
                 ++argi;
                 merge_core_pages = 0;
-            } else if (0 == strcmp(arg, "--default-merge-core-pages")) {
-                ++argi;
-                merge_core_pages = -1;
             } else {
                 /* This option was unrecognized as a runtime option,
                  * so it must be a toplevel option or a user option,
