@@ -833,20 +833,20 @@ os_validate(int movable, os_vm_address_t addr, os_vm_size_t len)
         return 0;
 
     if ((mem_info.State == MEM_RESERVE) && (mem_info.RegionSize >=len)) {
-      /* It would be correct to return here. However, support for Wine
-       * is beneficial, and Wine has a strange behavior in this
-       * department. It reports all memory below KERNEL32.DLL as
-       * reserved, but disallows MEM_COMMIT.
-       *
-       * Let's work around it: reserve the region we need for a second
-       * time. The second reservation is documented to fail on normal NT
-       * family, but it will succeed on Wine if this region is
-       * actually free.
-       */
-      VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
-      /* If it is wine, the second call has succeded, and now the region
-       * is really reserved. */
-      return addr;
+        /* It would be correct to return here. However, support for Wine
+         * is beneficial, and Wine has a strange behavior in this
+         * department. It reports all memory below KERNEL32.DLL as
+         * reserved, but disallows MEM_COMMIT.
+         *
+         * Let's work around it: reserve the region we need for a second
+         * time. The second reservation is documented to fail on normal NT
+         * family, but it will succeed on Wine if this region is
+         * actually free.
+         */
+        VirtualAlloc(addr, len, MEM_RESERVE, PAGE_EXECUTE_READWRITE);
+        /* If it is wine, the second call has succeded, and now the region
+         * is really reserved. */
+        return addr;
     }
 
     if (mem_info.State == MEM_RESERVE) {
@@ -856,11 +856,12 @@ os_validate(int movable, os_vm_address_t addr, os_vm_size_t len)
          * provision for MEM_RESERVE in the following code, I suppose: */
     }
 
-    if (!AVERLAX(VirtualAlloc(addr, len, (mem_info.State == MEM_RESERVE)?
-                              MEM_COMMIT: MEM_RESERVE, PAGE_EXECUTE_READWRITE)))
-        return 0;
+    os_vm_address_t actual;
 
-    return addr;
+    if (!AVERLAX(actual = VirtualAlloc(addr, len, (mem_info.State == MEM_RESERVE)?
+                                       MEM_COMMIT: MEM_RESERVE, PAGE_EXECUTE_READWRITE)))
+        return 0;
+    return actual;
 }
 
 /*
@@ -1224,6 +1225,13 @@ handle_access_violation(os_context_t *ctx,
                               MEM_COMMIT, PAGE_EXECUTE_READWRITE));
         }
         return 0;
+    } else {
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+        extern int immobile_space_handle_wp_violation(void*);
+        if (immobile_space_handle_wp_violation(fault_address)) {
+            return 0;
+        }
+#endif
     }
 
     if (fault_address == undefined_alien_address)
