@@ -144,8 +144,8 @@
 (with-test (:name :space-bounds-no-consing
                   :skipped-on :interpreter)
   ;; Asking for the size of a heap space should not cost anything!
-  (ctu:assert-no-consing (sb-vm::%space-bounds :static))
-  (ctu:assert-no-consing (sb-vm::space-bytes :static)))
+  (ctu:assert-no-consing (sb-vm:%space-bounds :static))
+  (ctu:assert-no-consing (sb-vm:space-bytes :static)))
 
 (with-test (:name (sb-vm::map-allocated-objects :no-consing)
                   :fails-on :cheneygc
@@ -591,10 +591,18 @@
    `(lambda ()
       (floor -114658225103614 84619.58))
     (() (values -1354984705 8473228.0)))
+  (checked-compile-and-assert (:optimize :safe)
+   `(lambda ()
+      (floor -302254842 50510.5))
+    (() (eval '(floor -302254842 50510.5))))
   (checked-compile-and-assert (:optimize :safe)
    `(lambda ()
       (ceiling 114658225103614 84619.58))
-    (() (values 1354984705 -8473228.0))))
+    (() (values 1354984705 -8473228.0)))
+  (checked-compile-and-assert (:optimize :safe)
+   `(lambda ()
+      (ceiling 285493348393 94189.93))
+    (() (values 3031039 0.0))))
 
 (with-test (:name :complex-float-contagion)
   (checked-compile-and-assert ()
@@ -629,6 +637,14 @@
        (equalp a b))
     (("" #*) t)))
 
+(with-test (:name :equalp-transform-zero-string)
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (equalp "" a))
+   ((#*) t)
+   ((#()) t)))
+
 (with-test (:name :fill-transform-returning-array-data)
   (let ((vector (make-array 10 :fill-pointer 2)))
     (checked-compile-and-assert ()
@@ -1096,3 +1112,65 @@
   (checked-compile
       '(lambda (x)
         (concatenate '(and string (satisfies eval)) x))))
+
+(with-test (:name :make-array-transform-deletion-notes)
+  (checked-compile
+   `(lambda (vector)
+      (let* ((length (length vector))
+             (new (make-array length :adjustable t
+                                     :fill-pointer length)))
+        new))
+   :allow-notes nil))
+
+(with-test (:name :ltn-analyze-cast-unlink)
+  (assert (nth-value 1 (checked-compile
+                        `(lambda (n)
+                           (* 2 n)
+                           (let ((p (make-array n :element-type 'double-float)))
+                             (dotimes (i n)
+                               (setf (aref p i)
+                                     (ignore-errors i)))))
+                        :allow-warnings t))))
+
+(with-test (:name :call-type-validation)
+  (checked-compile
+   `(lambda ()
+      (funcall (the (or cons function) *debugger-hook*)))))
+
+(with-test (:name :setf-schar-hairy-types)
+  (checked-compile-and-assert
+      ()
+      `(lambda (s v)
+         (setf (schar (the (satisfies eval) s) 0) v)
+         s)
+    (((copy-seq "abc") #\m) "mbc" :test #'equal)))
+
+(with-test (:name :check-function-designator-cast-key-lambda-var)
+  (checked-compile-and-assert
+      (:optimize '(:speed 3 :space 0))
+      `(lambda (p1 p4)
+         (declare (vector p1)
+                  ((member ,#'car "x" cdr) p4))
+         (stable-sort p1 #'<= :key p4))
+    (((vector '(2) '(3) '(1)) #'car) #((1) (2) (3)) :test #'equalp)))
+
+(with-test (:name :replace-zero-elements)
+  (checked-compile-and-assert
+      ()
+      '(lambda (x)
+        (declare ((simple-vector 2) x))
+        (replace x x :start1 2))
+    (((vector 1 2)) #(1 2) :test #'equalp))
+  (checked-compile-and-assert
+      ()
+      '(lambda (x)
+        (replace x x :start1 2))
+    (((vector 1 2)) #(1 2) :test #'equalp)))
+
+(with-test (:name :error-in-xep)
+  (checked-compile-and-assert
+      (:optimize :safe)
+      '(lambda (x)
+        (declare (type (satisfies error) x))
+        x)
+    (("") (condition 'error))))
