@@ -123,33 +123,33 @@
             (setf (sb-int:info :function :source-transform 'make-instance) wrapper)
             (with-test (:name (make-instance :non-constant-class))
               (assert (= 0 opt))
-              (let ((f (compile nil `(lambda (class)
-                                       (make-instance class :b t)))))
+              (let ((f (checked-compile `(lambda (class)
+                                           (make-instance class :b t)))))
                 (assert (= 1 (length (find-ctor-caches f))))
                 (assert (= 1 opt))
                 (assert (typep (funcall f 'one-slot-subclass) 'one-slot-subclass))))
             (with-test (:name (make-instance :constant-class-object))
-              (let ((f (compile nil `(lambda ()
-                                       (make-instance ,(find-class 'one-slot-subclass) :b t)))))
+              (let ((f (checked-compile `(lambda ()
+                                           (make-instance ,(find-class 'one-slot-subclass) :b t)))))
                 (assert (not (find-ctor-caches f)))
                 (assert (= 2 opt))
                 (assert (typep (funcall f) 'one-slot-subclass))))
             (with-test (:name (make-instance :constant-non-std-class-object))
-              (let ((f (compile nil `(lambda ()
-                                       (make-instance ,(find-class 'structure-object))))))
+              (let ((f (checked-compile `(lambda ()
+                                           (make-instance ,(find-class 'structure-object))))))
                 (assert (not (find-ctor-caches f)))
                 (assert (= 3 opt))
                 (assert (typep (funcall f) 'structure-object))))
             (with-test (:name (make-instance :constant-non-std-class-name))
-              (let ((f (compile nil `(lambda ()
-                                       (make-instance 'structure-object)))))
+              (let ((f (checked-compile `(lambda ()
+                                           (make-instance 'structure-object)))))
                 (assert (not (find-ctor-caches f)))
                 (assert (= 4 opt))
                 (assert (typep (funcall f) 'structure-object)))))
        (setf (sb-int:info :function :source-transform 'make-instance) transform))))
 
 (with-test (:name (make-instance :ctor-inline-cache-resize))
-  (let* ((f (compile nil `(lambda (name) (make-instance name))))
+  (let* ((f (checked-compile `(lambda (name) (make-instance name))))
          (classes (loop repeat (* 2 sb-pcl::+ctor-table-max-size+)
                         collect (class-name (eval `(defclass ,(gentemp) () ())))))
          (count 0)
@@ -186,7 +186,7 @@
 
 (with-test (:name (make-instance :ctor-default-initargs-1))
   (assert (aroundp (eval `(make-instance 'some-class))))
-  (let ((fun (compile nil `(lambda () (make-instance 'some-class)))))
+  (let ((fun (checked-compile `(lambda () (make-instance 'some-class)))))
     (assert (aroundp (funcall fun)))
     ;; make sure we tested what we think we tested...
     (let ((ctors (find-anonymous-callees fun :type 'sb-pcl::ctor)))
@@ -213,7 +213,7 @@
   (assert (= 0 *some-counter*))
   (assert (aroundp (eval `(make-instance 'some-class2))))
   (assert (= 1 *some-counter*))
-  (let ((fun (compile nil `(lambda () (make-instance 'some-class2)))))
+  (let ((fun (checked-compile `(lambda () (make-instance 'some-class2)))))
     (assert (= 1 *some-counter*))
     (assert (aroundp (funcall fun)))
     (assert (= 2 *some-counter*))
@@ -228,9 +228,9 @@
   (defclass type-check-thing ()
     ((slot :type (integer 0) :initarg :slot))))
 (with-test (:name (make-instance :no-compile-note-at-runtime))
-  (let ((fun (compile nil `(lambda (x)
-                             (declare (optimize safety))
-                             (make-instance 'type-check-thing :slot x)))))
+  (let ((fun (checked-compile `(lambda (x)
+                                 (declare (optimize safety))
+                                 (make-instance 'type-check-thing :slot x)))))
     (handler-bind ((sb-ext:compiler-note #'error))
       (funcall fun 41)
       (funcall fun 13))))
@@ -239,21 +239,20 @@
 (defmethod no-applicable-method ((gf (eql #'make-instance)) &rest args)
   (cons :no-applicable-method args))
 (with-test (:name :constant-invalid-class-arg)
-  (assert (equal
-           '(:no-applicable-method "FOO" :quux 14)
-           (funcall (compile nil `(lambda (x) (make-instance "FOO" :quux x))) 14)))
-  (assert (equal
-           '(:no-applicable-method 'abc zot 1 bar 2)
-           (funcall (compile nil `(lambda (x y) (make-instance ''abc 'zot x 'bar y)))
-                    1 2))))
+  (checked-compile-and-assert ()
+      `(lambda (x) (make-instance "FOO" :quux x))
+    ((14) '(:no-applicable-method "FOO" :quux 14)))
+  (checked-compile-and-assert ()
+      `(lambda (x y) (make-instance ''abc 'zot x 'bar y))
+    ((1 2) '(:no-applicable-method 'abc zot 1 bar 2))))
+
 (with-test (:name :variable-invalid-class-arg)
-  (assert (equal
-           '(:no-applicable-method "FOO" :quux 14)
-           (funcall (compile nil `(lambda (c x) (make-instance c :quux x))) "FOO" 14)))
-  (assert (equal
-           '(:no-applicable-method 'abc zot 1 bar 2)
-           (funcall (compile nil `(lambda (c x y) (make-instance c 'zot x 'bar y)))
-                    ''abc 1 2))))
+  (checked-compile-and-assert ()
+      `(lambda (c x) (make-instance c :quux x))
+    (("FOO" 14) '(:no-applicable-method "FOO" :quux 14)))
+  (checked-compile-and-assert ()
+      `(lambda (c x y) (make-instance c 'zot x 'bar y))
+    ((''abc 1 2) '(:no-applicable-method 'abc zot 1 bar 2))))
 
 (defclass sneaky-class (standard-class)
   ())
@@ -279,10 +278,10 @@
         (pushnew name (dirty-slots instance))))))
 
 (with-test (:name (make-instance :setf-slot-value-using-class-hits-other-slots))
-  (let ((fun (compile nil `(lambda (a c)
-                             (let ((i (make-instance 'sneaky :a a)))
-                               (setf (sneaky-c i) c)
-                               i)))))
+  (let ((fun (checked-compile `(lambda (a c)
+                                 (let ((i (make-instance 'sneaky :a a)))
+                                   (setf (sneaky-c i) c)
+                                   i)))))
     (loop repeat 3
           do (let ((i (funcall fun "a" "c")))
                (assert (equal '(c b a) (dirty-slots i)))
@@ -398,4 +397,3 @@
         (destructuring-bind (f-6-e-c f-6-e-1 f-6-e-2)
             (mapcar #'make `(,constant-name-2 1 2))
           (check f-6-e-c 1 f-6-e-1 1 f-6-e-2 2))))))
-
