@@ -38,7 +38,7 @@
 (defvar *count* 100)
 
 ;;; Base-case: detecting exhaustion
-(with-test (:name (:exhaust :basic) :broken-on '(and :sunos :x86-64))
+(with-test (:name (:exhaust :basic) :broken-on (and :sunos :x86-64))
   (assert (eq :exhausted
               (handler-case
                   (recurse)
@@ -50,7 +50,7 @@
 ;;; exhaustion checking after unwinding -- and that previous test
 ;;; didn't break it.
 (with-test (:name (:exhaust :non-local-control)
-                  :broken-on '(and :sunos :x86-64)
+                  :broken-on (and :sunos :x86-64)
                   :skipped-on :win32)
   (let ((exhaust-count 0)
         (recurse-count 0))
@@ -69,7 +69,7 @@
 ;;; Check that we can safely use user-provided restarts to
 ;;; unwind.
 (with-test (:name (:exhaust :restarts)
-                  :broken-on '(and :sunos :x86-64)
+                  :broken-on (and :sunos :x86-64)
                   :skipped-on :win32)
   (let ((exhaust-count 0)
         (recurse-count 0))
@@ -100,7 +100,7 @@
       (assert ok))))
 
 (with-test (:name (:exhaust :alien-stack)
-                  :skipped-on '(or (not :c-stack-is-control-stack)))
+                  :skipped-on (or (not :c-stack-is-control-stack)))
   (let ((ok nil))
     (labels ((exhaust-alien-stack (i)
                (with-alien ((integer-array (array int 500)))
