@@ -120,24 +120,17 @@
   (setf (aref (the myarraytype array) 0)
         sb-ext:double-float-positive-infinity))
 
-;;; bug 407
-;;;
-;;; FIXME: it may be that TYPE-ERROR is wrong, and we should
-;;; instead signal an overflow or coerce into an infinity.
-(defun bug-407a ()
-  (loop for n from (expt 2 1024) upto (+ 10 (expt 2 1024))
-        do (handler-case
-               (coerce n 'single-float)
-             (simple-type-error ()
-               (return-from bug-407a :type-error)))))
-(assert (eq :type-error (bug-407a)))
-(defun bug-407b ()
-  (loop for n from (expt 2 1024) upto (+ 10 (expt 2 1024))
-        do (handler-case
-               (format t "~E~%" (coerce n 'single-float))
-             (simple-type-error ()
-               (return-from bug-407b :type-error)))))
-(assert (eq :type-error (bug-407b)))
+(with-test (:name :bug-407a)
+  (assert-error
+   (loop for n from (expt 2 1024) upto (+ 10 (expt 2 1024))
+         do (coerce n 'single-float))
+   floating-point-overflow))
+
+(with-test (:name :bug-407b)
+  (assert-error
+   (loop for n from (expt 2 1024) upto (+ 10 (expt 2 1024))
+         do (format nil "~E~%" n))
+   floating-point-overflow))
 
 ;; 1.0.29.44 introduces a ton of changes for complex floats
 ;; on x86-64. Huge test of doom to help catch weird corner
