@@ -91,9 +91,10 @@
       do (assert (equal (apply 'nconc (copy-tree args)) result))
       do (let* ((exp `(nconc ,@ (mapcar (lambda (arg)
                                           `(copy-tree ',arg))
-                                        args)))
-                (fun (checked-compile `(lambda () ,exp))))
-           (assert (equal (funcall fun) result))))))
+                                        args))))
+           (checked-compile-and-assert ()
+               `(lambda () ,exp)
+             (() result))))))
 
 (with-test (:name (nconc :improper-list type-error))
   (let ((tests '(((3 (1 . 2)) 3)
@@ -196,104 +197,85 @@
       (test (cdr x-numbers) (member -2.0 numbers :key #'- :test '=))
       (test nil (member -1.0 numbers :key #'- :test 'eql)))))
 
-;;; assoc
-(with-test (:name assoc)
-  (macrolet ((test  (expected form)
-               (let ((numbers '((1 a) (2 b)))
-                     (tricky '(nil (a . b) nil (nil . c) (c . d))))
-                 `(progn
-                    (assert (equal ',expected (let ((numbers ',numbers)
-                                                    (tricky ',tricky))
-                                                (funcall fun ,@(cdr form)))))
-                    (assert (equal ',expected (funcall (lambda ()
-                                                         (declare (optimize speed))
-                                                         (let ((numbers ',numbers)
-                                                               (tricky ',tricky))
-                                                           ,form)))))
-                    (assert (equal ',expected (funcall (lambda ()
-                                                         (declare (optimize space))
-                                                         (let ((numbers ',numbers)
-                                                               (tricky ',tricky))
-                                                           ,form)))))))))
-    (let ((fun (car (list 'assoc))))
-      (test (1 a) (assoc 1 numbers))
-      (test (2 b) (assoc 2 numbers))
-      (test (1 a) (assoc 1 numbers :key 'identity))
-      (test (2 b) (assoc 2 numbers :key #'identity))
-      (test nil (assoc 1.0 numbers))
+(flet ((test (function needle haystack args expected)
+         (checked-compile-and-assert ()
+             `(lambda ()
+                (let ((function (car (list ',function)))
+                      (list ',haystack))
+                  (funcall function ,needle list ,@args)))
+           (() expected))
+         (checked-compile-and-assert ()
+             `(lambda ()
+                (let ((list ',haystack))
+                  (,function ,needle list ,@args)))
+           (() expected))))
 
-      (test (1 a) (assoc 1.0 numbers :test #'=))
-      (test (1 a) (assoc 1.0 numbers :test #'= :key nil))
-      (test (2 b) (assoc 2.0 numbers :test '=))
-      (test nil (assoc 0 numbers :test '=))
+  (with-test (:name assoc)
+    (let ((numbers '((1 a) (2 b)))
+          (tricky '(nil (a . b) nil (nil . c) (c . d))))
+      (test 'assoc 1    numbers '()                    '(1 a))
+      (test 'assoc 2    numbers '()                    '(2 b))
+      (test 'assoc 1    numbers '(:key 'identity)      '(1 a))
+      (test 'assoc 2    numbers '(:key #'identity)     '(2 b))
+      (test 'assoc 1.0  numbers '()                    nil)
 
-      (test (1 a) (assoc 0 numbers :test-not #'>))
-      (test (2 b) (assoc 1 numbers :test-not 'eql))
-      (test nil (assoc 0 numbers :test-not '<))
+      (test 'assoc 1.0  numbers '(:test #'=)           '(1 a))
+      (test 'assoc 1.0  numbers '(:test #'= :key nil)  '(1 a))
+      (test 'assoc 2.0  numbers '(:test '=)            '(2 b))
+      (test 'assoc 0    numbers '(:test '=)            nil)
 
-      (test (1 a) (assoc -1 numbers :key #'-))
-      (test (2 b) (assoc -2 numbers :key '-))
-      (test nil (assoc -1.0 numbers :key #'-))
+      (test 'assoc 0    numbers '(:test-not #'>)       '(1 a))
+      (test 'assoc 1    numbers '(:test-not 'eql)      '(2 b))
+      (test 'assoc 0    numbers '(:test-not '<)        nil)
 
-      (test (1 a) (assoc -1.0 numbers :key #'- :test '=))
-      (test (2 b) (assoc -2.0 numbers :key #'- :test '=))
-      (test nil (assoc -1.0 numbers :key #'- :test 'eql))
+      (test 'assoc -1   numbers '(:key #'-)            '(1 a))
+      (test 'assoc -2   numbers '(:key '-)             '(2 b))
+      (test 'assoc -1.0 numbers '(:key #'-)            nil)
+
+      (test 'assoc -1.0 numbers '(:key #'- :test '=)   '(1 a))
+      (test 'assoc -2.0 numbers '(:key #'- :test '=)   '(2 b))
+      (test 'assoc -1.0 numbers '(:key #'- :test 'eql) nil)
 
       ;; Bug reported by Paul Dietz: ASSOC should ignore NIL elements
       ;; in a alist
-      (test (nil . c) (assoc nil tricky :test #'eq)))))
+      (test 'assoc nil tricky  '(:test #'eq)      '(nil . c))))
 
-;;; rassoc
-(with-test (:name rassoc)
-  (macrolet ((test  (expected form)
-               (let ((numbers '((a . 1) (b . 2)))
-                     (tricky '(nil (b . a) nil (c . nil) (d . c))))
-                 `(progn
-                    (assert (equal ',expected (let ((numbers ',numbers)
-                                                    (tricky ',tricky))
-                                                (funcall fun ,@(cdr form)))))
-                    (assert (equal ',expected (funcall (lambda ()
-                                                         (declare (optimize speed))
-                                                         (let ((numbers ',numbers)
-                                                               (tricky ',tricky))
-                                                           ,form)))))
-                    (assert (equal ',expected (funcall (lambda ()
-                                                         (declare (optimize space))
-                                                         (let ((numbers ',numbers)
-                                                               (tricky ',tricky))
-                                                           ,form)))))))))
-    (let ((fun (car (list 'rassoc))))
-      (test (a . 1) (rassoc 1 numbers))
-      (test (b . 2) (rassoc 2 numbers))
-      (test (a . 1) (rassoc 1 numbers :key 'identity))
-      (test (b . 2) (rassoc 2 numbers :key #'identity))
-      (test nil (rassoc 1.0 numbers))
+  (with-test (:name rassoc)
+    (let ((numbers '((a . 1) (b . 2)))
+          (tricky '(nil (b . a) nil (c . nil) (d . c))))
+      (test 'rassoc 1    numbers '()                    '(a . 1))
+      (test 'rassoc 2    numbers '()                    '(b . 2))
+      (test 'rassoc 1    numbers '(:key 'identity)      '(a . 1))
+      (test 'rassoc 2    numbers '(:key #'identity)     '(b . 2))
+      (test 'rassoc 1.0  numbers '()                    nil)
 
-      (test (a . 1) (rassoc 1.0 numbers :test #'=))
-      (test (a . 1) (rassoc 1.0 numbers :test #'= :key nil))
-      (test (b . 2) (rassoc 2.0 numbers :test '=))
-      (test nil (rassoc 0 numbers :test '=))
+      (test 'rassoc 1.0  numbers '(:test #'=)           '(a . 1))
+      (test 'rassoc 1.0  numbers '(:test #'= :key nil)  '(a . 1))
+      (test 'rassoc 2.0  numbers '(:test '=)            '(b . 2))
+      (test 'rassoc 0    numbers '(:test '=)            nil)
 
-      (test (a . 1) (rassoc 0 numbers :test-not #'>))
-      (test (b . 2) (rassoc 1 numbers :test-not 'eql))
-      (test nil (rassoc 0 numbers :test-not '<))
+      (test 'rassoc 0    numbers '(:test-not #'>)       '(a . 1))
+      (test 'rassoc 1    numbers '(:test-not 'eql)      '(b . 2))
+      (test 'rassoc 0    numbers '(:test-not '<)        nil)
 
-      (test (a . 1) (rassoc -1 numbers :key #'-))
-      (test (b . 2) (rassoc -2 numbers :key '-))
-      (test nil (rassoc -1.0 numbers :key #'-))
+      (test 'rassoc -1   numbers '(:key #'-)            '(a . 1))
+      (test 'rassoc -2   numbers '(:key '-)             '(b . 2))
+      (test 'rassoc -1.0 numbers '(:key #'-)            nil)
 
-      (test (a . 1) (rassoc -1.0 numbers :key #'- :test '=))
-      (test (b . 2) (rassoc -2.0 numbers :key #'- :test '=))
-      (test nil (rassoc -1.0 numbers :key #'- :test 'eql))
+      (test 'rassoc -1.0 numbers '(:key #'- :test '=)   '(a . 1))
+      (test 'rassoc -2.0 numbers '(:key #'- :test '=)   '(b . 2))
+      (test 'rassoc -1.0 numbers '(:key #'- :test 'eql) nil)
 
-      (test (c . nil) (rassoc nil tricky :test #'eq)))))
+      (test 'rassoc nil  tricky  '(:test #'eq)          '(c . nil)))))
 
 ;;;; member-if & assoc-if & rassoc-if
 (with-test (:name (member-if assoc-if rassoc-if))
   (macrolet ((test (value form)
                `(let ((* ,value))
                   (assert (eval ,form))
-                  (assert (funcall (checked-compile '(lambda () ,form)))))))
+                  (checked-compile-and-assert (:optimize :safe)
+                      '(lambda () ,form)
+                    (() t)))))
     (test 'evenp
           (equal '(2 3 4) (member-if * (list 1 2 3 4))))
     (test 'evenp
@@ -318,7 +300,9 @@
   (macrolet ((test (value form)
                `(let ((* ,value))
                   (assert (eval ,form))
-                  (assert (funcall (checked-compile '(lambda () ,form)))))))
+                  (checked-compile-and-assert ()
+                      '(lambda () ,form)
+                    (() t)))))
     (test 'oddp
           (equal '(2 3 4) (member-if-not * (list 1 2 3 4))))
     (test 'oddp
@@ -340,23 +324,30 @@
 ;;; for ASSOC & MEMBER
 (with-test (:name (assoc member *print-case*))
   (let ((*print-case* :downcase))
-    (assert (eql 2
-                 (cdr (funcall (checked-compile `(lambda (i l) (assoc i l)))
-                               :b '((:a . 1) (:b . 2))))))
-    (assert (equal '(3 4 5)
-                   (funcall (checked-compile `(lambda (i l) (member i l)))
-                            3 '(1 2 3 4 5))))))
+    (checked-compile-and-assert ()
+        `(lambda (i l) (assoc i l))
+      ((:b '((:a . 1) (:b . 2))) '(:b . 2)))
+    (checked-compile-and-assert ()
+        `(lambda (i l) (member i l))
+      ((3 '(1 2 3 4 5)) '(3 4 5)))))
 
 ;;; bad bounding index pair to SUBSEQ on a list
 (with-test (:name (subseq sb-kernel:bounding-indices-bad-error))
-  (let ((list (list 0 1 2 3 4 5)))
-    (assert-error (subseq list 4 2) sb-kernel:bounding-indices-bad-error)))
+  (multiple-value-bind (fun failure-p warnings)
+      (checked-compile `(lambda ()
+                          (let ((list (list 0 1 2 3 4 5)))
+                            (subseq list 4 2)))
+                       :allow-warnings t)
+    (assert failure-p)
+    (assert (= (length warnings) 1))
+    (assert-error (funcall fun)  sb-kernel:bounding-indices-bad-error)))
 
 ;;; ADJOIN must apply key to item as well
 (with-test (:name (adjoin :key))
-  (let ((fun (checked-compile `(lambda (x y)
-                                 (adjoin x y :key #'car :test #'string=)))))
-    (assert (equal '((:b)) (funcall fun (list 'b) (list '(:b))))))
+  (checked-compile-and-assert ()
+      `(lambda (x y)
+         (adjoin x y :key #'car :test #'string=))
+    (((list 'b) (list '(:b))) '((:b))))
 
   #+(or sb-eval sb-fasteval)
   (assert (equal '((:b))
@@ -405,3 +396,7 @@
 (with-test (:name (adjoin :folding))
   (flet ((%f () (adjoin 'x '(a b))))
     (assert (not (eq (%f) (%f))))))
+
+(with-test (:name (butlast :dotted))
+  (assert (null (butlast '(1 2 . 3) 4)))
+  (assert (null (nbutlast (list* 1 2 3) 4))))
