@@ -482,10 +482,10 @@
       (ignore-errors
         (delete-file file)))))
 
-(with-test (:name (:print-readable :character :utf-8) :skipped-on '(not :sb-unicode))
+(with-test (:name (:print-readable :character :utf-8) :skipped-on (not :sb-unicode))
   (test-readable-character (code-char #xfffe) :utf-8))
 
-(with-test (:name (:print-readable :character :iso-8859-1) :skipped-on '(not :sb-unicode))
+(with-test (:name (:print-readable :character :iso-8859-1) :skipped-on (not :sb-unicode))
   (test-readable-character (code-char #xfffe) :iso-8859-1))
 
 (with-test (:name (format :character-directive :colon))
