@@ -80,7 +80,8 @@
 
 (in-package :cl-user)
 
-(with-test (:name (profile :threads))
+(with-test (:name (profile :threads)
+                  :broken-on :win32)
   (profile "PROFILE-TEST")
   ;; This used to signal an error with threads
   (let* ((n #+sb-thread 5 #-sb-thread 1)
