@@ -26,11 +26,11 @@
 ;; for most arguments of RANDOM near the size of a random chunk and the
 ;; RANDOM-CHUNK used was always 32 bits, even under 64 bit wordsize which
 ;; yields even more disastrous distributions.
-(with-test (:name (:random :integer :set-of-integers :distribution))
+(with-test (:name (random integer :set-of-integers :distribution))
   (let* ((high (floor (expt 2 33) 3))
          (mid (floor high 2))
-         (fun (compile nil `(lambda (x)
-                              (random (if x ,high 10)))))
+         (fun (checked-compile `(lambda (x)
+                                  (random (if x ,high 10)))))
          (n1 0)
          (n 10000))
     (dotimes (i n)
@@ -48,10 +48,10 @@
     (when (< n1 (* n 5/12))
       (error "bad RANDOM distribution: expected ~d, got ~d" (/ n 2) n1))))
 
-(with-test (:name (:random :integer :set-of-integers :chunk-size))
+(with-test (:name (random integer :set-of-integers :chunk-size))
   (let* ((high (expt 2 64))
-         (fun (compile nil `(lambda (x)
-                              (random (if x ,high 10)))))
+         (fun (checked-compile `(lambda (x)
+                                  (random (if x ,high 10)))))
          (n 200)
          (x 0))
     (dotimes (i n)
@@ -63,38 +63,39 @@
 
 ;;; Some tests for basic integer RANDOM functionality.
 
-(with-test (:name (:random :integer :error-if-invalid-random-state))
-  (dolist (optimize '(((speed 0) (compilation-speed 3))
-                      ((speed 3) (compilation-speed 0) (space 0))))
-    (dolist (expr `((lambda (x state)
-                      (declare (optimize ,@optimize))
-                      (random x state))
-                    (lambda (x state)
-                      (declare (optimize ,@optimize))
-                      (declare (type integer x))
-                      (random x state))
-                    (lambda (x state)
-                      (declare (optimize ,@optimize))
-                      (declare (type (integer 100 200) x))
-                      (random x state))
-                    (lambda (x state)
-                      (declare (optimize ,@optimize))
-                      (random (if x 10 20) state))))
-      (let ((fun (compile nil expr)))
-        (assert-error (funcall fun 150 nil) type-error)))))
+(with-test (:name (random integer :error-if-invalid-random-state))
+  (map-optimize-declarations
+   (lambda (optimize)
+     (dolist (expr `((lambda (x state)
+                       (declare (optimize ,@optimize))
+                       (random x state))
+                     (lambda (x state)
+                       (declare (optimize ,@optimize))
+                       (declare (type integer x))
+                       (random x state))
+                     (lambda (x state)
+                       (declare (optimize ,@optimize))
+                       (declare (type (integer 100 200) x))
+                       (random x state))
+                     (lambda (x state)
+                       (declare (optimize ,@optimize))
+                       (random (if x 10 20) state))))
+       (let ((fun (checked-compile expr)))
+         (assert-error (funcall fun 150 nil) type-error))))
+   :speed '(0 3) :safety nil :space '(nil 0) :compilation-speed '(0 3)))
 
-(with-test (:name (:random :integer :distribution))
-  (let ((generic-random (compile nil '(lambda (x)
-                                        (random x)))))
+(with-test (:name (random integer :distribution))
+  (let ((generic-random (checked-compile '(lambda (x)
+                                            (random x)))))
     ;; Check powers of two: Every bit in the output should be sometimes
     ;; 0, sometimes 1.
     (dotimes (e 200)
       (let* ((number (expt 2 e))
              (foo (lambda ()
                     (funcall generic-random number)))
-             (bar (compile nil `(lambda ()
-                                  (declare (optimize speed))
-                                  (random ,number)))))
+             (bar (checked-compile `(lambda ()
+                                      (declare (optimize speed))
+                                      (random ,number)))))
         (flet ((test (fun)
                  (let ((x-and (funcall fun))
                        (x-ior (funcall fun)))
@@ -123,9 +124,9 @@
                                collect r))))
       (let ((foo (lambda ()
                    (funcall generic-random number)))
-            (bar (compile nil `(lambda ()
-                                 (declare (optimize speed))
-                                 (random ,number)))))
+            (bar (checked-compile `(lambda ()
+                                     (declare (optimize speed))
+                                     (random ,number)))))
         (flet ((test (fun)
                  (let* ((min (funcall fun))
                         (max min))
