@@ -19,7 +19,7 @@
 ;;; Test that symbols are properly normalized in SB-UNICODE builds
 #+sb-unicode
 (with-test (:name (:normalizing-reader)
-                  :skipped-on '(not :sb-unicode))
+                  :skipped-on (not :sb-unicode))
   (labels ((str (&rest chars)
              (coerce chars 'string))
            (symbol (&rest chars)
