@@ -448,16 +448,6 @@
             (map '(cons t (cons t null)) '+ '(1 2 3) '(10 10 10)))
    type-error))
 
-(defstruct ship size name)
-(with-test (:name (find :derive-type))
-  (let ((f (checked-compile '(lambda (x list)
-                              (ship-size (find x list :key 'ship-name))))))
-    ;; The test of SHIP-P in the SHIP-SIZE call is optimized into (NOT NULL).
-    ;; Therefore the code header for F does not reference #<LAYOUT for SHIP>
-    (assert (not (ctu:find-code-constants f :type 'sb-kernel:layout)))
-    ;; And the function is safe.
-    (assert-error (funcall f nil nil) type-error)))
-
 (with-test (:name (search :singleton-transform))
   (checked-compile-and-assert ()
     `(lambda (e) (search '(a) '(b) :end1 e))
