@@ -12,9 +12,14 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(in-package "SB!KERNEL")
+(unless (find-package "BEFORE-XC-TESTS")
+  (make-package "BEFORE-XC-TESTS" :use '("SB!XC" "SB!KERNEL" "SB!INT")))
+(do-external-symbols (s "SB!XC") ; Import all symbols from SB!XC, then use CL
+  (shadowing-import s "BEFORE-XC-TESTS"))
+(import '(sb!kernel::type-union2) "BEFORE-XC-TESTS")
+(cl:use-package '("COMMON-LISP") "BEFORE-XC-TESTS")
 
-(/show "beginning tests/type.before-xc.lisp")
+(in-package "BEFORE-XC-TESTS")
 
 (assert (type= (specifier-type '(and fixnum (satisfies foo)))
                (specifier-type '(and (satisfies foo) fixnum))))
@@ -23,21 +28,21 @@
 (assert (type= (specifier-type '(and (member 1.0 2 3) single-float))
                (specifier-type '(member 1.0))))
 
-(assert (sb-xc:typep #(1 2 3) 'simple-vector))
-(assert (sb-xc:typep #(1 2 3) 'vector))
-(assert (not (sb-xc:typep '(1 2 3) 'vector)))
-(assert (not (sb-xc:typep 1 'vector)))
+(assert (typep #(1 2 3) 'simple-vector))
+(assert (typep #(1 2 3) 'vector))
+(assert (not (typep '(1 2 3) 'vector)))
+(assert (not (typep 1 'vector)))
 
-(assert (sb-xc:typep '(1 2 3) 'list))
-(assert (sb-xc:typep '(1 2 3) 'cons))
-(assert (not (sb-xc:typep '(1 2 3) 'null)))
-(assert (not (sb-xc:typep "1 2 3" 'list)))
-(assert (not (sb-xc:typep 1 'list)))
+(assert (typep '(1 2 3) 'list))
+(assert (typep '(1 2 3) 'cons))
+(assert (not (typep '(1 2 3) 'null)))
+(assert (not (typep "1 2 3" 'list)))
+(assert (not (typep 1 'list)))
 
-(assert (sb-xc:typep nil 'null))
-(assert (sb-xc:typep nil '(member nil)))
-(assert (sb-xc:typep nil '(member 1 2 nil 3)))
-(assert (not (sb-xc:typep nil '(member 1 2 3))))
+(assert (typep nil 'null))
+(assert (typep nil '(member nil)))
+(assert (typep nil '(member 1 2 nil 3)))
+(assert (not (typep nil '(member 1 2 3))))
 
 (assert (type= *empty-type*
                (type-intersection (specifier-type 'list)
@@ -99,7 +104,6 @@
                           integer (integer 0 1)
                           character standard-char
                           (member 1 2 3)))
-  (/show type-specifier)
   (let ((ctype (specifier-type type-specifier)))
 
     (assert (type= *empty-type* (type-intersection ctype *empty-type*)))
@@ -124,22 +128,21 @@
 
     (assert (csubtypep *empty-type* ctype))
     (assert (csubtypep ctype *universal-type*))))
-(/show "finished with identities-should-be-identities block")
 
-(assert (sb-xc:subtypep 'simple-vector 'vector))
-(assert (sb-xc:subtypep 'simple-vector 'simple-array))
-(assert (sb-xc:subtypep 'vector 'array))
-(assert (not (sb-xc:subtypep 'vector 'simple-vector)))
-(assert (not (sb-xc:subtypep 'vector 'simple-array)))
+(assert (subtypep 'simple-vector 'vector))
+(assert (subtypep 'simple-vector 'simple-array))
+(assert (subtypep 'vector 'array))
+(assert (not (subtypep 'vector 'simple-vector)))
+(assert (not (subtypep 'vector 'simple-array)))
 
 (macrolet ((assert-secondnil (expr) `(assert (null (nth-value 1 ,expr)))))
-  (assert-secondnil (sb-xc:subtypep t '(satisfies foo)))
-  (assert-secondnil (sb-xc:subtypep t '(and (satisfies foo) (satisfies bar))))
-  (assert-secondnil (sb-xc:subtypep t '(or (satisfies foo) (satisfies bar))))
-  (assert-secondnil (sb-xc:subtypep '(satisfies foo) nil))
-  (assert-secondnil (sb-xc:subtypep '(and (satisfies foo) (satisfies bar))
+  (assert-secondnil (subtypep t '(satisfies foo)))
+  (assert-secondnil (subtypep t '(and (satisfies foo) (satisfies bar))))
+  (assert-secondnil (subtypep t '(or (satisfies foo) (satisfies bar))))
+  (assert-secondnil (subtypep '(satisfies foo) nil))
+  (assert-secondnil (subtypep '(and (satisfies foo) (satisfies bar))
                                     nil))
-  (assert-secondnil (sb-xc:subtypep '(or (satisfies foo) (satisfies bar))
+  (assert-secondnil (subtypep '(or (satisfies foo) (satisfies bar))
                                     nil)))
 
 ;;; tests of 2-value quantifieroids FOO/TYPE
@@ -217,129 +220,129 @@
                         (specifier-type '(or (single-float -1.0 1.0)
                                              (single-float 0.1))))))
 
-(assert (sb-xc:typep #\, 'character))
-(assert (sb-xc:typep #\@ 'character))
+(assert (typep #\, 'character))
+(assert (typep #\@ 'character))
 
 (assert (type= (type-intersection (specifier-type '(member #\a #\c #\e))
                                  (specifier-type '(member #\b #\c #\f)))
               (specifier-type '(member #\c))))
 
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'package 'instance)
+    (subtypep 'package 'instance)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'symbol 'instance)
+    (subtypep 'symbol 'instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'package 'funcallable-instance)
+    (subtypep 'package 'funcallable-instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'symbol 'funcallable-instance)
+    (subtypep 'symbol 'funcallable-instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'funcallable-instance 'function)
+    (subtypep 'funcallable-instance 'function)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'array 'instance)
+    (subtypep 'array 'instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'character 'instance)
+    (subtypep 'character 'instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'number 'instance)
+    (subtypep 'number 'instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'package '(and (or symbol package) instance))
+    (subtypep 'package '(and (or symbol package) instance))
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and (or double-float integer) instance) 'nil)
+    (subtypep '(and (or double-float integer) instance) 'nil)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and (or double-float integer) funcallable-instance) 'nil)
+    (subtypep '(and (or double-float integer) funcallable-instance) 'nil)
   (assert yes)
   (assert win))
-(multiple-value-bind (yes win) (sb-xc:subtypep 'instance 'type-specifier)
+(multiple-value-bind (yes win) (subtypep 'instance 'type-specifier)
   (assert (not yes))
   (assert (not win)))
-(multiple-value-bind (yes win) (sb-xc:subtypep 'type-specifier 'instance)
+(multiple-value-bind (yes win) (subtypep 'type-specifier 'instance)
   (assert (not yes))
   (assert win))
-(multiple-value-bind (yes win) (sb-xc:subtypep 'class 'type-specifier)
+(multiple-value-bind (yes win) (subtypep 'class 'type-specifier)
   (assert yes)
   (assert win))
-(multiple-value-bind (yes win) (sb-xc:subtypep 'classoid 'type-specifier)
+(multiple-value-bind (yes win) (subtypep 'classoid 'type-specifier)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and (function (t)) funcallable-instance) 'nil)
+    (subtypep '(and (function (t)) funcallable-instance) 'nil)
   (assert (not win))
   (assert (not yes)))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and fixnum function) 'nil)
+    (subtypep '(and fixnum function) 'nil)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and fixnum hash-table) 'nil)
+    (subtypep '(and fixnum hash-table) 'nil)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(function) '(function (t &rest t)))
+    (subtypep '(function) '(function (t &rest t)))
   (assert (not yes))
   (assert win))
 ;; Used to run out of stack.
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'null '(or unk0 unk1))
+    (subtypep 'null '(or unk0 unk1))
   (assert (not yes))
   (assert (not win)))
 
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and function instance) nil)
+    (subtypep '(and function instance) nil)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep nil '(and function instance))
+    (subtypep nil '(and function instance))
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and function funcallable-instance) 'funcallable-instance)
+    (subtypep '(and function funcallable-instance) 'funcallable-instance)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'funcallable-instance '(and function funcallable-instance))
+    (subtypep 'funcallable-instance '(and function funcallable-instance))
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'stream 'instance)
+    (subtypep 'stream 'instance)
   (assert (not win))
   (assert (not yes)))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep 'stream 'funcallable-instance)
+    (subtypep 'stream 'funcallable-instance)
   (assert (not yes))
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and stream instance) 'instance)
+    (subtypep '(and stream instance) 'instance)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and stream funcallable-instance) 'funcallable-instance)
+    (subtypep '(and stream funcallable-instance) 'funcallable-instance)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and stream instance) 'stream)
+    (subtypep '(and stream instance) 'stream)
   (assert yes)
   (assert win))
 (multiple-value-bind (yes win)
-    (sb-xc:subtypep '(and stream funcallable-instance) 'stream)
+    (subtypep '(and stream funcallable-instance) 'stream)
   (assert yes)
   (assert win))
 
@@ -361,4 +364,3 @@
 ;; but it should also be EQ to (MEMBER NIL T)
 (assert (eq (specifier-type '(member nil t)) (specifier-type 'boolean)))
 
-(/show "done with tests/type.before-xc.lisp")
